Package: FastqCleaner
Type: Package
Title: A Shiny Application for Quality Control, Filtering and Trimming
        of FASTQ Files
Version: 1.29.0
Date: 2022-05-01
Authors@R: c(person(given = "Leandro", family = "Roser", role = c("aut", "cre"), 
             email = "learoser@gmail.com"), 
             person(given = "Fernán", family = "Agüero", role = "aut"),
             person(given = "Daniel", family = "Sánchez", role = "aut"))
Description: An interactive web application for quality control,  filtering and trimming of FASTQ files. This user-friendly tool combines a pipeline for data processing  based on Biostrings and ShortRead infrastructure, with a cutting-edge visual environment.
             Single-Read and Paired-End files can be locally processed. Diagnostic interactive plots (CG content, per-base sequence quality, etc.) are provided for both the input and output files.
License: MIT + file LICENSE
Imports: methods, shiny, stats, IRanges, Biostrings, ShortRead, DT,
        S4Vectors, graphics, htmltools, shinyBS, Rcpp (>= 0.12.12)
Suggests: BiocStyle, testthat, knitr, rmarkdown
LinkingTo: Rcpp
Collate: 'roxygen.auxiliar.R' 'auxiliar.R' 'matching.R'
        'server_functions.R' 'n_filter.R' 'seq_filter.R'
        'complex_filter.R' 'adapter_filter.R' 'launch_fqc.R'
        'length_filter.R' 'fixed_filter.R' 'trim3q_filter.R'
        'unique_filter.R' 'plotObjects.R' 'qmean_filter.R' 'simulate.R'
        'RcppExports.R'
biocViews:
        QualityControl,Sequencing,Software,SangerSeq,SequenceMatching
VignetteBuilder: knitr
Packaged: 2025-10-31 21:47:31 UTC; biocbuild
Encoding: UTF-8
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/FastqCleaner
git_branch: devel
git_last_commit: 464323c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Author: Leandro Roser [aut, cre],
  Fernán Agüero [aut],
  Daniel Sánchez [aut]
Maintainer: Leandro Roser <learoser@gmail.com>

# Generated by roxygen2: do not edit by hand

export(adapter_filter)
export(check_encoding)
export(complex_filter)
export(fixed_filter)
export(inject_letter_random)
export(launch_fqc)
export(length_filter)
export(n_filter)
export(qmean_filter)
export(random_length)
export(random_qual)
export(random_seq)
export(seq_filter)
export(seq_names)
export(trim3q_filter)
export(unique_filter)
import(Rcpp)
import(htmltools)
importFrom(Biostrings,BStringSet)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,PhredQuality)
importFrom(Biostrings,dinucleotideFrequency)
importFrom(Biostrings,oligonucleotideFrequency)
importFrom(Biostrings,quality)
importFrom(Biostrings,replaceLetterAt)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,trimLRPatterns)
importFrom(DT,dataTableOutput)
importFrom(DT,renderDataTable)
importFrom(IRanges,end)
importFrom(IRanges,narrow)
importFrom(IRanges,start)
importFrom(IRanges,width)
importFrom(S4Vectors,isConstant)
importFrom(ShortRead,FastqSampler)
importFrom(ShortRead,ShortReadQ)
importFrom(ShortRead,id)
importFrom(ShortRead,nFilter)
importFrom(ShortRead,sread)
importFrom(ShortRead,yield)
importFrom(ShortRead,occurrenceFilter)
importFrom(graphics,hist)
importFrom(methods,as)
importFrom(methods,is)
importFrom(shinyBS,bsAlert)
importFrom(shinyBS,bsModal)
importFrom(shinyBS,bsPopover)
importFrom(stats,quantile)
useDynLib(FastqCleaner)

<span><h1>  FastqCleaner [![Build Status](https://travis-ci.org/leandroroser/FastqCleaner.svg?branch=master)](https://travis-ci.org/leandroroser/FastqCleaner) [![AppVeyor Build Status](https://ci.appveyor.com/api/projects/status/github/leandroroser/FastqCleaner?branch=master&svg=true)](https://ci.appveyor.com/project/leandroroser/FastqCleaner) [![R](https://img.shields.io/badge/R%3E%3D-3.0-red.svg)]() [![CRAN](https://img.shields.io/cran/l/devtools.svg)]()  </h1> </span>

A Shiny web-app to clean FASTQ files with R and Bioconductor



## This is a development repository
### Please install the application using the canonical way using Bioconductor

```python
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("FastqCleaner")
```

## Note: due to a change in Shiny, the Bioconductor version is working with Shiny <= 1.6. For the development version, please install using the following code:

```python
install.packages("devtools")
devtools::install_github("https://github.com/leandroroser/FastqCleaner")
```

- The development version will be pushed to Bioconductor after proper checks (note from the author, 2022-07-01)

#'  Remove a fixed number of bases of a ShortReadQ object from 3' or 5'
#' @param input  \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @param trim3 Number of bases to remove from 3'
#' @param trim5 Number of bases to remove from 5'
#' @description The program removes a given number of bases from the 3' or 5'
#' regions of the sequences contained in a ShortReadQ object
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
#' object
#' @examples
#' require('Biostrings')
#' require('ShortRead')
#' 
#' # create 6 sequences of width 20
#'  
#' set.seed(10)
#' input <- random_seq(6, 20)
#' 
#' # create qualities of width 20 
#' 
#' set.seed(10)
#' input_q <- random_qual(c(30,40), slength = 6, swidth = 20, 
#' encod = 'Sanger')
#' 
#' 
#' # create names
#' input_names <- seq_names(6)
#' 
#' # create ShortReadQ object
#' my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)
#' 
#' # apply the filter 
#' filtered3 <- fixed_filter(my_read, trim5 = 5)
#' 
#' filtered5 <- fixed_filter(my_read, trim3 = 5)
#' 
#' filtered3and5 <- fixed_filter(my_read, trim3 = 10, trim5 = 5)
#' 
#' # look at the trimmed sequences
#' sread(filtered3)
#' sread(filtered5)
#' sread(filtered3and5)
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export
#'
#'


fixed_filter <- function(input, trim3 = NA, trim5 = NA) {

if (!is.na(trim3)) {
    input <- input[!(width(input) <= trim3)]
    input <- narrow(input, start = trim3 + 1)
}

if (!is.na(trim5)) {
    input <- input[!(width(input) <= trim5)]
    input <- narrow(input, end = width(input) - trim5)
}

input
}

#' Remove left and right full and partial patterns
#' @param subject \code{\link[Biostrings:DNAString-class]{DNAString}} or 
#' \code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
#' @param Lpattern 5' pattern,
#'  \code{\link[Biostrings:DNAString-class]{DNAString}} object
#' @param Rpattern 3' pattern,
#'  \code{\link[Biostrings:DNAString-class]{DNAString}} object
#' @param error_rate Error rate (value in [0, 1]). 
#' The error rate is the proportion of mismatches allowed between
#' the adapter and the aligned portion of the subject.
#' For a given adapter A, the number of allowed mismatches between each 
#' subsequence s of A and the subject is computed as: error_rate * L_s,
#' where L_s is the length of the subsequence s.
#' @param with.indels Allow indels? 
#' @param anchored Can the adapter or partial adapter be within 
#' the sequence? (anchored = FALSE)
#' or only in the terminal regions of the sequence? (anchored = TRUE).
#' Default TRUE (trim only flanking regions)
#' @param fixed Parameter passed to 
#' \code{\link[Biostrings]{trimLRPatterns}} 
#' Default 'subject',  ambiguities in the pattern only are interpreted 
#' as wildcard. See the argument fixed in 
#' \code{\link[Biostrings]{trimLRPatterns}} 
#' @param ranges Return ranges? Default FALSE
#' @param checks Perform internal checks? Default TRUE
#' @param min_match_flank Do not trim in flanks of the subject,
#'  if a match has min_match_flank of less length. Default 1L 
#'  (only trim with >=2 coincidences in a flank match)
#' @param ... additional parameters passed to
#' \code{\link[Biostrings]{trimLRPatterns}} 
#' @return  Edited \code{\link[Biostrings:DNAString-class]{DNAString}} or 
#' \code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
#' @description This set of programs are internal, 
#' and the function adapter_filter is recommended for trimming. 
#' The programs can remove adapters and partial 
#' adapters from 3' and 5'. The adapters can be anchored or not. 
#' When indels are allowed, the error rate consists in the edit distance. 
#' IUPAC simbols are allowed. The methods use the 
#' \code{\link[Biostrings]{trimLRPatterns}} function
#'  of the \pkg{Biostrings} package, with some additions 
#'  to take into account e.g., partial adaptors.
#'  IUPAC symbols are allowed in all the cases. The present function 
#' also removes partial adapters, without the need of additional steps
#'  (for example, creating a padded adapter with 'Ns', etc). 
#' A similar result to the output of \code{\link[Biostrings]{trimLRPatterns}} 
#' can be obtained with the option anchored = TRUE.
#' When several matches are found, the function removes the subsequence 
#' that starts in the first match when cutRseq is used, or ends 
#' in the last match when cutLseq is used.
#' 
#' @examples
#' library(Biostrings)
#' 
#' subject <- DNAStringSet(c('ATCATGCCATCATGAT',
#' 'CATGATATTA', 'TCATG', 'AAAAAA', 'AGGTCATG'))
#' 
#' Lpattern <- Rpattern <- 'TCATG'
#' 
#' FastqCleaner:::cutLseq(subject, Lpattern)
#' FastqCleaner:::cutLseq(subject, Lpattern, ranges = TRUE)
#' FastqCleaner:::cutRseq(subject, Rpattern)
#' 
#' 
#' FastqCleaner:::cutLseq(subject, Lpattern, anchored = FALSE)
#' FastqCleaner:::cutLseq(subject, Lpattern,  error_rate = 0.2)
#' FastqCleaner:::cutLseq(subject, Lpattern,  error_rate = 0.2, 
#' with.indels = TRUE)
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @rdname matching
#' @keywords internal


cutRseq <- function(subject, Rpattern, 
                    with.indels = FALSE,
                    fixed = "subject", 
                    error_rate = 0.2, 
                    anchored = TRUE, 
                    ranges = FALSE,
                    checks = TRUE, 
                    min_match_flank = 2L, 
                    ...) {
    
   Rpattern <- DNAString(Rpattern)
  
   if (error_rate > 1 || error_rate < 0) {
    stop("error_rate must be a number between 0 and 1")
   }

   if (checks) {
    
       if (!is(Rpattern, "DNAString")) {
           stop("Rpattern must be a character string or a DNAString object")
       }
 
       csub <- class(subject)
       if (csub != "DNAStringSet") {
           stop("subject must be a DNAString or DNAStringSet object")
       }
    
       if (csub == "DNAString") {
           subject <- as(subject[[1]], "DNAStringSet")
       }
    
   }
  
   p <- length(Rpattern)
   s_width <- width(subject)
   s <- max(width(subject))
  
   if(error_rate > 0) {
       flank_seq <- as.integer(seq_len(p) * error_rate)
   } else {
       flank_seq <- rep(0, length(seq_len(p)))
   }
  
  
   if (min_match_flank >= 1L) {
       if (p > min_match_flank) {
           flank_seq[seq_len(min_match_flank)] <- -1
       } else {
           return(subject)
       }
   }
 
   if(!anchored) {
       Rpattern <- as.character(Rpattern)
       maxlen <-  max(width(subject)) - nchar(Rpattern)
       if(maxlen > 0) {
           Rpattern <- paste0(Rpattern,  paste(rep("N",maxlen), collapse = ""))
       }
       Rpattern <- DNAString(Rpattern)
       flank_seq <- c(flank_seq, rep(0,maxlen))
   }
  
   out <- trimLRPatterns(Rpattern = Rpattern,
                       subject = subject, 
                       max.Rmismatch = flank_seq, 
                       with.Rindels = with.indels, 
                       Rfixed = fixed, 
                       ...)
  
  
   if (ranges) {
       out <- IRanges::IRanges(start = rep(1, length(out)), end = width(out))
   } 
  
   out
}

#' Remove left and right full and partial patterns
#' @rdname matching

cutLseq <- function(subject, Lpattern,
                    with.indels = FALSE, 
                    fixed = "subject", 
                    error_rate = 0.2, 
                    anchored = TRUE, 
                    ranges = FALSE, 
                    min_match_flank = 3L, 
                    checks = TRUE, ...) {


   Lpattern <- DNAString(Lpattern)

   if (checks) {
       if (!is(Lpattern, "DNAString")) {
           stop("Rpattern must be a character string or a DNAString object")
       }

       csub <- class(subject)
       if (csub != "DNAStringSet") {
           stop("subject must be a DNAStringSet object")
       }

       if (csub == "DNAString") {
           subject <- as(subject[[1]], "DNAStringSet")
       }
   }

   p <- length(Lpattern)
   s_width <- width(subject)
   s <- max(width(subject))


   if(error_rate > 0) {
       flank_seq <- as.integer(seq_len(p) * error_rate)
   } else {
       flank_seq <- rep(0, length(seq_len(p)))
   }


   if (min_match_flank >= 1L) {
       if (p > min_match_flank) {
           flank_seq[seq_len(min_match_flank)] <- -1
       } else {
           return(subject)
   }
   }


   if(!anchored) {
       Lpattern <- as.character(Lpattern)
       maxlen <-  max(width(subject)) - nchar(Lpattern)
       if(maxlen > 0) {
           Lpattern <- paste0(paste(rep("N",maxlen), collapse = ""), Lpattern)
       }
       Lpattern <- DNAString(Lpattern)
       flank_seq <- c(flank_seq, rep(0,maxlen))
  }


   out <- trimLRPatterns(Lpattern = Lpattern,
                       subject = subject, 
                       max.Lmismatch = flank_seq, 
                       with.Lindels = with.indels, 
                       Lfixed = fixed, 
                       ...)


   if (ranges) {
       out <- IRanges::IRanges(start = rep(1, length(out)), end = width(out))
   } 

   out
}

context("test-matching.R")
skip_on_cran()

library(Biostrings)
Lpattern <- Rpattern <- "TCATG"
subject <- DNAStringSet(c("ATCATGCCATCATGAT", "ATTTGGAATCAT",
"CATGATATTA", "TCATG", "AAAAAA", "AGGTCATG"))

test_that("cutLseq works", {

  # cutLseq
  out <- cutLseq(subject, Lpattern, anchored = FALSE, ranges = FALSE,  min_match_flank = 1)
  expect_equal(width(out), c(2L, 12L, 6L, 0L, 6L, 0L))

  out <- cutLseq(subject, Lpattern, anchored = TRUE, ranges = FALSE, min_match_flank = 1)
  expect_equal(width(out), c(16L, 12L, 6L, 0L, 6L, 8L))
  
  
})


test_that("cutRseq works", {
  
  out <- FastqCleaner:::cutRseq(subject, Rpattern, anchored = FALSE, ranges = FALSE)
  expect_equal(width(out),  c(1L, 8L, 10L, 0L, 6L, 3L))
  
  out <- FastqCleaner:::cutRseq(subject, Rpattern, anchored = TRUE, ranges = FALSE)
  expect_equal(width(out),  c(16L, 8L, 10L, 0L, 6L, 3L))
  
})

#' Filter sequences by their average quality
#' @param input  \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @param minq Quality threshold
#' @param q_format Quality format used for the file, as returned 
#' by check.encoding
#' @param check.encod Check the encoding of the sequence? This argument 
#' is incompatible with q_format
#' @description The program removes the sequences with a quality 
#' lower the 'minq' threshold 
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
#' object
#' @examples 
#' 
#' require(ShortRead)
#' 
#' set.seed(10)
#' # create 30 sequences of width 20
#' input <- random_seq(30, 20)
#' 
#' # create qualities of width 20 
#' ## high quality (15 sequences)
#' set.seed(10)
#' my_qual <- random_qual(c(30,40), slength = 15, swidth = 20,
#'                        encod = 'Sanger')
#' ## low quality (15 sequences)
#' set.seed(10)
#' my_qual_2 <-   random_qual(c(5,30), slength = 15, swidth = 20,
#'                            encod = 'Sanger')
#' 
#' # concatenate vectors
#' input_q<- c(my_qual, my_qual_2)
#' 
#' # create names
#' input_names <- seq_names(30)
#' 
#' # create ShortReadQ object
#' my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)
#' 
#' # watch the average qualities
#' alphabetScore(my_read) / width(my_read)
#'
#' # apply the filter
#' filtered <- qmean_filter(my_read, minq = 30)
#'
#' # watch the average qualities
#' alphabetScore(my_read) / width(my_read)
#'
#'
#' # watch the filtered sequences
#' sread(filtered)
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export


qmean_filter <- function(input, minq, q_format = NULL, check.encod = TRUE) {

if (is.null(q_format) && !check.encod) {
    stop("Additional argument must be supplied:
    q_format must be not null or check.encod must be TRUE")
}

if (!is.null(q_format) && check.encod) {
    check.encod <- FALSE
}

input <- create_uniform_width(input, type = "fastq")

is_edited <- attributes(input)$editedWidth

qual <- quality(quality(input))


myqual_mat <- matrix(utf8ToInt(as.character(unlist(qual))), 
                    nrow = length(qual), 
                    byrow = TRUE)

if (is_edited) {
    zero_q <- which(myqual_mat == 125)
}

if (!is.null(q_format)) {
    my_encoding <- q_format
    myqual_mat <- myqual_mat - my_encoding$q
}


if (is_edited) {
    myqual_mat[zero_q] <- NA
}

if (check.encod) {
    my_encoding <- check_encoding(myqual_mat)
    q_value <- my_encoding$q
if (my_encoding$y %in% c(1, 2)) {
    myqual_mat <- myqual_mat - 33
} else {
    myqual_mat <- myqual_mat - 64
}
}


if (my_encoding$y %in% c(1, 2, 3)) {
    myqual_mat[is.na(myqual_mat)] <- 0
} else if (my_encoding$y == 4) {
    myqual_mat[is.na(myqual_mat)] <- 3 
} else if (my_encoding$y == 5) {
    myqual_mat[is.na(myqual_mat)] <- -5 
}


input[rowMeans(myqual_mat, na.rm = TRUE) >= minq]
}

#' Remove duplicated sequences in a FASTQ file
#' @param input \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @description This program is a wrapper to  
#' \code{\link[ShortRead:srFilter]{occurrenceFilter}}. 
#' It removes the duplicated sequences of a FASTQ file.
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @examples 
#' 
#' require('Biostrings')
#' require('ShortRead')
#' 
#' set.seed(10)
#' s <- random_seq(10, 10)
#' s <- sample(s, 30, replace = TRUE)
#' q <- random_qual(30, 10)
#' n <- seq_names(30)
#'
#' my_read <- ShortReadQ(sread = s, quality = q, id = n)
#' 
#' # check presence of duplicates
#' isUnique(as.character(sread(my_read)))
#' 
#' # apply the filter
#'filtered <- unique_filter(my_read)
#'
#' isUnique(as.character(sread(filtered)))
#' @export

unique_filter <- function(input) {
    uniquef <- occurrenceFilter(withSread = TRUE)
    input[uniquef(input)] 
}

panel2Input <-  function(id) {
 ns <- NS(id)

tabPanel("About", sidebarLayout(
 sidebarPanel(
  width = 3,
  class = "sideBar",
  h3("FastqCleaner"),
  p("A program to clean FASTQ files"),
  br(),
  hr(),
  div()
 ),
 
 mainPanel(
  br(),
  br(),
  tags$a(
   href = "https://github.com/leandroroser",
   target = "_blank",
   fluidRow(
    id = ns("leandroUp"),
    column(3, id = ns("about"),
           div(class = "textname", "Follow the project in GitHub!")),
    tags$img(id = ns("github"), src = "images/github.svg")
   )
  ),
  
  fluidRow(class = "leandroDown",
           br(),
           column(
            3,
            id = ns("myMail"),
            div("Feedback to:", style = "color:blue"),
            tags$a("learoser@gmail.com", href =
                    "mailto:learoser@gmail.com",
                   style = "color:black;font-weight:bold;")
           ))
 ) # end mainPanel
) # end sidebarLayout
) # end tabPanel
}

#' Launch FastqCleaner application
#' @param launch.browser Launch in browser? Default TRUE
#' @param ... Additional parameters passed to \code{\link[shiny]{runApp}}
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @examples 
#' # Uncomment and paste in te console to launch the application:
#' # launch_fqc() 
#' 
#' NULL
#' @return Launch the application, without return value
#' @export


launch_fqc <- function(launch.browser = TRUE, ...) {
    fpath <- system.file("application", package = "FastqCleaner")
    shiny::runApp(fpath, launch.browser = launch.browser, ...)
}

#' check onclick
#' @description Function to put a tickmark on click
#' @return Change value of reactive output, without return 
#' @keywords internal

check_onclick_ <- function(.menu_react, .butt_number, my_envir) {

output <- NULL

check_onclick <- function(menu_react, butt_number) {
element <- paste0("check", butt_number)
output[[element]] <- shiny::renderPrint({
    if (menu_react() == TRUE) {
        shiny::HTML(paste0("<script>
        \t\t\t\t\t   $(\"#button", 
        butt_number, " span\").addClass(\"checked\");
        \t\t\t\t\t   </script>"))
    } else {
        shiny::HTML(paste0("<script>
        \t\t\t\t\t   if($(\"#button", 
        butt_number, " span\").hasClass(\"checked\")) {
        \t\t\t\t\t   $(\"#button", 
        butt_number, " span\").removeClass(\"checked\");}
        \t\t\t\t\t   
        \t\t\t\t\t   </script>"))
    }
})
}
environment(check_onclick) <- my_envir
check_onclick(.menu_react, .butt_number)
}

#' outputClean_
#' @return Vector with chunks length and width  information 
#' @keywords internal

outputClean_ <- function(.myFile, .lengthWidthVec, my_envir) {

outputClean <- function(myFile, lengthWidthVec) {
file_length <- length(myFile)
file_width <- Biostrings::width(ShortRead::sread(myFile))
lengthWidthVec[[1]] <- sum(c(lengthWidthVec[[1]], file_length))
lengthWidthVec[[2]] <- unique(c(lengthWidthVec[[2]], file_width))
if (length(lengthWidthVec[[2]]) > 1) {
    lengthWidthVec[[2]] <- c(min(lengthWidthVec[[2]]), 
    max(lengthWidthVec[[2]]))
}
lengthWidthVec
}
environment(outputClean) <- my_envir
outputClean(.myFile, .lengthWidthVec)
}

#' messageFun_
#' @return Changes the state of reactive vector, without return 
#' @keywords internal


messageFun_ <- function(.who, .chunck, .which_read, my_envir) {
messageFun <- function(who, chunck, which_read, envir) {

readsWidth[[who]][[which_read]] <- outputClean_(chunck, 
readsWidth[[who]][[which_read]], envir)
chunck_length <- readsWidth[[who]][[which_read]][[1]]
chunck_width <- readsWidth[[who]][[which_read]][[2]]
if (length(chunck_width) > 1) {
    chunck_width <- paste0(min(chunck_width), "-", 
    max(chunck_width), " cicles")
} else {
    chunck_width <- paste0(readsWidth[[who]][[which_read]][[2]], 
    " cicles")
}
messages[[who]][which_read] <- paste0("Processed: ", chunck_length, 
                                        " reads. ", 
                                        chunck_width)
}
environment(messageFun) <- my_envir
messageFun(.who, .chunck, .which_read, my_envir)
}


#' create_cleanfunction_
#' @description Create a function to process FASTQ files in function
#' of the Shiny parameters selected by the user
#' @return Function with selected cleaning operations
#' @keywords internal

create_cleanfunction_ <- function(my_envir, 
.which_read = c("FORWARD", "REVERSE")) {
.which_read <- match.arg(.which_read)
force(my_envir)  

out <- function() {}

NCleanData <- lowComplexData <- SeqInput <- function() {}
FilterbymeanQ <- TrimCleanData <- FixedCleanData <- function() {}
SizeCleanData <- FilterbymeanQ <- TrimCleanData <- function() {}
FixedCleanData <- SizeCleanData <- DuplicCleanData <- function() {}

fun_body <- function(which_read) {

if (NCleanData()) {
    e1 <- expression(fileToProcess <- n_filter(fileToProcess,
    as.numeric(input$rm.N)), 
    messageFun_("nFilterCatch", 
    fileToProcess, read, my_envir))
} else {
    e1 <- expression()
}

if (lowComplexData()) {
    e2 <- expression(fileToProcess <- complex_filter(fileToProcess,
    input$complexThres), 
    messageFun_("lowComplexFilterCatch", 
    fileToProcess, read, my_envir))
} else {
    e2 <- expression()
}

if (SeqInput()) {
    if (which_read == "FORWARD") {
        e3 <- expression(fileToProcess <- adapter_filter(fileToProcess,
            Lpattern = input$LpatternF, 
            Rpattern = input$RpatternF,
            rc.L = input$reverseLF, 
            rc.R = input$reverseRF, 
            first = input$first_forward,
            with_indels = input$indels, 
            error_rate = input$errate, 
            anchored = input$anchored, 
            min_match_flank = input$min_match_flank, 
            checks = FALSE), 
        messageFun_("seqFilterCatch", fileToProcess, 
        .which_read = 1,  my_envir))
} else {
        e3 <- expression(fileToProcess <- adapter_filter(fileToProcess, 
            Lpattern = input$LpatternR, 
            Rpattern = input$RpatternR, 
            rc.L = input$reverseLR, 
            rc.R = input$reverseRR,
            first = input$first_reverse, 
            with_indels = input$indels, 
            error_rate = input$errate,
            anchored = input$anchored,
            min_match_flank = input$min_match_flank, 
            checks = FALSE), 
        messageFun_("seqFilterCatch", fileToProcess, 
        .which_read = 2,  my_envir))
}  # end 'REVERSE'

} else {
    e3 <- expression()
}

if (FilterbymeanQ()) {
    e4 <- expression(fileToProcess <- qmean_filter(fileToProcess,
        input$thresmeanQ, 
        q_format = my_encoding[["value"]]), 
    messageFun_("meanQFilterCatch", fileToProcess, read, my_envir))
} else {
e4 <- expression()
}

if (TrimCleanData()) {
    e5 <- expression(fileToProcess <- trim3q_filter(fileToProcess, 
        input$thresQual, 
        q_format = my_encoding[["value"]]), 
        messageFun_("trim3FilterCatch", fileToProcess, read, my_envir))
} else {
e5 <- expression()
}

if (FixedCleanData()) {
    e6 <- expression(fileToProcess <- fixed_filter(fileToProcess, input$rm3, 
    input$rm5), 
    messageFun_("fixedFilterCatch", fileToProcess, read, my_envir))
} else {
    e6 <- expression()
}

if (SizeCleanData()) {
    e7 <- expression(fileToProcess <- length_filter(fileToProcess, input$rmMin, 
    input$rmMax), 
    messageFun_("lengthFilterCatch", fileToProcess, read,  my_envir))
} else {
e7 <- expression()
}

if (DuplicCleanData()) {
    e8 <- expression(fileToProcess <- unique_filter(fileToProcess),
    messageFun_("duplicFilterCatch", fileToProcess, read, my_envir))
} else {
    e8 <- expression()
}
    e9 <- expression(fileToProcess)
    as.call(c(as.name("{"), e1, e2, e3, e4, e5, e6, e7, e8, e9))
}

environment(fun_body) <- my_envir
body(out) <- fun_body(which_read = .which_read)
formals(out) <- alist(fileToProcess = , read = , my_envir = )
environment(out) <- my_envir

out
}

#' processingFunction_
#' @description This function is the core of the application. It is used 
#' for the program to process the FASTQ file/s in the environment 
#' of the Shiny app. Note that this program makes a call to
#' create_cleanfunction
#' @return Processes the input FASTQ file, without return
#' @keywords internal


processingFunction_ <- function(my_envir) {

input <- this_envir <- filepath <- session <- NULL

create_processing_function <- function() {

if (input$fileTypeOut == "fastq") {
    compress <- FALSE
} else {
    compress <- TRUE
}


i <- 0


if (input$fileTypeIn == "SR") {
    .cleanfunction <- create_cleanfunction_(this_envir, 
    .which_read = "FORWARD")
    no_output <- as.character(body(.cleanfunction))[2] == "fileToProcess"

} else {
    .cleanfunctionF <- create_cleanfunction_(this_envir, 
    .which_read = "FORWARD")
    .cleanfunctionR <- create_cleanfunction_(this_envir,
    .which_read = "REVERSE")
    no_output <- as.character(body(.cleanfunctionF))[2] == "fileToProcess"


}


if (input$fileTypeIn == "SR" && no_output) {
    outname <- sprintf("%s_out", filepath$x)
if (file.exists(outname)) {
    file.remove(outname)
}
    file.symlink(filepath$x, outname)
}

if (input$fileTypeIn == "PE" && no_output) {

    outname1 <- sprintf("%s_out", (filepath$x)[1])
    outname2 <- sprintf("%s_out", (filepath$x)[2])

if (file.exists(outname1)) {
    file.remove(outname1)
}

if (file.exists(outname2)) {
    file.remove(outname2)
}

file.symlink((filepath$x)[1], outname1)
file.symlink((filepath$x)[2], outname2)
}


if (input$fileTypeIn == "SR" && !no_output) {


if (file.exists(sprintf("%s_out", filepath$x))) {
    file.remove(sprintf("%s_out", filepath$x))
    file.create(sprintf("%s_out", filepath$x))
}

cat("Counting lines in file ...\n")
lfile <- ShortRead::countLines(filepath$x)/4
cat(lfile, "reads found\n\n")
largo <- 0L
ancho <- c(-1L, -1L)


progress <- shiny::Progress$new(session, min = 0, max = lfile)
progress$set(message = "Processing the file",
detail = "This may take a while...")

stream <- ShortRead::FastqStreamer(filepath$x, n = input$nfile)


while (TRUE) {
    passfile <- try(ShortRead::yield(stream), silent = TRUE)
    if(length(passfile) == 0) {

    if(length(unique(ancho)) > 1) {
        cycle_info <- paste0(ancho[1], "-", ancho[2], " cycles")
    } else {
        cycle_info <- paste0(unique(ancho), " cycles")
    }

messages[["outResult"]] <- paste0(largo, " reads. ", cycle_info, 
"\n",
sprintf("%s_out", (filepath$x)))

break

} else {
progress$set(value = i)
cat(i, " reads processed", "\n")
i <- i + length(passfile)

outFile <- .cleanfunction(passfile, read = 1, this_envir)
ShortRead::writeFastq(outFile, sprintf("%s_out", 
filepath$x), mode = "a", 
compress = compress)

largo <- largo + length(outFile)
ancho <- unique(range(c(ancho[ancho != -1], Biostrings::width(outFile))))
cat(ancho, "\n")

}
}

close(stream)
progress$close()
}

if (input$fileTypeIn == "PE" && !no_output) {

    cat("Counting lines in file ...\n")
    lfile <- (ShortRead::countLines((filepath$x)[1]))/4
    cat(lfile, "reads found\n\n")

    largoL <- largoR <- 0L
    anchoL <- anchoR <- c(-1L, -1L)

    for (j in filepath$x) {
        if (file.exists(sprintf("%s_out", j))) {
        file.remove(sprintf("%s_out", j))
    }
        file.create(sprintf("%s_out", j))
    }

    progress <- shiny::Progress$new(session, min = 0, max = lfile)
    progress$set(message = "Processing the file",
    detail = "This may take a while...")

    streamL <- ShortRead::FastqStreamer((filepath$x)[1], n = input$nfile)
    streamR <- ShortRead::FastqStreamer((filepath$x)[2], n = input$nfile)

    while (TRUE) {

        passfileL <- try(ShortRead::yield(streamL), silent = TRUE)
        passfileR <- try(ShortRead::yield(streamR), silent = TRUE)

        if(length(passfileL) == 0 && length(passfileR) == 0) {

            if(length(unique(anchoL)) > 1) {
                cycle_info_L <- paste0(anchoL[1], "-", anchoL[2], " cycles")
            } else {
                cycle_info_L <- paste0(unique(anchoL), " cycles")
            }

            messages[["outResult"]][1] <- paste0(largoL, " reads. ", 
                                            cycle_info_L, "\n",
            sprintf("%s_out", (filepath$x)[1]))

            if (length(unique(anchoR)) > 1) {
                cycle_info_R <- paste0(anchoR[1], "-", anchoR[2], " cycles")
            } else {
                cycle_info_R <- paste0(unique(anchoR), " cycles")
            }

            messages[["outResult"]][2] <- paste0(largoR, " reads. ", 
                                                cycle_info_R, "\n", 
            sprintf("%s_out", (filepath$x)[2]))

            break

        } else {
        progress$set(value = i)
        cat(i, " reads processed", "\n")
        i <- i + length(passfileL)

        outFileL <- .cleanfunctionF(passfileL, read = 1, this_envir)
        outFileR <- .cleanfunctionR(passfileR, read = 2, this_envir)

        inL <- ShortRead::id(passfileL)
        outL <- ShortRead::id(outFileL)
        indexL <- as.character(inL) %in% as.character(outL)
        names(indexL) <- seq(along = indexL)
        indexL <- names(indexL[indexL])

        inR <- ShortRead::id(passfileR)
        outR <- ShortRead::id(outFileR)
        indexL <- as.character(inL) %in% as.character(outL)
        names(indexR) <- seq(along = indexR)
        indexR <- names(indexR[indexR])

        indexpairL <- indexL %in% indexR
        indexpairR <- indexR %in% indexL

        outFileL <- outFileL[indexpairL]
        outFileR <- outFileR[indexpairR]

        ShortRead::writeFastq(outFileL, sprintf("%s_out", (filepath$x)[1]), 
        mode = "a", compress = compress)

        ShortRead::writeFastq(outFileR, sprintf("%s_out", (filepath$x)[2]), 
        mode = "a", compress = compress)


        largoL <- largoL + length(outFileL)
        largoR <- largoR + length(outFileR)

        anchoL <- unique(range(c(anchoL[anchoL != -1], 
                Biostrings::width(outFileL))))
        anchoR <- unique(range(c(anchoR[anchoR != -1], 
                Biostrings::width(outFileR))))

    }
    }

        close(streamL)
        close(streamR)
        progress$close()

        }
    }

environment(create_processing_function) <- my_envir
create_processing_function()
}

#' Filter sequences with low quality in 3' tails
#' @param input \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @param rm.3qual Quality threshold for 3' tails
#' @param q_format Quality format used for the file, 
#' as returned by check_encoding
#' @param check.encod Check the encoding of the sequence? This argument 
#' is incompatible with q_format. Default TRUE
#' @param remove_zero Remove zero-length sequences?
#' @description The program removes from the 3' tails of the sequences 
#' a set of nucleotides showing a quality < a threshold value in a 
#' ShortReadQ object
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
#' object
#' @examples 
#'
#' require('Biostrings')
#' require('ShortRead')
#'
#' # create 6 sequences of width 20
#' set.seed(10)
#' input <- random_seq(6, 20)
#' 
#' # create qualities of width 15 and paste to qualities
#' # of length 5 used for the tails.
#' # for two of the sequences, put low qualities in tails
#' 
#' set.seed(10)
#' my_qual <- random_qual(c(30,40), slength = 6, swidth = 15, 
#' encod = 'Sanger')
#' 
#' set.seed(10)
#' tails <-   random_qual(c(30,40), slength = 6, swidth = 5, 
#'  encod = 'Sanger')
#'  
#' set.seed(10)
#' tails[2:3] <- random_qual(c(3, 20), slength = 2, 
#' swidth = 5,  encod = 'Sanger')
#' my_qual <- paste0(my_qual, tails)
#' input_q <- BStringSet(my_qual)

#' # create names
#' input_names <- seq_names(6)
#' 
#' # create ShortReadQ object
#' my_read <- ShortReadQ(sread = input,
#' quality = input_q, id = input_names)
#' 
#' # apply the filter 
#' filtered <- trim3q_filter(my_read, rm.3qual = 28)
#' 
#' # look at the trimmed sequences
#' sread(filtered)

#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export


trim3q_filter <- function(input, rm.3qual, q_format = NULL, check.encod = TRUE, 
                        remove_zero = TRUE) {

if (is.null(q_format) && !check.encod) {
    stop("Additional argument must be supplied: 
        q_format must be not null or check.encod must be TRUE")
}

if (!is.null(q_format) && check.encod) {
    check.encod <- FALSE
}


input <- create_uniform_width(input, type = "fastq")
is_edited <- attributes(input)$editedWidth

seqs <- sread(input)
qual <- quality(quality(input))

myqual_mat <- matrix(utf8ToInt(as.character(unlist(qual))), 
nrow = length(qual), 
byrow = TRUE)

if (is_edited) {
    zero_q <- which(myqual_mat == 125)
}

if (!is.null(q_format)) {
    my_encoding <- q_format
    myqual_mat <- myqual_mat - my_encoding$q
}


if (is_edited) {
    myqual_mat[zero_q] <- NA
}

if (check.encod) {
    my_encoding <- check_encoding(myqual_mat)
    q_value <- my_encoding$q
if (my_encoding$y %in% c(1, 2)) {
    myqual_mat <- myqual_mat - 33
} else {
    myqual_mat <- myqual_mat - 64
}
}


if (my_encoding$y %in% c(1, 2, 3)) {
    myqual_mat[is.na(myqual_mat)] <- 0
} else if (my_encoding$y == 4) {
    myqual_mat[is.na(myqual_mat)] <- 3  
} else if (my_encoding$y == 5) {
    myqual_mat[is.na(myqual_mat)] <- -5 
}

at <- myqual_mat < as.integer(rm.3qual)
letter_subject <- DNAString(paste(rep.int("N", ncol(at)), collapse = ""))


letter <- as(IRanges::Views(letter_subject, start = 1,
    end = rowSums(at)), "DNAStringSet")

injectedseqs <- replaceLetterAt(seqs, at, letter)


adapter <- paste(rep("N", max(width(injectedseqs))), sep = "", collapse = "")
mismatchVector <- c(rep(0, width(adapter)))

trimCoords <- trimLRPatterns(Rpattern = adapter,
    subject = injectedseqs,
    max.Rmismatch = mismatchVector, 
    ranges = TRUE)

out <- narrow(input, start = start(trimCoords), end = end(trimCoords))

if (remove_zero) {
    which.zero <- which(width(out) == 0)
if (length(which.zero) > 0) {
    out <- out[-which.zero]
}
}
out
}

panel1 <- function(input, output, session, messages, rList) {
    
    output$N <- renderPrint({
        cat(messages()$nFilterCatch, sep = "\n")
    })
    
    output$filepath <- renderPrint({
        
        if (any(rList()$filepath == "")) {
            cat("No file selected")
        } else {
            # if(dataInput() == '[1] \'error\'\n') { dataInput <- NULL return('Invalid
            # file') } else {
            cat(rList()$filepath, sep = "\n")
        }
    })
    
    output$lowComplexFilter <- renderPrint({
        cat(messages()$lowComplexFilterCatch, sep = "\n")
    })
    
    output$adaptFilter <- renderPrint({
        cat(messages()$seqFilterCatch, sep = "\n")
    })
    
    output$meanQFilter <- renderPrint({
        if (!is.null(rList()$meanQ) && is.null(rList()$thresmeanQ)) {
            cat("Please select a quality threshold value")
        } else {
            cat(messages()$meanQFilterCatch, sep = "\n")
        }
    })
    
    output$TrimQual <- renderPrint({
        if (!is.null(rList()$rm.qual) && is.null(rList()$thresQual)) {
            cat("Please select a quality threshold value")
        } else {
            cat(messages()$trim3FilterCatch, sep = "\n")
        }
    })
    
    output$trimTails <- renderPrint({
        if (!is.null(rList()$rmFixed) && is.null(rList()$rm3) && is.null(rList()$rm5)) {
            cat("Please select remove from 3' and/or 5'")
        } else {
            cat(messages()$fixedFilterCatch, sep = "\n")
        }
    })
    
    output$sizeFilter <- renderPrint({
        if (!is.null(rList()$rmSize) && is.null(rList()$rmMin) && is.null(rList()$rmMax)) {
            cat("Please select min and/or max values")
        } else {
            cat(messages()$lengthFilterCatch, sep = "\n")
        }
    })
    
    output$DuplicFilter <- renderPrint({
        cat(messages()$duplicFilterCatch, sep = "\n")
    })  # // END SALIDAS VENTANA 2 -->
    
    
    output$outResult <- renderPrint({
        if (any(rList()$filepath == "")) {
            cat("No file processed")
        } else {
            cat(messages()$outResult, sep = "\n")
        }
    })
}

#' Create random sequences
#' @description Create a
#' \code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
#' with random sequences
#' @param slength Number of sequences
#' @param swidth Width of the sequences
#' @param nuc Create sequences of DNA (nucleotides = c('A', 'C', 'G', 'T')) or
#'  RNA (nucleotides = c('A, 'C', 'G', 'U'))?. Default: 'DNA'
#' @param prob A vector of four probability values used 
#' to set the frequency of the nucleotides 'A', 'C', 'G', 'T', for DNA,
#' or 'A', 'C', 'G', 'U', for RNA. For example = c(0.25, 0.25, 0.5, 0).
#' Default is = c(0.25, 0.25, 0.25, 0.25) (equiprobability for the 4 bases).
#' If the sum of the probabilities is > 1, the values will be nomalized 
#' to the range [0, 1].
#' @return \code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
#' 
#' @examples 
#' 
#' # For reproducible examples, make a call to set.seed before 
#' # running each random function
#' 
#' set.seed(10)
#' s1 <- random_seq(slength = 10, swidth = 20)
#' s1
#' 
#' set.seed(10)
#' s2 <- random_seq(slength = 10, swidth = 20, 
#' prob = c(0.6, 0.1, 0.3, 0))
#' s2
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export

random_seq <- function(slength, swidth, nuc = c("DNA", "RNA"), 
                        prob = c(0.25, 
                        0.25, 0.25, 0.25)) {
    
    nuc <- match.arg(nuc)
    if (length(prob) > 4 || !is.numeric(prob) || any(prob < 0)) {
        stop("prob must be a numeric vector of length 4")
    }
    
    if (any(swidth < 0) || length(slength) != 1) {
        stop("swidth must be > 1; slength must be of length 1")
    }
    
    if (length(swidth) > 1) {
        if (length(swidth) != slength) 
            stop("length(swidth) and slength must be equal")
        
    } else {
        swidth <- rep(swidth, slength)
    }
    
    if (nuc == "DNA") {
        samplefrom <- c("A", "C", "G", "T")
    } else if (nuc == "RNA") {
        samplefrom <- c("A", "C", "G", "U")
    }
    
    
    prob <- prob/sum(prob)

    
    out <- list()
    for (i in seq_len(slength)) {
        samp <- sample(samplefrom, size = swidth[i], replace = TRUE, prob = prob)
        out[[i]] <- paste(samp, collapse = "")
    }
    DNAStringSet(do.call("c", out))
}


#' Create random qualities for a given encoding
#' @param qual quality range for the sequences. It must be a range included 
#' in the selected encoding:
#' 
#' 'Sanger' = [0, 40]
#' 
#' 'Illumina1.8' = [0, 41]
#' 
#' 'Illumina1.5' = [0, 40]
#' 
#' 'Illumina1.3' = [3, 40]
#' 
#' 'Solexa' = [-5, 40]
#'
#' example: for a range from 20 to 30 in Sanger encoding,
#'  pass the argument = c(20, 30)
#' @param slength number of sequences
#' @param swidth width of the sequences
#' @param encod sequence encoding
#' @param prob a vector of range = range(qual), with probabilities to set 
#' the frequency of each quality value. Default is equiprobability.
#' If the sum of the probabilities is > 1, the values will be nomalized 
#' to the range [0, 1].
#' @description Create a \code{\link[Biostrings:XStringSet-class]{BStringSet}} 
#' object
#' with random qualities
#' @return  \code{\link[Biostrings:XStringSet-class]{BStringSet}} object
#' @examples 
#' 
#' q <- random_qual(30, 20)
#' q
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export

random_qual <- function(slength, swidth, qual = NULL,
    encod = c("Sanger", "Illumina1.8", 
    "Illumina1.5", "Illumina1.3", "Solexa"), prob = NULL) {
    
    encod <- match.arg(encod)
    
    
    if (any(swidth < 0) || length(slength) != 1) {
        stop("swidth must be > 1; slength must be of length 1")
    }
    
    if (length(swidth) > 1) {
        if (length(swidth) != slength) 
            stop("length(swidth) and slength must be equal")
        
    } else {
        swidth <- rep(swidth, slength)
    }
    
    
    
    selection <- switch(encod, Sanger = 1, Illumina1.8 = 2,
        Illumina1.5 = 3, Illumina1.3 = 4, 
        Solexa = 5, stop("Invalid encoding"))
    

    seqrange <- list(0:40, 0:41, 0:40, 3:40, -5:40)
    
    seqrange <- seqrange[[selection]]
    
    if (!is.null(qual)) {
        if (!all(qual %in% seqrange)) {
            stop(paste0("Quality values out of range. 
                The values must be in the range [", 
                min(seqrange), ",", max(seqrange), "]"))
        }
    } else {
        
        qual <- seqrange
    }

    
    my_scores <- seq(min(qual), max(qual)) + c(33, 33, 33, 64, 64)[selection]
    samplefrom <- vapply(my_scores, int2asc, character(1))
    
    lscores <- length(my_scores)
    if (is.null(prob)) {
        prob <- rep(1, lscores)/lscores
    } else {
    if (length(prob) == lscores || !is.numeric(prob) || any(prob < 0)) {
    stop(paste0("prob must be a positive numeric vector of length swidth (", 
        swidth, ")"))
        }
    }

    out <- list()
    for (i in seq_len(slength)) {
        out[[i]] <- paste(sample(samplefrom, size = swidth[i], replace = TRUE, 
                                prob = prob), 
            collapse = "")
    }
    BStringSet(do.call("c", out))
}


#' Create fastq/sequences/qualities with uniform width
#' @param input input to edit
#' @param type type of the input: 'fastq' (ShortReadQ),
#'  'sequence' (DNAStringSet), 'quality' (BStringset)
#' @return ShortReadQ object or character vector with sequences 
#' or qualities, with uniform widht (padded with Ns or \})
#' @keywords internal

create_uniform_width <- function(input,
                                type = c("fastq", "sequence", "quality")) {
    
    if (type == "fastq") {
        
        seqs <- sread(input)
        qual <- PhredQuality(quality(quality(input)))
        ids <- id(input)
    }
    
    check_width <- length(unique(width(input))) != 1
    
    if (check_width) {
        
        input_width <- width(input)
        mean_width <- max(input_width)
        which_below <- which(input_width < mean_width)
        howmany <- mean_width - input_width[which_below]
        
        if (type == "fastq") {
            

            to_paste_s <- cpp_create_stringvec(howmany, "N")  # cpp source
            seqs[which_below] <- paste0(seqs[which_below], to_paste_s)
            

            to_paste_q <- cpp_create_stringvec(howmany, "}")
            qual[which_below] <- paste0(qual[which_below], to_paste_q)
            
            out <- ShortReadQ(sread = seqs, quality = qual, id = ids)
            attr(out, "editedWidth") <- TRUE
            return(out)
            
        } else if (type == "sequence") {
            
            to_paste_s <- cpp_create_stringvec(howmany, "N")
            input[which_below] <- paste0(input[which_below], to_paste_s)
            
            attr(input, "editedWidth") <- TRUE
            return(input)
            
        } else if (type == "quality") {
            to_paste_q <- cpp_create_stringvec(howmany, "}")
            input[which_below] <- paste0(input[which_below], to_paste_q)
            
            attr(input, "editedWidth") <- TRUE
            return(input)
            
        }

    } else {
        attr(input, "editedWidth") <- FALSE
        return(input)
    }
}


#' Inject a letter in a set of sequences at random positions 
#' @param my_seq character vector with sequences to inject
#' @param how_many_seqs How many sequences pick to inject Ns. 
#' An interval [min_s, max_s] with min_s minimum 
#' and max_s maximum sequences can be passed. In this case, 
#' a value is picked from the interval. 
#' If NULL, a random value within the interval [1, length(my_seq)] is picked.  
#' @param how_many_letters How many times inject the letter 
#' in the i sequences that are going to be injected. An interval [min_i max_i]
#' can be passed. In this case, a value is randomly 
#' picked for each sequence i. This value represents the number of times
#' that the letter will be injected in the sequence i.
#' If NULL, a random value within the interval [1, width(my_seq[i])] 
#' is picked for each sequence i.
#' @param letter Letter to inject. Default: 'N'
#' @return  character vector
#' @examples 
#' 
#' # For reproducible examples, make a call to set.seed before 
#' # running each random function
#' 
#' set.seed(10)
#' s <- random_seq(slength = 10, swidth = 20)
#' 
#' set.seed(10)
#' s <- inject_letter_random(s, how_many_seqs = 1:30, how_many= 2:10)
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export


inject_letter_random <- function(my_seq, how_many_seqs = NULL, 
                                how_many_letters = NULL, letter = "N") {

    
    if (!is.null(how_many_seqs)) {
        if (any(how_many_seqs < 0)) 
            stop("how_many_seqs must be > 0")
    }
    
    if (!is.null(how_many_letters)) {
        if (any(how_many_letters < 0)) 
            stop("how_many_letters must be > 0")
    }
    
    
    if (is.null(how_many_seqs)) {
        how_many_seqs <- sample(length(my_seq), 1)
    } else {
        how_many_seqs <- c(max(min(how_many_seqs), 1), 
        min(max(how_many_seqs), length(my_seq)))
        how_many_seqs <- sample(how_many_seqs[1]:how_many_seqs[2], size = 1)
    }
    
    inject_to <- sample(seq_along(my_seq), size = how_many_seqs)
    
    for (i in inject_to) {
        this_seq <- strsplit(as.character(my_seq[i]), "")[[1]]
        this_seq_len <- length(this_seq)
        
        if (is.null(how_many_letters)) {
            this_how_many <- sample(seq_len(this_seq_len), size = 1)
        } else {
            this_how_many <- c(max(min(how_many_letters), 1), 
            min(max(how_many_letters), this_seq_len))
            this_how_many <- sample(this_how_many[1]:this_how_many[2], 1)
        }
        
        positions <- sample(this_seq_len, size = this_how_many)
        this_seq[positions] <- letter
        this_seq <- paste(this_seq, collapse = "")
        my_seq[i] <- this_seq
    }
    my_seq
}


#' Create sequences names
#' @param n Number of reads
#' @param base_name Base name for strings
#' @param sep Character separing base names and the read number. Default: '_
#' @description Create \code{\link[Biostrings:XStringSet-class]{BStringSet}} 
#' object with names
#' @return  \code{\link[Biostrings:XStringSet-class]{BStringSet}} object
#' @examples 
#' snames <- seq_names(10)
#' snames
#' snames2 <- seq_names(10, base_name = 's', sep = '.')
#' snames2
#' @export

seq_names <- function(n, base_name = "s", sep = "_") {
    BStringSet(paste0(base_name, sep, seq_len(n)))
}

#' Create a named object with random sequences and qualities
#' @param n number of sequences
#' @param widths width of the sequences
#' @param random_widths width must be picked at random from the 
#' passed parameter 'widths', considering the value
#' as an interval where any integer can be picked. Default TRUE. 
#' Otherwise, widths are picked only from the vector passed.
#' @param replace sample widths with replacement? Default TRUE.
#' @param nuc create sequences of DNA (nucleotides = c('A', 'C', 'G', 'T')) or
#'  RNA (nucleotides = c('A, 'C', 'G', 'U'))?. Default: 'DNA'
#' @param len_prob vector with probabilities for each width value. 
#' Default NULL (equiprobability)
#' @param seq_prob a vector of four probabilities values to set 
#' the frequency of the nucleotides 'A', 'C', 'G', 'T', for DNA,
#' or 'A', 'C', 'G', 'U', for RNA. For example = c(0.25, 0.25, 0.5, 0).
#' Default is = c(0.25, 0.25, 0.25, 0.25) (equiprobability for the 4 bases).
#' If the sum of the probabilities is > 1, the values will be nomalized 
#' to the range [0, 1].
#' @param q_prob a vector of range = range(qual), with probabilities to set 
#' the frequency of each quality value. Default is equiprobability.
#' If the sum of the probabilities is > 1, the values will be nomalized 
#' to the range [0, 1].
#' @param qual quality range for the sequences. It must be a range included 
#' in the selected encoding:
#' 
#' 'Sanger' = [0, 40]
#' 
#' 'Illumina1.8' = [0, 41]
#' 
#' 'Illumina1.5' = [0, 40]
#' 
#' 'Illumina1.3' = [3, 40]
#' 
#' 'Solexa' = [-5, 40]
#'
#' example: for a range from 20 to 30 in Sanger encoding, 
#' pass the argument = c(20, 30)
#' @param encod sequence encoding
#' @param base_name Base name for strings
#' @param sep Character separing base names and the read number. Default: '_'
#' @return  \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @description Create a \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}}
#' object with random sequences and qualities

#' @examples 
#' 
#' # For reproducible examples, make a call to set.seed before 
#' # running each random function
#' 
#' set.seed(10)
#' s1 <- random_seq(slength = 10, swidth = 20)
#' s1
#' 
#' set.seed(10)
#' s2 <- random_seq(slength = 10, swidth = 20, 
#' prob = c(0.6, 0.1, 0.3, 0))
#' s2
#' 
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export

random_length <- function(n, widths, random_widths = TRUE,
                        replace = TRUE, len_prob = NULL, 
                        seq_prob = c(0.25, 0.25, 0.25, 0.25),
                        q_prob = NULL, nuc = c("DNA", "RNA"), qual = NULL, 
                        encod = c("Sanger", "Illumina1.8", "Illumina1.5",
                                "Illumina1.3", "Solexa"), base_name = "s", 
                        sep = "_") {
    
    
    if (any(widths < 1)) {
        stop("lengths must be > 1")
    }
    if (random_widths) {
        widths <- sample(min(widths):max(widths), n, replace = replace)
    } else {
        widths <- sample(widths, size = n, replace = replace, prob = len_prob)
    }
    
    s_out <- random_seq(slength = n, swidth = widths, nuc = nuc, 
                        prob = seq_prob)
    
    
    q_out <- random_qual(slength = n, swidth = widths, 
                        qual = qual, encod = encod, 
                        prob = q_prob)
    
    n_out <- seq_names(n = n, base_name = base_name, sep = sep)
    
    ShortReadQ(s_out, q_out, n_out)
}

context("test-adapter_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

set.seed(10)
input <- random_seq(6, 43)
# create qualities of width 50

set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 50, 
                       encod = "Sanger")

# create names
input_names <- seq_names(length(input))



test_that("test that adapter remotion from right works", {

  ### FULL ADAPTER
  # add adapter in 3'
  adapter <- "ATCGACT"
  ## Remove reverse complement from right
  my_seqs <- paste0(input, adapter)
  my_seqs <- DNAStringSet(my_seqs)
  
  # create ShortReadQ object
  my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)
  
  # trim adapter
  filtered <- adapter_filter(my_read, Rpattern = adapter)
  
  expect_equal(unique(width(filtered)), 43)
  
  
  ### PARTIAL ADAPTER
  
  adapter <- "ATCGACT"
  subadapter <- subseq(adapter, 1, 4)
  ## Remove reverse complement from right
  my_seqs <- paste0(input, subadapter)
  my_seqs <- DNAStringSet(my_seqs)
  
  # create ShortReadQ object
  my_read <- ShortReadQ(sread = my_seqs, quality = subseq(input_q, 1, 47), id = input_names)
  
  # trim adapter
  filtered <- adapter_filter(my_read, Rpattern = adapter)
  
  expect_equal(unique(width(filtered)), 43)
  
})


test_that("test partial remotion from right reverse-complemented works", {

  ## TEST RC RIGHT
  
  # add adapter in 3'
  adapter <- "ATCGACT"
  ## Remove reverse complement from right
  my_seqs <- paste0(input, as.character(Biostrings::reverseComplement(DNAString(adapter))))
  my_seqs <- DNAStringSet(my_seqs)
  
# create ShortReadQ object
  my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)
  
  # trim adapter
  filtered <- adapter_filter(my_read, Rpattern = adapter, rc.R = TRUE)
 
  expect_equal(unique(width(filtered)), 43)
  
})

test_that("test that partial remotion from left works", {
  
  
  ## FULL ADAPTER
  # add adapter in 3'
  adapter <- "ATCGACT"
  ## Remove reverse complement from right
  my_seqs <- paste0(adapter, input)
  my_seqs <- DNAStringSet(my_seqs)
  
  # create ShortReadQ object
  my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)
  
  # trim adapter
  filtered <- adapter_filter(my_read, Lpattern = adapter)
  
  expect_equal(unique(width(filtered)), 43)
  
  
  ### PARTIAL ADAPTER
  adapter <- "ATCGACT"
  subadapter <- subseq(adapter, 4, 7)
  ## Remove reverse complement from right
  my_seqs <- paste0(subadapter, input)
  my_seqs <- DNAStringSet(my_seqs)
  
  # create ShortReadQ object
  my_read <- ShortReadQ(sread = my_seqs, quality = subseq(input_q, 1, 47), id = input_names)
  
  # trim adapter
  filtered <- adapter_filter(my_read, Lpattern = adapter)
  
  expect_equal(unique(width(filtered)), 43)
  
})


test_that("test that partial remotion from left reverse-complemented works", {
  
  # add adapter in 3'
  adapter <- "ATCGACT"
  ## Remove reverse complement from right
  my_seqs <- paste0( as.character(Biostrings::reverseComplement(DNAString(adapter))), input)
  my_seqs <- DNAStringSet(my_seqs)
  
  # create ShortReadQ object
  my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)
  
  # trim adapter
  filtered <- adapter_filter(my_read, Lpattern = adapter, rc.L = TRUE)
  
  expect_equal(unique(width(filtered)), 43)
  
})


test_that("test that remotion of partial adapters within sequences works", {
  
## FROM RIGHT 
# create qualities of width 50
set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, 
                       swidth = 47, encod = "Sanger")

# create names
input_names <- BStringSet(paste0("v", seq_len(length(input))))
adapter <- "ATCGACT"
## Remove reverse complement from right
set.seed(10)
my_seqs <- paste0(random_seq(6, 20), 
                             adapter, random_seq(6, 20))
my_seqs <- DNAStringSet(my_seqs)

# create ShortReadQ object
my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)

# trim adapter
filtered <- adapter_filter(my_read, Rpattern = adapter)

expect_equal(unique(width(filtered[-c(4, 6)])), 47)

### FROM LEFT
filtered <- adapter_filter(my_read, Lpattern = adapter)

expect_equal(unique(width(filtered)), 47)

})

context("test-n_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")
set.seed(10)

# create 6 sequences of width 20
set.seed(10)
input_seq <- random_seq(50, 20)

set.seed(10)
input_seq <- inject_letter_random(input_seq, 30, 
                                  how_many_letters = seq_len(10))
set.seed(10)
input_qual <- random_qual(50, 20)
input_names <- seq_names(50)
my_read <- ShortReadQ(input_seq, input_qual, input_names)

test_that("n_filter works", {
  
# apply the filter 
filtered <- n_filter(my_read, rm.N = 5)

expect_true(all(letterFrequency(sread(filtered), "N") <= 5))

})

context("test-fixed_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

# create 6 sequences of width 20
set.seed(10)
input <- random_seq(6, 20)

# create qualities of width 20 
set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 20, 
                       encod = "Sanger")


# create names
input_names <- seq_names(6)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)


test_that("fixed filter works", {
  # apply the filter 
  filtered3 <- fixed_filter(my_read, trim5 = 5)
  
  filtered5 <- fixed_filter(my_read, trim3 = 5)
  
  filtered3and5 <- fixed_filter(my_read, trim3 = 10, trim5 = 5)
  
  expect_that(unique(width(filtered3)), equals(15))
  expect_that(unique(width(filtered5)), equals(15))
  expect_that(unique(width(filtered3and5)), equals(5))
  
})

context("test-complex_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

# create  sequences of different width
set.seed(10)
input <- lapply(c(0, 6, 10, 16, 20, 26, 30, 36, 40), 
  function(x) random_seq(1, x))


# create repetitive "CG" sequences with length adequante 
# for a total length:
# input +  CG = 40
set.seed(10)
CG <- lapply(c(20, 17, 15, 12, 10, 7, 5, 2, 0), 
             function(x) paste(rep("CG", x), collapse = ""))


# concatenate input and CG
input  <- mapply("paste", input, CG, sep = "")
input <- DNAStringSet(input)


# create qualities of widths 40
set.seed(10)
input_q <- random_qual(c(30,40), slength = 9, swidth = 40, 
                        encod = "Sanger")


# create names
input_names <- seq_names(9)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

test_that("complex filter works", {

  # apply the filter,
  filtered <- complex_filter(my_read)
  
  freq <- dinucleotideFrequency(sread(filtered))
  freq  <- freq /rowSums(freq)
  H <- -rowSums(freq  * log2(freq), na.rm = TRUE)
  H_max <- 3.908135  # max entropy
  
  expect_that(max(H), is_less_than(H_max))

})

context("test-trim3q_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

# create 6 sequences of width 20
set.seed(10)
input <- random_seq(6, 20)

# create qualities of width 15 and paste to qualities of length 5 used for the tails.
# for two of the sequences, put low qualities in tails
set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 15, encod = "Sanger")
set.seed(10)
tails <-   random_qual(c(30,40), slength = 6, swidth = 5, encod = "Sanger")
set.seed(10)
tails[2:3] <- random_qual(c(3, 20), slength = 2, swidth = 5,  encod = "Sanger")
input_q <- paste0(input_q, tails)
input_q <- BStringSet(input_q)

# create names
input_names <- seq_names(6)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)


test_that("trim3q filter works", {
  
  # apply the filter 
  filtered <- trim3q_filter(my_read, rm.3qual = 28)

  expect_true(all(width(filtered)[-c(2,3)] == 20))
  expect_true(all(width(filtered)[c(2,3)] == 15))
})

context("test-seq_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

set.seed(10)
input <- random_length(30, 3:7)

test_that("seq filter works", {
  
rm.seq  = c("GAGA", "ATTTAT", "TAGC")
match_before <- unlist(lapply(rm.seq, function(x) grep(x, as.character(sread(input)))))
filtered <- seq_filter(input,rm.seq =  rm.seq)
match_after <- unlist(lapply(rm.seq, function(x) grep(x, as.character(sread(filtered)))))

expect_gt(length(match_before),  0)
expect_equal(length(match_after), 0)
})

context("test-qmean_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")


# create 6 sequences of width 20
set.seed(10)
input <- random_seq(30, 20)

# create qualities of width 20 
## high quality (15 sequences)
set.seed(10)
my_qual <- random_qual(c(30,40), slength = 15, swidth = 20,
                       encod = "Sanger")
## low quality (15 sequences)
set.seed(10)
my_qual_2 <-   random_qual(c(5,30), slength = 15, swidth = 20, 
                            encod = "Sanger")

# concatenate vectors
input_q<- c(my_qual, my_qual_2)

# create names
input_names <- seq_names(30)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)


test_that("q_mean filter works", {

  filtered <- qmean_filter(my_read, minq = 30)
  
  # average qualities
  avq <- alphabetScore(filtered) / width(filtered)
  
  expect_true(all(avq >=30))
})

context("test-length_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

#create  ShortReadQ object width widths between 1 and 100
set.seed(10)
input <- random_length(100, widths = seq_len(100))

test_that("length filter works", {
  
  ## apply the filter, removing sequences with  10>length> 80
  filtered <- length_filter(input, rm.min = 10, rm.max = 80)
  
  expect_true(all(width(filtered) >= 10))
  expect_true(all(width(filtered) <= 80))
})

context("test-unique_filter.R")
skip_on_cran()

require("Biostrings")
require("ShortRead")

set.seed(10)
s <- random_seq(10, 10)
s <- sample(s, 30, replace = TRUE)
q <- random_qual(30, 10)
n <- seq_names(30)

my_read <- ShortReadQ(sread = s, quality = q, id = n)

test_that("unique filter works", {
  # apply the filter
filtered <- unique_filter(my_read)
expect_true(!all(isUnique(as.character(sread(my_read)))))  
expect_true(all(isUnique(as.character(sread(filtered)))))  
})

panel1Input <-  function (id) {
 ns <- NS(id)
 
 
 tabPanel("File Operations", sidebarLayout(
  
  sidebarPanel(width = 3, class = "sideBar",
               h3("FastqCleaner"),
               p("A program to clean FASTQ files"),
               br(),
               hr(),
               br(),
               div() # .well.sideBar div
  ),
  
  mainPanel(
   
   # input/output ---------------------------------
   fluidRow(
    br(),
    column(6, class = "left",
           h3(class = "blackTag", "Input/Output")),
    column(3, class = "right"),
    br(), br(), br(), br(), br()
   ),
   
   fluidRow(
    column(6,
           p(class = "results", "File input:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("filepath")))
    ),
    column(6,
           p(class = "results", "File output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("outResult")))
    ),
    br(), br(), br(), br(), br()
   ),
   
   # file operations ------------------------------
   fluidRow(
    br(),
    # esta funcion permite enviar mensajes
    column(6, class = "left",
           h3(class = "blackTag", 
              "File operations")
    ),
    column(3, class = "right"),
    br(), br(), br(), br(), br()
   ),
   
   # Outputs right ---------------------------------------------
   fluidRow(
    
    column(6,
           p(class = "results", "N filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("N"))),
           p(class = "results", "Low complexity filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("lowComplexFilter"))),
           p(class = "results", "Adapter filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("adaptFilter"))),
           p(class = "results", "Average quality filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("meanQFilter")))
    ),
    
    # outputs left --------------------------------------------------
    column(6,
           p(class = "results", "Low quality 3' tails filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("TrimQual"))),
           p(class = "results", "Fixed-length tails trim output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("trimTails"))),
           p(class = "results", "Length filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("sizeFilter"))),
           p(class = "results", "Duplicated filter output:"),
           div(style = "width:80%",
               verbatimTextOutput(ns("DuplicFilter")))
           )
    
   ) # end fluidRow
   
  ) # end mainpanel
 ) # end sidebarLayout
 ) # end tabpanel
}

#' Remove a set of sequences
#' @description Removes a set of sequences 
#' @param  input \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @param rm.seq Ccharacter vector with sequences to remove
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
#' object
#' @examples 
#' 
#' require(ShortRead)
#' 
#' set.seed(10)
#' input <- random_length(30, 3:7)
#' rm.seq  = c('TGGTC', 'CGGT', 'GTTCT', 'ATA')
#' 
#' # verify that some sequences match
#' match_before <- unlist(lapply(rm.seq,
#'  function(x) grep(x, as.character(sread(input)))))
#' 
#' filtered <- seq_filter(input,rm.seq =  rm.seq)
#' 
#' # verify that matching sequences were removed
#' match_after <- unlist(lapply(rm.seq, 
#' function(x) grep(x, as.character(sread(filtered)))))
#' 
#' @export

seq_filter <- function(input, rm.seq) {
reads <- sread(input)
matchseq <- lapply(rm.seq, function(x) {
    Biostrings::which.isMatchingStartingAt(x, subject = reads)
})
matchseq <- unlist(lapply(matchseq, function(x) which(x == 1)))
if (length(matchseq) > 0) {
    return(input[-matchseq])
}
input
}

## ----eval=FALSE, include=TRUE-------------------------------------------------
# library('FastqCleaner')
# launch_fqc()

## ----eval=FALSE, include=TRUE-------------------------------------------------
# download.file("https://goo.gl/hb4Kr9", "example_fastq.gz")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------
### Examples

## ----include=FALSE------------------------------------------------------------
require("Biostrings")
require("ShortRead")
require("FastqCleaner")
set.seed(10)

## ----eval = FALSE-------------------------------------------------------------
# require("Biostrings")
# require("ShortRead")
# require("FastqCleaner")

## ----eval=TRUE, echo=TRUE-----------------------------------------------------

# create sequences
set.seed(15) 
# nota that the use of set.seed before the call to the 
# random generators allows reproducibility of the
# examples

input <- random_seq(6, 43)
input

# create qualities of width 50
set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 50, 
encod = "Sanger")

# create names
input_names <- seq_names(length(input))


### FULL ADAPTER IN 3'
adapter <- "ATCGACT"

# Create sequences with adapter
my_seqs <- paste0(input, adapter)
my_seqs <- DNAStringSet(my_seqs)
my_seqs

# create ShortReadQ object
my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)

# trim adapter
filtered <- adapter_filter(my_read, Lpattern = adapter)
sread(filtered)

### PARTIAL ADAPTER IN 5'
adapter <- "ATCGACT"
subadapter <- subseq(adapter, 1, 4)

# Create sequences with adapter
my_seqs <- paste0(input, subadapter)
my_seqs <- DNAStringSet(my_seqs)
my_seqs

# create ShortReadQ object
my_read <- ShortReadQ(sread = my_seqs, quality = subseq(input_q, 1, 47), 
id = input_names)

# trim adapter
filtered <- adapter_filter(my_read, Rpattern = adapter)
sread(filtered)


## -----------------------------------------------------------------------------

# create  sequences of different width
set.seed(10)
input <- lapply(c(0, 6, 10, 16, 20, 26, 30, 36, 40), 
            function(x) random_seq(1, x))


# create repetitive "CG" sequences with length adequante 
# for a total length input +  CG = 40

CG <- lapply(c(20, 17, 15, 12, 10, 7, 5, 2, 0), 
            function(x) paste(rep("CG", x), collapse = ""))

# concatenate input and CG
input  <- mapply("paste", input, CG, sep = "")
input <- DNAStringSet(input)
input

# plot relative entropy (E, Shannon 1948)
H_plot <- function(x, H_max = 3.908135) {
    freq <- dinucleotideFrequency(x)
    freq  <- freq /rowSums(freq)
    H <- -rowSums(freq  * log2(freq), na.rm = TRUE)
    plot(H/H_max, type="l", xlab = "Sequence", ylab= "E")
    points(H/H_max, col = "#1a81c2", pch = 16, cex = 2)
}

H_plot(input)

## -----------------------------------------------------------------------------
# create qualities of widths 40
set.seed(10)
input_q <- random_qual(c(30,40), slength = 9, swidth = 40, 
    encod = "Sanger")

# create names
input_names <- seq_names(9)


# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# apply the filter, 
filtered <- complex_filter(my_read)
sread(filtered)

H_plot(sread(filtered))

## -----------------------------------------------------------------------------

# create sequences, qualities and names of width 20
set.seed(10)
input <- random_seq(6, 20)
input

set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 20, 
    encod = "Sanger")

input_names <- seq_names(6)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# apply the filter 
filtered3 <- fixed_filter(my_read, trim5 = 5)
sread(filtered3)

filtered5 <- fixed_filter(my_read, trim3 = 5)
sread(filtered5)

filtered3and5 <- fixed_filter(my_read, trim3 = 10, trim5 = 5)
sread(filtered3and5)


## -----------------------------------------------------------------------------

# create  ShortReadQ object width widths between 1 and 60
set.seed(10)
input <- random_length(10, widths = 1:60)
sread(input)

# apply the filter, removing sequences with length < 5 or length> 30
filtered <- length_filter(input, rm.min = 5, rm.max = 30)
sread(filtered)

## -----------------------------------------------------------------------------
# create 10 sequences of width 20
set.seed(10)
input <- random_seq(10, 20)
input

# inject N's
set.seed(10)
input <- inject_letter_random(input, how_many_seqs = 1:5,
    how_many = 1:10)
input

#'  
hist(letterFrequency(input, "N"), breaks = 0:10, 
    main  = "Ns Frequency", xlab = "# Ns",
    col = "#1a81c2")

## -----------------------------------------------------------------------------

# Create qualities, names and ShortReadQ object
set.seed(10)
input_q <- random_qual(10, 20)
input_names <- seq_names(10)
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# Apply the filter 
filtered <- n_filter(my_read, rm.N = 3)
sread(filtered)
hist(letterFrequency(sread(filtered), "N"), 
    main = "Ns distribution", xlab = "",
    col = "#1a81c2")

## -----------------------------------------------------------------------------

# create 30 sequences of width 20, 15 with low quality and 15 with high quality
set.seed(10)
input <- random_seq(30, 20)

set.seed(10)
my_qual_H <- random_qual(c(30,40), slength = 15, swidth = 20,
    encod = "Sanger")

set.seed(10)
my_qual_L <- random_qual(c(5,30), slength = 15, swidth = 20, 
    encod = "Sanger")
input_q<- c(my_qual_H, my_qual_L)

input_names <- seq_names(30)
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# Plot of average qualities
qual_plot <- function(x, cutoff) {
q <- alphabetScore(x) / width(x)
plot(q, type="l", xlab = "Sequence", ylab= "Average quality", ylim = c(0, 40))
points(q, col = "#1a81c2", pch = 16, cex = 2)
lines(seq_along(q), rep(cutoff, length(q)), type="l", col = "red", lty=2)
text(length(q), cutoff+2, cutoff)
}

#' Average qualities before
qual_plot(my_read, cutoff = 30)

## -----------------------------------------------------------------------------
# Apply the filter
filtered <- qmean_filter(my_read, minq = 30)

# Average qualities after
qual_plot(filtered, cutoff = 30)


## -----------------------------------------------------------------------------

# Generate random sequences
set.seed(10)
input <- random_length(30, 3:7)

# Remove sequences that contain the following patterns:
rm.seq  = c("TGGTC", "CGGT", "GTTCT", "ATA")
match_before <- unlist(lapply(rm.seq, function(x) grep(x, 
as.character(sread(input)))))
match_before

filtered <- seq_filter(input,rm.seq =  rm.seq)

# Verify that matching sequences were removed
match_after <- unlist(lapply(rm.seq, function(x) {
                grep(x, as.character(sread(filtered)))}))
match_after

## -----------------------------------------------------------------------------

# Create 6 sequences of width 20
set.seed(10)
input <- random_seq(6, 20)
input

# Create Phred+33 qualities of width 15 and paste to qualities of length 
# 5 used for the tails.
# for three of the sequences, put low qualities in tails
set.seed(10)
my_qual <- random_qual(c(30,40), slength = 6, swidth = 15, 
    encod = "Sanger")
set.seed(10)
tails <-   random_qual(c(30,40), slength = 6, swidth = 5, 
    encod = "Sanger")

# Low quality tails in sequences 2, 3 & 4
set.seed(10)
tails[2:4] <- random_qual(c(3, 20), slength = 3, swidth = 5,
    encod = "Sanger")
my_qual <- paste0(my_qual, tails)
input_q <- BStringSet(my_qual)
input_q

# Watch qualities before filtering
as.matrix(PhredQuality(input_q))

# Create names and ShortReadQ object
input_names <- seq_names(6)
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# Apply the filter 
filtered <- trim3q_filter(my_read, rm.3qual = 28)
sread(filtered)


## -----------------------------------------------------------------------------

# Create duplicated sequences
s <- random_seq(10, 10)
s <- sample(s, 30, replace = TRUE)

# Create a ShortReadQ object
q <- random_qual(30, 10)
n <- seq_names(30)
my_read <- ShortReadQ(sread = s, quality = q, id = n)

# Check presence of duplicates
isUnique(as.character(sread(my_read)))

# Apply the filter
filtered <- unique_filter(my_read)
isUnique(as.character(sread(filtered)))

##########################
# FastqCleaner installer
##########################

message("\n")
message("######################")
message("   FastqCleaner")
message("######################\n")
message("Installing dependencies, this may take a while...")
message("\n")

R_repo <- "https://cloud.r-project.org/"

# set internet options
options(download.file.method = "libcurl", url.method = "libcurl")

# set library -------------------------------------------------------------------------------------
where <- normalizePath(Sys.getenv("R_LIBS_USER"))
if(!dir.exists(where)) {
  dir.create(where, recursive =  TRUE)
}
.libPaths(c(.libPaths(), where))

github_where <- paste("--library=", paste0("'", where, "'"))

# Check OS ---------------------------------------------------------------------------------------
get_os <- function(){
  sysinf <- Sys.info()
  if (!is.null(sysinf)){
    os <- sysinf['sysname']
    if (os == 'Darwin')
      os <- "osx"
  } else { ## mystery machine
    os <- .Platform$OS.type
    if (grepl("^darwin", R.version$os))
      os <- "osx"
    if (grepl("linux-gnu", R.version$os))
      os <- "linux"
  }
  tolower(os)
}

this_os <- get_os()

# Check dependences for each os

if (this_os == "osx") {
  test_xcode <- class(try(system2("gcc --version", stdout = TRUE), silent = TRUE))
  if(is(test_xcode, "try-error")) {
    
    stop(toupper("Command Line Tools is required. Please paste install Command Line Tools pasting the following 
          command in a Terminal and then selecting install in the popup dialog:\n 
          xcode-select --install"))
  }
  if(length(list.files("/opt/X11/bin", pattern = "Xquartz")) == 0) {
  stop(toupper("X11 is required. Please visit www.xquartz.org to download and install Xquartz."))
  }
}

if(this_os == "linux") {
  is_ubuntu <- grep("ubuntu", sessionInfo()$running,  ignore.case = TRUE)
  if(length(is_ubuntu) > 0) {
    p1 <- is(try(system2("dpkg -l |grep libssl-dev", stdout =  TRUE)), 
                "try-error")
    p2 <- is(try(system2("dpkg -l |grep libcurl4-openssl-dev", stdout =  TRUE)),
                "try-error")
    msg <- ""
    what <- ""
    if(p1) {
      msg <- ("libssl-dev in required\n")
      what <- "libssl-dev "
    }
    if(p2) {
      msg <- c(msg, "libcurl4-openssl-dev in required\n")
      what <- paste0(what, "libcurl4-openssl-dev ")
    }
    if(p1 || p2) {
      stop(toupper(paste0(msg, "Please install the dependences with the following command\n: sudo apt get install ", what)))
    }
  } else {
    cat(toupper("-----------------------------------------------------------------------------------\n
Important: please check that the 'libcurl' and 'libssl' packages corresponding 
to your Linux distribution are installed. If the installation fails, 
                    install them and try again.\n
-----------------------------------------------------------------------------------\n\n"))
  }
  
}

if(this_os == "windows") {
  if(!require(devtools)) {
  cat(toupper("Installing devtools...\n"))
  try(install.packages("devtools", lib = where, repos = R_repo, quiet = TRUE, verbose = FALSE), silent = TRUE)
  }
  
  if(!require(devtools)) {
    warning(toupper("Cannot install devtools. Trying to install Rtools..."))
    install.packages("installr",lib = where, repos = R_repo)
    rtoolsexit <- installr::install.Rtools()
    if(!rtoolsexit) {
      stop("Failed.\nRtools needs to be manually installed from: https://cran.r-project.org/bin/windows/Rtools/")
    }
  }
}


# install dependences------------------------------------------------------------------------------
packages <- c("devtools", "Rcpp", "DT", "shiny", "methods", "graphics", "shinyBS", "RCurl")

# install Bioconductor dependences----------------------------------------------------------------
bpackages <- c("IRanges", "Biostrings", "ShortRead", "S4Vectors")
bpackages <- c("IRanges", "Biostrings", "ShortRead", "S4Vectors")
source("https://bioconductor.org/biocLite.R")
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")


for(i in bpackages) {
for(i in bpackages) {
  if (!require(i, character.only = TRUE)) {
  if (!require(i, character.only = TRUE)) {
    biocLite(i, lib = where,suppressUpdates = TRUE, dependencies = TRUE)
    BiocManager::install(i, lib = where,suppressUpdates = TRUE, dependencies = TRUE)
  }
  }
}
}

# install package --------------------------------------------------------------------------------
library("devtools")
withr::with_libpaths(where, 
install_github("leandroroser/FastqCleaner", build_vignettes = TRUE, verbose = TRUE, args = github_where)
)
# EOF

#' Remove full and partial adapters from a ShortReadQ object
#' @param input  \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @param Lpattern 5' pattern (character or
##' \code{\link[Biostrings:DNAString-class]{DNAString}} object)
#' @param Rpattern 3' pattern (character or 
#' \code{\link[Biostrings:DNAString-class]{DNAString}} object)
#' @param rc.L Reverse complement Lpattern? default FALSE
#' @param rc.R Reverse complement Rpatter? default FALSE
#' @param first trim first right('R') or left ('L') side of sequences
#'  when both Lpattern and Rpattern are passed
#' @param error_rate Error rate (value in the range [0, 1] 
#' The error rate is the proportion of mismatches allowed between
#' the adapter and the aligned portion  of the subject.
#' For a given adapter A, the number of allowed mismatches between each 
#' subsequence s of A and the subject is computed as: error_rate * L_s,
#' where L_s is the length of the subsequence s
#' @param with_indels Allow indels? This feature is available only 
#' when the error_rate is not null
#' @param anchored Adapter or partial adapter within sequence
#' (anchored = FALSE, default) or only in 3' and 5' terminals? 
#' (anchored = TRUE)
#' @param fixed Parameter passed to 
#' \code{\link[Biostrings]{trimLRPatterns}} 
#' Default 'subject',  ambiguities in the pattern only are interpreted 
#' as wildcard. See the argument fixed in 
#' \code{\link[Biostrings]{trimLRPatterns}} 
#' @param remove_zero Remove zero-length sequences? Default TRUE
#' @param checks Perform checks? Default TRUE
#' @param min_match_flank Do not trim in flanks of the subject,
#'if a match has min_match_flank of less length. Default 1L 
#'(only trim with >=2 coincidences in a flank match)
#' @param ... additional parameters passed to 
#' \code{\link[Biostrings]{trimLRPatterns}} 
#' @return  Edited \code{\link[Biostrings:DNAString-class]{DNAString}} or 
#' \code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
#' @description This program can remove adapters and partial 
#' adapters from 3' and 5', using the functions
#' \code{\link[Biostrings]{trimLRPatterns}} 
#'  The program extends the methodology of
#' the  \code{\link[Biostrings]{trimLRPatterns}} function of \pkg{Biostrings},
#' being also capable of removing adapters present within reads and with other
#' additional otpions 
#' (e.g., threshold of minimum number of bases for trimming). 
#' For a given position in the read, the two Biostrings functions return TRUE 
#' when a match is present between a substring of the read and the adapter.
#' As \code{\link[Biostrings]{trimLRPatterns}} , adapter_filter also selects
#  the longest subsequence that includes the aligned
#' region and goes up to the end of the sequence in the corresponding flank 
#' as the best match. The default error rate is 0.2.
#' If several valid matches are found, the function removes the
#' largest subsequence. Adapters can be anchored or not.  
#' When indels are allowed, the second method uses the 'edit distance' between 
#' the subsequences and the adapter
#' 
#' @examples
#' 
#' require('Biostrings')
#' require('ShortRead')
#' 
#' # create 6 sequences of width 43
#' set.seed(10)
#' input <- random_seq(6, 43)
#' 
#' # add adapter in 3' 
#' adapter <- "ATCGACT"
#' 
#' input <- paste0(input, as.character(DNAString(adapter)))
#' input <- DNAStringSet(input)
#' 
#' # create qualities of width 50
#' set.seed(10)
#' input_q <- random_qual(c(30,40), slength = 6, swidth = 50, 
#' encod = 'Sanger')
#'
#' # create names
#' input_names <- seq_names(length(input))
#'
#' # create ShortReadQ object
#' my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)
#'
#' # trim adapter
#' filtered <- adapter_filter(my_read, Rpattern = adapter)
#'
#' # look at the filtered sequences
#' sread(filtered)
#' 
#' 
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
#' object
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export
#'

adapter_filter <- function(input, Lpattern = "", 
    Rpattern = "", rc.L = FALSE, rc.R = FALSE, 
    first = c("R", "L"), with_indels = FALSE, 
    error_rate = 0.2, anchored = TRUE, 
    fixed = "subject", remove_zero = TRUE, 
    checks = TRUE, min_match_flank = 3L, ...) {
    
    first <- match.arg(first)
    
    if (is.character(Lpattern)) {
        Lpattern <- sub(" ", "", Lpattern)
        Lpattern <- DNAString(Lpattern)
    }
    
    if (is.character(Rpattern)) {
        Rpattern <- sub(" ", "", Rpattern)
        Rpattern <- DNAString(Rpattern)
    }
    
    if (rc.L) {
        Lpattern <- reverseComplement(Lpattern)
    }
    
    if (rc.R) {
        Rpattern <- reverseComplement(Rpattern)
    }
    

    
    if (length(Lpattern) > 0 && length(Rpattern) > 0) {
        
        if (first == "L") {
            
            out <- cutLseq(subject = sread(input),
                Lpattern = Lpattern, fixed = fixed, 
                error_rate = error_rate, anchored = anchored,
                checks = checks,  min_match_flank = min_match_flank, 
                ranges = TRUE, ...)
            
            out <- cutRseq(subject = out, Rpattern = Rpattern, 
                fixed = fixed,  
                error_rate = error_rate, anchored = anchored, 
                checks = checks, min_match_flank = min_match_flank, 
                ranges = TRUE, ...)
        } else {
            
            out <- cutRseq(subject = sread(input), 
                Rpattern = Rpattern, fixed = fixed, 
                error_rate = error_rate, anchored = anchored, 
                checks = checks, min_match_flank = min_match_flank,
                ranges = TRUE, ...)
            
            out <- cutLseq(subject = out, Lpattern = Lpattern,
                fixed = fixed,  error_rate = error_rate, anchored = anchored, 
                checks = checks, min_match_flank = min_match_flank, 
                ranges = TRUE, ...)
        }
        
        
    } else if (length(Lpattern) == 0 && length(Rpattern) > 0) {

        
        out <- cutRseq(subject = sread(input), 
            Rpattern = Rpattern, fixed = fixed, 
            error_rate = error_rate, anchored = anchored, 
            checks = checks, min_match_flank = min_match_flank,
            ranges = TRUE, ...)
        
    } else if (length(Lpattern) > 0 && length(Rpattern) == 0) {

        out <- cutLseq(subject = sread(input), 
            Lpattern = Lpattern, fixed = fixed, 
            error_rate = error_rate, anchored = anchored, 
            checks = checks, min_match_flank = min_match_flank, 
            ranges = TRUE, ...)
        

    } else if (length(Lpattern) == 0 && length(Rpattern) == 0) 
        {
            
            return(input)
        }  
    
    
    
    out <- narrow(input, start = start(out), end = end(out))
    
    
    if (remove_zero) {
        which.zero <- which(width(out) == 0)
        if (length(which.zero) > 0) {
            out <- out[-which.zero]
        }
    }
    
    out
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Compute CG content
#' @param x StringMatrix with bases, of reads x cycles
NULL

cpp_GC_content <- function(x) {
    .Call('_FastqCleaner_cpp_GC_content', PACKAGE = 'FastqCleaner', x)
}

#'Compute the base content in a DNA matrix
#'@param x matrix with nucleotides for each position (column)
#'of the reads (rows)
#'@param relative. Logical. If true (default), the function
#' computes relative frequencies
NULL

cpp_base_content <- function(x, relative = TRUE) {
    .Call('_FastqCleaner_cpp_base_content', PACKAGE = 'FastqCleaner', x, relative)
}

#' check by row if the values of a matrix are above a 
#' threshold of values in a vector returns a matrix of rows by threshold values
#' @param x NumericMatrix to be checked
#' @param thresVector vector with thresholds
NULL

cpp_check_quality_threshold <- function(x, thresVector) {
    .Call('_FastqCleaner_cpp_check_quality_threshold', PACKAGE = 'FastqCleaner', x, thresVector)
}

#' create a StringVector using the size of a given NumericVector
#' @param vec Input NumericVector
#' @param x character used to fill the output vector
NULL

cpp_create_stringvec <- function(vec, x) {
    .Call('_FastqCleaner_cpp_create_stringvec', PACKAGE = 'FastqCleaner', vec, x)
}

#' Select the correct match position for matching operations with
#' multiple strings 
#' @description The program takes as input a logical matrix that 
#' indicates with TRUE or FALSE if a match is present in a given position
#'  or not, respectively. Each column is a sequence, each row a cycle
#'  for NGS experiments
#' @param m LogicalMatrix with TRUE and FALSE values
#' @param width length of each sequence. This allow to discard fake matches
#' @param origin If origin = "start", select the first match. 
#' If origin = "end", selects the last match.
#' If origin  = "end_inverted" selects the first match starting 
#' from n_i positions, where n is the  length of the sequence represented 
#' in the column i.  This case works when the last match is 
#' seeked in reversed sequences.
NULL

cpp_which_true <- function(m, width, origin) {
    .Call('_FastqCleaner_cpp_which_true', PACKAGE = 'FastqCleaner', m, width, origin)
}

  
 navbarPage(
    id = "navbar",
    title = list(tags$div(
      tags$img(src = "images/icon.png",
        alt = "icon"),
      tags$p(" FastqCleaner")
    )),
    header = tags$head(
          tags$link(
              rel = "icon",
              href = "images/icon.png",
              type = "image/png",
              width = "1em",
              height = "1em"
          ),
          tags$title("FastqCleaner"),
          tags$link(rel = "stylesheet",
              type = "text/css",
              href = "styles/masterCss.css")
      ),
    
    
    tabPanel( "Clean the Data", sidebarLayout(
      
      sidebarPanel(
        width = 3,
        class = "sideBar",
        h3("FastqCleaner"),
        p("A program to clean FASTQ files"),
        br(),
        hr(),
        br(),
        div() 
        
      ),
      
      mainPanel(
        
        fluidRow(
          br(),
          
          column(6, class = "left",
            singleton(tags$head(
              tags$script(
                'Shiny.addCustomMessageHandler("testmessage",
                function(message) { alert(message);
                });'                               )
      )),
            h3(class = "blackTag", "Select the operations")
            ),
          
          column(3, class = "right"),
          br(),
          br(),
          br(),
          br()
            ), 
        
        bsPopover("help", "",
          "Help - documentation",
          options = list(container = "body")
        ),
        
        
        fluidRow(
          
          column(6, class = "accordion-container",
            tags$button(id = "button1", class = "accordion", 
              "1. Remove by N(s) #",
              tags$span()
            ),
            
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rm.N",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                numericInput("maxN",
                  label = p(class = "controlcheck-p1", 
                    "Maximum number of N(s)"),
                  value = 3,
                  min = 0
                ),
                uiOutput("error_maxN")
              )
            ), 
            
            bsPopover("maxN", "",
              "The maximum number of Ns in a read",
              options = list(container = "body")
            ),
            
            
            tags$button(
              id = "button2",
              class = "accordion",
              "2. Remove low complexity sequences",
              tags$span()
            ),
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rm.lowcomplex",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                numericInput("complexThres",
                  label = p(class = "controlcheck-p1", 
                    "Complexity threshold"),
                  value = 0.5,
                  min = 0
                ),
                uiOutput("error_complexThres")
              )
            ),
            
            bsPopover("complexThres","",
              "Shanon entropy/reference entropy",
              options = list(container = "body")
            ),
            
            
            tags$button(id = "button3", class = "accordion",
              "3. Remove adapters",
              tags$span()
            ),
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rm.adapt",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                
                textInput("LpatternF",
                  label = p(class = "controlcheck-p1",
                    "Left forward adapter"),
                  value = ""
                ),
                checkboxInput("reverseLF",
                  label = p(class = "controlcheck-p1-radio",
                    "use reverse complement?"),
                  value = FALSE
                ),
                textInput("RpatternF",
                  label = p(class = "controlcheck-p1",
                    "Right forward adapter"),
                  value = ""
                ),
                checkboxInput("reverseRF",
                  label = p(class = "controlcheck-p1-radio",
                    "use reverse complement?"),
                  value = FALSE
                ),
                textInput("LpatternR",
                  label = p(class = "controlcheck-p1",
                    "Left reverse adapter"),
                  value = ""
                ),
                checkboxInput("reverseLR",
                  label = p(class = "controlcheck-p1-radio",
                    "use reverse complement?"),
                  value = FALSE
                ),
                textInput("RpatternR",
                  label = p(class = "controlcheck-p1",
                    "Right reverse adapter"),
                  value = ""
                ),
                checkboxInput("reverseRR",
                  label = p(class = "controlcheck-p1-radio",
                    "use reverse complement?"),
                  value = FALSE
                ),
                
                
                tags$button(id = "advancedAdapterButton", 
                  class = "adButton",
                  "Advanced",
                  style="margin-left:2em"
                ),
                br(), br(),
                
                
                fluidRow(id = "advancedAdapterMenu",
                  column(12,style = "float:left;",
                    checkboxInput("indels",
                      label = p(class = "controlcheck-p1-radio",
                        "with Indels?"),
                      value = FALSE
                    ),
                    checkboxInput("anchored",
                      label = p(class = "controlcheck-p1-radio",
                        "anchored adapters?"),
                      value = TRUE
                    ),
                    
                    numericInput("min_match_flank",
                      label = p(class = "controlcheck-p1",
                        "Minimum match in flanks"),
                      value = 1,
                      min = 0,
                      step = 1
                    ),
                    
                    
                    numericInput("errate",
                      label = p(class = "controlcheck-p1",
                        "Error rate"),
                      value = 0.2,
                      min = 0,
                      max = 1,
                      step = 0.1
                    ),
                    uiOutput("error_errate"),
                    
                    
                    radioButtons("first_forward",
                      label = p(class = "controlcheck-p1", 
                        "Remove first in PE forward"),
                      choices = list("5'" = "L", "3'" = "R"),
                      selected = "R"
                    ),
                    radioButtons("first_reverse",
                      label = p(class = "controlcheck-p1", 
                        "Remove first in PE reverse"),
                      choices = list("5'" = "L", "3'" = "R"),
                      selected = "R"
                    )
                  ) 
                )
              )
              
            ),
            
            bsPopover("LpatternF", "",
              "The left pattern for forward sequences",
              options = list(container = "body")
            ),
            
            
            bsPopover("RpatternF", "",
              "The right pattern for forward sequences",
              options = list(container = "body")
            ),
            
            bsPopover("LpatternR", "",
              "The left pattern for reverse sequences",
              options = list(container = "body")
            ),
            
            bsPopover("RpatternR", "",
              "The right pattern for reverse sequences",
              options = list(container = "body")
            ),
            
            bsPopover("indels", "", 
              "allow indels?",
              options = list(container = "body")
            ),
            
            bsPopover("anchored", "", 
              "anchored adapters?",
              options = list(container = "body")
            ),
            
            bsPopover("min_match_flank", "",
              "minimum overlap between adapter and reads 
              to trim for a match in flanking region of reads",
              options = list(container = "body")
            ),
            
            bsPopover("errate", "",
              "error rate value for error rate method",
              options = list(container = "body")
            ),
            
            
            tags$button(id = "button4", class = "accordion",
              "4. Filter by average quality?",
              tags$span()
            ),
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("meanQ",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                
                numericInput("thresmeanQ",
                  label = p(class = "controlcheck-p1",
                    "Average Quality threshold"),
                  value = 20,
                  min = 0
                ),
                uiOutput("error_thresmeanQ")
              )
              
            ),
            
            bsPopover("thresmeanQ", "",
              "Mean quality threshold",
              options = list(container = "body")
            )
            
          ), 
          
          column(6, class = "accordion-container",
            
            
            tags$button(id = "button5", class = "accordion",
              "5. Trim low quality 3' tails?",
              tags$span()
            ),
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rm.qual", label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                numericInput("thresQual",
                  label = p(class = "controlcheck-p1", 
                    "Quality threshold"),
                  value = 20,
                  min = 0
                ),
                uiOutput("error_thresQual")
                
              )
            ), 
            
            bsPopover("thresQual", "",
              "Threshold quality in  tail",
              options = list(container = "body")
            ),
            
            tags$button(id = "button6", class = "accordion",
              "6. Trim 3' or 5' by a fixed number?",
              tags$span()
            ),
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rmFixed",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                numericInput("rm3",
                  label = p(class = "controlcheck-p1", 
                    "Trim 3'"),
                  value = NULL,
                  min = 0
                ),
                numericInput("rm5",
                  label = p(class = "controlcheck-p1", 
                    "Trim 5'"),
                  value = NULL,
                  min = 0
                ),
                uiOutput("error_rm35")
              )
            ), 
            
            bsPopover("rm3", "",
              "Number of bases to trim from 3 end",
              options = list(container = "body")
            ),
            
            bsPopover("rm5", "",
              "Number of bases to trim from 5 end",
              options = list(container = "body")
            ),
            
            tags$button(id = "button7", class = "accordion",
              "7. Filter sequences by length?",
              tags$span()
            ),
            
            div(class = "panelCustom",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rmSize",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                ),
                numericInput("rmMin",
                  label = p(class = "controlcheck-p1", 
                    "Min length"),
                  value = NULL,
                  min = 0
                ),
                numericInput("rmMax",
                  label = p(class = "controlcheck-p1", 
                    "Max length"),
                  value = NULL,
                  min = 0
                ),
                uiOutput("error_rmMinMax")
              )
            ), 
            
            bsPopover("rmMin", "",
              "Minimum number of bases in a read",
              options = list(container = "body")
            ),
            
            bsPopover("rmMax", "",
              "Maximum number of bases in a read",
              options = list(container = "body")
            ),
            
            tags$button(id = "button8", class = "accordion",
              "8. Remove duplicated sequences?",
              tags$span()
            ),
            
            div(class = "panelCustom",
              style = "margin-bottom:20em",
              wellPanel(class = "inAccordion",
                checkboxGroupInput("rm.duplic",
                  label = "",
                  choices = list("Use filter?" = 1),
                  selected = 0
                )
              )
            ) 
          ),
          
          
          br(), br()
          
          ), 
        
        
        br(), br(), br(),
        
        
        fluidRow(
          column(6, class = "left",
            h3(class = "blackTag", "Select a file")),
          column(3, class = "right"),
          br(),br(),br(),br()
        ),
        
        
        fluidRow(id = "filecontainer",
          column(6,
            fluidRow(
              column(12, style = "float:left; margin-left:2em;margin-top:1em;",
                radioButtons("fileTypeIn", 
                  p(class="controlcheck-p3", "Library type"),
                  choices = c("single-end reads" = "SR", 
                    "paired-end reads" = "PE"))
              ),
              column(12, style = "float:left; margin-left:2em;",
                htmlOutput("selection"),
                actionButton(class = "setButton", "goButton", "Run!")
              ),
              
              column(12, style = "float:left; margin-left:2em; margin-top: 2em",
                p(class="controlcheck-p3", "File selected:"),
                verbatimTextOutput("myPath")
              ),
              column(12, style = "float:left; margin-left:2em;margin-top:1em;",
                p(class="controlcheck-p3", "Encoding:"),
                verbatimTextOutput("encoding")
              )
            )
          ), 
          
          
          column(6,
            fluidRow(
              column(12, style = "float:left; margin-left:2em;",
                style = "float:left; margin-right:2em;;margin-top:1em;",
                radioButtons("fileTypeOut",
                  p(class="controlcheck-p3", "Output format"),
                  choices = c("FASTQ" = "fastq", "gz (compressed)" = "gz"),
                  selected = "fastq")
              ),
              column(12, style = "float:left; margin-left:2em;",
                actionButton(class = "setButton", "resetSelections", "Clear"),
                actionButton(class = "setButton", "resetAll", "Reset")
              )
            )
          ),
          
          
          
          fluidRow(
            column(12, 
              tags$button(id = "advancedButton",
                class = "adButton",
                "Advanced"
              )
            ),
            
            column(12,
              fluidRow(id = "advancedMenu",
                column(6, style = "float:left; margin-left:2em;",
                  radioButtons("customEncod",
                    p(class ="controlcheck-p1 encoding",
                      "The program will auto detect encoding.
                      You can also select a custom format of the following:"),
                    choices = c(
                      "None selected" = 0,
                      "Sanger    | Q range: [33-73]" = 1,
                      "Illumina 1.8+   | Q range: [33-74]" = 2,
                      "Illumina 1.3+   | Q range: [64-104]" = 3,
                      "Illumina 1.5+   | Q range: [66-104]" = 4,
                      "Solexa    | range: [59-104]" = 5
                    )
                    ) 
              ), 
                column(6, style = "float:left; margin-left:2em;",
                  numericInput("nfile",
                    label = p(class ="controlcheck-p1 nfile", 
                      "Reads sampled for encoding detection and file processing"),
                    value = 100000
                  )
                )
                
              ) 
            )
          ) 
        ), 
        
        
        
        tags$script(src = 'scripts/javascript.js'),
        
        singleton(tags$head(
          tags$script(src = 'shared/jqueryui/jquery-ui.min.js')
        )),
        
        tags$script(src = "scripts/highcharts.js"),
        tags$script(src = "scripts/highcharts-more.js"),
        tags$script(src = "scripts/exporting.js"),
        tags$script(src = "scripts/offline-exporting.js"),
        tags$script(src = "scripts/FqC_highcharts_style.js"),
        tags$script(src = "scripts/heatmap.js"),
        
        htmlOutput('archivo'),
        htmlOutput('check1'),
        htmlOutput('check2'),
        htmlOutput('check3'),
        htmlOutput('check4'),
        htmlOutput('check5'),
        htmlOutput('check6'),
        htmlOutput('check7'),
        htmlOutput('check8'),
        htmlOutput('runDone'),
        htmlOutput('set_encoding_radio')
        
            ) 
    ) 
      ), 
    
    panel1Input("panel1"),
    
    
    tabPanel("Live Results", sidebarLayout(
      
      sidebarPanel(width = 3, id = "menuPlot", class = "sideBarPlot",
        numericInput("sample", label = p(class = "controlcheck-p3", 
          "Sample size"),  value = 10000),
        radioButtons("filePlot",
          p(class = "controlcheck-p3", "File to plot:"),
          choices = c("Input" = "my_input", "Output" = "my_output")
        ),
        br(),
        selectInput("plotType",
          label = p(class = "controlcheck-p3", "Diagnostic plots"),
          choices = c(
            "Select a plot..." = "",
            "Per cycle quality" = "a",
            "Per cycle mean base quality" = "b",
            "Mean quality distribution" = "c",
            "% reads with Phred scores > threshold" = "d",
            "Per cycle base proportion (barplot)" = "e",
            "Per cycle base proportion (lineplot)" = "f",
            "CG content over all reads" = "g",
            "Read length" = "h",
            "Read occurrence" = "i",
            "Relative k-mer diversity" = "j"
            
            
          ),
          selected = "none"
        ), 
        br(),
        numericInput(
          "klength",
          label = p(class = "controlcheck-p3", "Select kmer size"),
          value = 6
        ),
        br(),
        numericInput(
          "maxFreq",
          label = p(class = "controlcheck-p3", 
            "Top sequences in duplication level analysis"),
          value = 20
        ),
        
        br(),br(),br(),br(),br(),br(),br(),br(),
        br(),br(),br(),br(),br(),br(),br(),br()
      ),
      
      
      mainPanel(
        htmlOutput("plot1"),
        htmlOutput("plot2"),
        uiOutput("plotPanel"),
        bsModal("modalTable",
          title = "Most represented sequences",
          trigger = "tabBut",
          size = "large",
          DT::dataTableOutput('freqTable1'),
          DT::dataTableOutput('freqTable2')
        ),
        uiOutput("tabBut")
      )
      
    ) 
    ), 
    
    panel2Input("panel2"),
    
    panel3Input("panel3")
    
) 

#' Filter sequences of a FASTQ file by length 
#' @param input  \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
#' @param rm.min Threshold value for the minimun number of bases 
#' @param rm.max Threshold value for the  maximum number of bases
#' @description The program removes from a ShortReadQ object those sequences
#' with a length lower than rm.min or/and higher than rm.max
#' @return  Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
#' object
#' 
#' @examples 
#' require('Biostrings')
#' require('ShortRead')
#' 
#' # create  ShortReadQ object width widths between 1 and 100
#'  
#'  set.seed(10)
#' input <- random_length(100, widths = 1:100)
#' 
#' # apply the filter, removing sequences length < 10 or length > 80
#' filtered <- length_filter(input, rm.min = 10, rm.max = 80)
#' 
#' # look at the filtered sequences
#' sread(filtered)
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export
#'
#'


length_filter <- function(input, rm.min = NA, rm.max = NA) {
if (!is.na(rm.min) && !is.na(rm.max)){
    if (rm.min > rm.max) {
        stop("The value of rm.min cannot be smaller than rm.max")
    }
}

if (!is.na(rm.max)) {
    input <- input[width(input) <= rm.max]
}
if (!is.na(rm.min)) {
    input <- input[rm.min <= width(input)]
}

input
}

# Auxiliar commands for documentation of the package with roxygen2 in Rstudio

#' @useDynLib FastqCleaner
#' @import  Rcpp
# @importFrom shiny renderDataTable
#' @importFrom graphics hist
#' @importFrom methods as is
#' @importFrom S4Vectors isConstant
#' @importFrom DT renderDataTable dataTableOutput
#' @importFrom stats quantile
#' @importFrom shinyBS bsPopover bsModal bsAlert
#' @importFrom Biostrings DNAString reverseComplement dinucleotideFrequency PhredQuality quality trimLRPatterns oligonucleotideFrequency DNAStringSet BStringSet replaceLetterAt
#' @importFrom ShortRead sread id ShortReadQ FastqSampler yield nFilter
#' @importFrom IRanges narrow start end width
#' @import htmltools


utils::globalVariables(c("DNAString", "reverseComplement", "sread", "narrow", 
                          "start", "end", "width", "dinucleotideFrequency", "PhredQuality", 
                          "quality", "id", "ShortReadQ", " width", "trimLRPatterns", "FastqSampler", 
                          "yield", "nFilter", "oligonucleotideFrequency", "DNAStringSet", 
                          "BStringSet", "replaceLetterAt", "occurrenceFilter", "method"))

NULL

#' Is natural number
#' @return Logical
#' @keywords  internal

isNaturalNumber <- function(x) {


if (length(x) != length(unlist(x))) {
    return(FALSE)
}
x <- unlist(x)


if (any(is.na(x))) {
    return(FALSE)
}

if (!is.numeric(x)) {
    return(FALSE)
}

if (!all(x == round(x) & x > 0)) {
    return(FALSE)
}
TRUE
}


#' ASCII to integer
#' @return  Integer
#' @keywords internal

asc2int <- function(x) strtoi(charToRaw(x), 16L)


#' Integer to ASCII 
#' @return ASCII character
#' @keywords internal

int2asc <- function(n) rawToChar(as.raw(n))


#' Check quality encoding
#' @param x Quality values
#' @param custom custom encoding from the following:
#' 
#' 'Sanger' --------> expected range: [0, 40]
#' 
#' 'Illumina1.8' --------> expected range: [0, 41]
#' 
#' 'Illumina1.5' --------> expected range: [0, 40]
#' 
#' 'Illumina1.3' --------> expected range: [3, 40]
#' 
#' 'Solexa' --------> expected range: [-5, 40]
#' 
#' @examples
#' 
#' require(Biostrings)
#' 
#' x <- list(PhredQuality(0:40), SolexaQuality(-5:40), IlluminaQuality(3:40))
#' x <- lapply(x, function(i)utf8ToInt(as.character(i)[1]))
#' lapply(x, check_encoding)
#'  
#' SolexaQuality(0:40)
#' IlluminaQuality(0:40)
#' @return  List with encoding information
#' @author Leandro Roser \email{learoser@@gmail.com}
#' @export

check_encoding <- function(x = NULL, custom = NULL) {

if (!is.null(x) && !is.null(custom)) {
    stop("Either only a set of qualities 
    or a custom encoding must be selected")
}


if (!is.null(custom) && !custom %in% seq_len(5)) {
    stop("argument custom must be in the range [1, 5]")
}

outFormats <- list(
    A = list(x = "Sanger", y = 1, q = 33, range = "[33 - 73]"), 
    B = list(x = "Illumina 1.8+", y = 2, q = 33, range = "[33; 74]"), 
    C = list(x = "Illumina 1.5+", y = 3, q = 64, range = "[66; 104]"), 
    D = list(x = "Illumina 1.3+", y = 4, q = 64, range = "[64; 104]"),
    E = list(x = "Solexa", y = 5, q = 64, range = "[59; 104]"))

if (!is.null(x)) {
    xmin <- min(x, na.rm = TRUE)
    xmax <- max(x, na.rm = TRUE)

if (xmin < 33 || xmax > 104) {
    stop(paste0("Quality values corrupt. Found [min = ", 
        xmin, " max=", xmax, "],  
    but a range between [33; 104] was expected\n"))
}

if (xmin >= 33 && xmax <= 73) 
    out <- outFormats$A  
else if (xmin >= 33 && xmax <= 74) 
    out <- outFormats$B  
else if (xmin >= 66 && xmax <= 104) 
    out <- outFormats$C 
else if (xmin >= 64 && xmax <= 104) 
    out <- outFormats$D 
else if (xmin >= 59 && xmax <= 104) 
    out <- outFormats$E 

} else {
if (custom == 1) 
    out <- outFormats$A  
else if (custom == 2) 
    out <- outFormats$B 
else if (custom == 3) 
    out <- outFormats$C  
else if (custom == 4) 
    out <- outFormats$D 
else if (custom == 5) 
    out <- outFormats$E 
}

out

}

check_onclick_ <- FastqCleaner:::check_onclick_
messageFun_ <- FastqCleaner:::messageFun_
processingFunction_ <- FastqCleaner:::processingFunction_
outputClean_ <- FastqCleaner:::outputClean_
create_cleanfunction_ <- FastqCleaner:::create_cleanfunction_
check_encoding <- FastqCleaner::check_encoding


shinyServer(
  function(input, output, session) {
    
    if (Sys.getenv("SHINY_PORT") == "") 
      options(shiny.maxRequestSize = 10000 * 1024^2)
    
    session$onSessionEnded(function() {
      stopApp()
    })  
    this_envir <- environment()
    
    
    hasRun <- reactiveValues(x = FALSE)
    
    
    output$myPath <- renderPrint(cat("choose a file..."))
    output$encoding <- renderPrint(cat("..."))
    
    output$runDone <- renderPrint({
      HTML(paste0(""))
    })
    
    tPlot <- reactiveValues(x = list())
    
    filepath <- reactiveValues(x = "")
    
    observeEvent(input$fileTypeIn, {
      filepath$x <- ""
      output$myPath <- renderPrint(cat("choose a file..."))
      output$encoding <- renderPrint(cat("..."))
      tPlot$x <- list(HTML("<script></script>"))
    })
    
    
    readsWidth <- reactiveValues(nFilterCatch = list(list(numeric(), numeric()), 
      list(numeric(), numeric())), 
      lowComplexFilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())),
      seqFilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())),
      meanQFilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())), 
      trim3FilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())), 
      fixedFilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())), 
      lengthFilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())), 
      duplicFilterCatch = list(list(numeric(), 
        numeric()), list(numeric(), numeric())))
 

    messages <- reactiveValues(nFilterCatch = c("Filter unused", ""), 
      lowComplexFilterCatch = c("Filter unused", ""), 
      seqFilterCatch = c("Filter unused", ""), 
      meanQFilterCatch = c("Filter unused", ""), 
      trim3FilterCatch = c("Filter unused", ""), 
      fixedFilterCatch = c("Filter unused", ""), 
      lengthFilterCatch = c("Filter unused", ""), 
      duplicFilterCatch = c("Filter unused", ""), 
      outResult = c("No file processed yet", ""))
    
    
    observe({
      if (input$resetSelections == 0) {
        return()
      }
      isolate({
        
        passfile <- NULL
        vars <- c("nFilterCatch", "lowComplexFilterCatch", "seqFilterCatch", 
          "meanQFilterCatch", "trim3FilterCatch", "fixedFilterCatch", 
          "lengthFilterCatch", "duplicFilterCatch")
        
        for (i in vars) {
          messages[[i]] <- c("Filter unused", "")
          readsWidth[[i]] <- list(list(numeric(), 
            numeric()), list(numeric(), 
              numeric()))
        }
        
        messages$outResult = c("No file processed yet", "")
        
        inputNames <- c("rm.N", "rm.lowcomplex", "rm.adapt", 
                        "rm.qual", "rmFixed", 
                        "rmSize", "rm.duplic", "nuc")
        
        for (i in inputNames) {
          updateCheckboxGroupInput(session, i, selected = character(0))
        }
        
        output$runDone <- renderPrint({
          HTML(paste0("<script> 
            $(\"#button1 span\").removeClass(\"checked\");
            $(\"#button2 span\").removeClass(\"checked\");
            $(\"#button3 span\").removeClass(\"checked\");
            $(\"#button4 span\").removeClass(\"checked\");
            $(\"#button5 span\").removeClass(\"checked\");
            $(\"#button6 span\").removeClass(\"checked\");
            </script>"))
        })
        
        showModal(modalDialog(title = "FastqCleaner message", 
          "Selections reseted!", 
          footer = modalButton("Ok")))
        
        })
    })
    
    observeEvent(input$resetAll, {
      
      filepath$x <- ""
      output$myPath <- renderPrint(cat("choose a file..."))
      output$encoding <- renderPrint(cat("..."))
      passfile <- NULL
      
      vars <- c("nFilterCatch", "lowComplexFilterCatch", 
        "seqFilterCatch", "meanQFilterCatch",
        "trim3FilterCatch", "fixedFilterCatch", 
        "lengthFilterCatch", "duplicFilterCatch")
      
      for (i in vars) {
        messages[[i]] <- c("Filter unused", "")
        readsWidth[[i]] <- list(list(numeric(), numeric()), 
          list(numeric(), numeric()))
      }
      
      messages$outResult = c("No file processed yet", "")
      
      inputNames <- c("rm.N", "rm.lowcomplex", "rm.adapt",
        "rm.qual", "rmFixed", 
        "rmSize", "rm.duplic", "nuc")
      
      for (i in inputNames) {
        updateCheckboxGroupInput(session, i, selected = character(0))
      }
      output$runDone <- renderPrint({
        HTML(paste0("<script> 
          $(\"#button1 span\").removeClass(\"checked\");
          $(\"#button2 span\").removeClass(\"checked\");
          $(\"#button3 span\").removeClass(\"checked\");
          $(\"#button4 span\").removeClass(\"checked\");
          $(\"#button5 span\").removeClass(\"checked\");
          $(\"#button6 span\").removeClass(\"checked\");
          </script>"))
      })
      
      output$set_encoding_radio <- renderText({
        out <- list()
        for (i in seq_len(5)) {
          out[[i + 1]] <- HTML({
            paste0("<script>
              $( \"input[name='customEncod'][value ='", 
              i, "']\" ).prop(\"checked\", false);
              </script>")
          })
          }
        out[[1]] <- HTML({
          paste0("<script>
            $( \"input[name='customEncod'][value ='", 
            0, "']\" ).prop(\"checked\", true);
            </script>")
        })
        unlist(out)
        })  
      
      showModal(modalDialog(title = "FastqCleaner message",
        "All reseted!", footer = modalButton("Ok")))
      })
    
    
    error_selection <- reactiveValues(error_maxN = FALSE, 
      error_complexThres = FALSE,
      error_errate = FALSE, 
      error_thresmeanQ = FALSE,
      error_thresQual = FALSE, 
      error_rm35 = FALSE, 
      error_rmMinMax = FALSE)
    
    
    
    NCleanData <- reactive({
      if (!is.null(input$rm.N)) {
        TRUE
      } else {
        FALSE
      }
    })
    
    check_onclick_(NCleanData, 1, this_envir)
    
    
    output$error_maxN <- renderUI({
      if (is.na(input$maxN) || input$maxN > 0) {
        error_selection$error_maxN <- FALSE
        tags$span()
      }
      if (!is.na(input$maxN)) {
        if (input$maxN < 0) {
          error_selection$error_maxN <- TRUE
          tags$span("error: please select a value > 0",
            class = "errormsg")
        }
      }
    })
    
    
    lowComplexData <- reactive({
      if (!is.null(input$rm.lowcomplex)) {
        TRUE
      } else {
        FALSE
      }
    })
    
    check_onclick_(lowComplexData, 2, this_envir)
    
    output$error_complexThres <- renderUI({
      if (is.na(input$complexThres) || input$complexThres > 0) {
        error_selection$error_complexThres <- FALSE
        tags$span()
      }
      if (!is.na(input$complexThres)) {
        if (input$complexThres < 0) {
          error_selection$error_complexThres <- TRUE
          tags$span("error: please select a value > 0", 
            class = "errormsg")
        }
      }
    })
    
    
    SeqInput <- reactive({
      if (!is.null(input$rm.adapt)) {
        TRUE
      } else {
        FALSE
      }
    })
    
    output$error_errate <- renderUI({
      if (is.na(input$errate) || input$errate > 0) {
        error_selection$error_complexThres <- FALSE
        tags$span("")
      }
      if (!is.na(input$errate)) {
        if (input$errate < 0 || input$errate > 1) {
          error_selection$errate <- TRUE
          tags$span("error: please select a value between 0 and 1", 
            class = "errormsg")
        }
      }
    })
    
    check_onclick_(SeqInput, 3, this_envir)
    
    
    
    FilterbymeanQ <- reactive({
      if (!is.null(input$meanQ) && !is.null(input$thresmeanQ)) {
        TRUE
      } else {
        FALSE
      }
    })
    
    output$error_thresmeanQ <- renderUI({
      if (is.na(input$thresmeanQ) || input$thresmeanQ > 0) {
        error_selection$error_thresmeanQ <- FALSE
        tags$span("")
      }
      if (!is.na(input$thresmeanQ)) {
        if (input$thresmeanQ < 0) {
          error_selection$error_thresmeanQ <- TRUE
          tags$span("error: please select a value < 0", 
            class = "errormsg")
        }
      }
    })
    
    check_onclick_(FilterbymeanQ, 4, this_envir)
    
    
    
    TrimCleanData <- reactive({
      if (!is.null(input$rm.qual) && !is.null(input$thresQual)) {
        TRUE
      } else {
        FALSE
      }
    })
    
    
    output$error_thresQual <- renderUI({
      if (is.na(input$thresQual) || input$thresQual > 0) {
        error_selection$error_thresQual <- FALSE
        tags$span("")
      }
      if (!is.na(input$thresQual)) {
        if (input$thresQual < 0) {
          error_selection$error_thresQual <- TRUE
          tags$span("error: please select a value < 0", class = "errormsg")
        }
      }
    })
    
    check_onclick_(TrimCleanData, 5, this_envir)
    
    
    FixedCleanData <- reactive({
      if ((!is.null(input$rmFixed) && 
          !is.null(input$rm3)) || 
          (!is.null(input$rmFixed) && 
              !is.null(input$rm5))) {
        TRUE
      } else {
        FALSE
      }
    })
    
    
    
    output$error_rm35 <- renderUI({
      error_selection$error_rm35 <- ifelse(is.na(input$rm3), 
        FALSE, 
        ifelse(input$rm3 >  0, FALSE, TRUE)) || ifelse(is.na(input$rm5), 
          FALSE, ifelse(input$rm5 > 
              0, FALSE, TRUE))
      if (error_selection$error_rm35) {
        tags$span("error: please select values > 0", class = "errormsg")
      } else {
        tags$span("")
      }
    })
    
    
    check_onclick_(FixedCleanData, 6, this_envir)
    
    
    
    SizeCleanData <- reactive({
      if ((!is.null(input$rmSize) && !is.null(input$rmMin)) || 
          (!is.null(input$rmSize) && 
              !is.null(input$rmMax))) {
        TRUE
      } else {
        FALSE
      }
    })
    
    
    output$error_rmMinMax <- renderUI({
      error_selection$error_rmMinMax <- ifelse(is.na(input$rmMin), 
        FALSE, ifelse(input$rmMin > 0, FALSE, TRUE)) || 
        ifelse(is.na(input$rmMax), 
          FALSE, ifelse(input$rmMax >  0, FALSE, TRUE))
      if (error_selection$error_rmMinMax) {
        tags$span("error: please select values > 0", class = "errormsg")
      } else {
        tags$span("")
      }
    })
    
    
    check_onclick_(SizeCleanData, 7, this_envir)
    
    
    
    DuplicCleanData <- reactive({
      if (!is.null(input$rm.duplic)) {
        TRUE
      } else {
        FALSE
      }
    })
    
    check_onclick_(DuplicCleanData, 8, this_envir)
    
    
    output$selection <- renderUI({
      if (input$fileTypeIn == "SR") {
        actionButton(class = "setButton", "selection", "File")
      } else {
        tagList(actionButton(class = "setButton",
          "selection", "File 1"), 
          actionButton(class = "setButton", 
            "selection2", "File 2"))
      }
    })
    
    
    selection_detect <- reactiveValues(x = FALSE, y = FALSE)
    
    observeEvent(input$selection, {
      filepath$x <- try(file.choose(), silent = TRUE)
      selection_detect$x <- TRUE
      if (length(filepath$x) == 0) {
        filepath$x <- ""
        output$myPath <- renderPrint(cat("choose a file..."))
        output$encoding <- renderPrint(cat("..."))
        selection_detect$x <- FALSE
      }
    })
    
    observeEvent(input$selection2, {
      filepath$x[2] <- try(file.choose(), silent = TRUE)
      selection_detect$y <- TRUE
      if (!selection_detect$x) {
        filepath$x <- ""
        showModal(modalDialog(title = "FastqCleaner message",
          "Please load first File 1", 
          footer = modalButton("Ok")))
      }
    })
    
    my_encoding <- reactiveValues(value = NULL)
    
    observe({
      
      if (input$fileTypeIn == "SR") {
        req(selection_detect$x)
      } else {
        req(selection_detect$x)
        req(selection_detect$y)
      }
      
      isolate({
        progress <- Progress$new(session, min = 0, max = length(filepath$x))
        progress$set(message = "Analyzing file...")
        
        printPath <- TRUE
        printEncoding <- TRUE
        sampleQual <- integer()
        
        cat("Checking file and encoding...\n")
        t1 <- proc.time()
        
        if (length(filepath$x) > 1) {
          nsamp <- round(input$sample)/2
        } else {
          nsamp <- input$sample
        }
        
        for (i in filepath$x) {
          temporalPath <- try(ShortRead::FastqStreamer(i, n = nsamp), 
            silent = TRUE)
          thisTemporalFile <- try(ShortRead::yield(temporalPath), 
            silent = TRUE)
          try(close(temporalPath), silent = TRUE)
          
          if (length(thisTemporalFile) == 0 || 
              is(thisTemporalFile, "try-error")) {
            filepath$x <- ""
            showModal(modalDialog(title = "FastqCleaner message",
              "Invalid file: ShortRead cannot recognize the file format, 
              or your file is empty", 
              footer = modalButton("Ok")))
            printPath <- FALSE
            printEncoding <- FALSE
            break
          }
          
          new_sample <- Biostrings::quality(Biostrings::quality(thisTemporalFile))
          new_sample <- utf8ToInt(as.character(unlist(new_sample)))
          sampleQual <- c(sampleQual, new_sample)
          rm(thisTemporalFile)
        }
        my_encoding$value <- check_encoding(x = sampleQual)
        encodType <- "Encoding auto detected"
        
        t2 <- proc.time()
        outtime <- t2 - t1
        cat("OK...checks performed in ", outtime[3], " seconds\n\n")
        if (input$customEncod != 0) {
          my_encoding_custom <- check_encoding(custom = input$customEncod)
          if (!identical(my_encoding, my_encoding_custom)) {
            warning(paste0("The program detected the following encoding: ", 
              my_encoding, "\n", " You selected: my_encoding_custom"))
          }
          
          my_encoding$value <- my_encoding_custom
          encodType <- "Encoding selected by user"
        }
        
        radio_to_check <- my_encoding$value[["y"]]
        output$set_encoding_radio <- renderText({
          out <- list()
          for (i in seq_len(6)) {
            if (radio_to_check != i) {
              out[[i]] <- HTML({
                paste0("<script>
                  $( \"input[name='customEncod'][value ='", 
                  i, "']\" ).prop(\"checked\", false);
                  </script>")
              })
              } else {
                out[[i]] <- HTML({
                  paste0("<script>
                    $( \"input[name='customEncod'][value ='", 
                    i, "']\" ).prop(\"checked\", true);
                    </script>")
                })
                }
            }
          unlist(out)
        })  
        progress$close()
        
        if (printPath) {
          output$myPath <- renderPrint(cat(length(filepath$x), 
            ifelse(length(filepath$x) == 1, 
              "file selected", 
              "files selected")))
        }
        
        
        if (printEncoding) {
          output$encoding <- renderPrint(cat("Encoding:", 
            my_encoding$value[["x"]], 
            "; sample range: ", my_encoding$value[["range"]], 
            "; n = ", input$nfile, 
            "\n", encodType))
        }
        
        selection_detect$x <- FALSE
        selection_detect$y <- FALSE
      })
      })
    
    observe({
      
      if (input$goButton == 0) {
        return()
      }
      isolate({
        
        if (any(filepath$x == "")) {
          
          showModal(modalDialog(title = "FastqCleaner message",
            "Please select a file first!", 
            footer = modalButton("Ok")))
          return()
        }
        
        errors_in_selection <- unlist(reactiveValuesToList(error_selection))
        if (any(errors_in_selection)) {
          showModal(modalDialog(title = "FastqCleaner message", 
            "Invalid values present. Please revise the entered values", 
            footer = modalButton("Ok")))
          return()
        }
        
        vars <- c("nFilterCatch", "lowComplexFilterCatch", "seqFilterCatch", 
          "meanQFilterCatch", "trim3FilterCatch",
          "fixedFilterCatch", "lengthFilterCatch", 
          "duplicFilterCatch")
        
        for (i in vars) {
          messages[[i]] <- c("Filter unused", "")
          readsWidth[[i]] <- list(list(numeric(), numeric()), list(numeric(), 
            numeric()))
        }
        messages$outResult = c("No file processed yet", "")
        
        
        proctime_start <- proc.time()
        
        
        processingFunction_(this_envir)
        
        
        proctime_end <- proc.time()
        
        proctime <- proctime_end - proctime_start
        
        cat("-------------------------------\n")
        cat("\nProcessing finished\n\n")
        cat("Processing performed in ", proctime[1], "seconds\n")
        
        
        cat("Processing successful!\n")
        showModal(modalDialog(title = "FastqCleaner message", "Done!", 
          footer = modalButton("Ok")))
        hasRun$x <- TRUE
      })
      
    })  
    
    reactP2 <- reactive(reactiveValues(filepath = filepath$x,
      meanQ = input$meanQ,
      thresmeanQ = input$thresmeanQ, 
      rm.qual = input$rm.qual, 
      thresQual = input$thresQual,
      rmFixed = input$rmFixed, 
      rm3 = input$rm3, 
      rm5 = input$rm5, 
      rMin = input$rmMin, 
      rmMax = input$rmMax))
    
    
    callModule(panel1, "panel1", messages = reactive(messages), rList = reactP2)
    
    
    reactP3 <- reactive(reactiveValues(filepath = filepath$x,
      fileTypeIn = input$fileTypeIn, 
      goButton = input$goButton))
    
    
    temporalPlot <- reactiveValues(x = list())
    
    isPaired <- reactiveValues()
    observe({
      if (input$fileTypeIn == "SR") {
        isPaired$x <- FALSE
      } else {
        isPaired$x <- TRUE
      }
    })
    
    
    observeEvent(input$fileTypeIn, 
      temporalPlot$x <- list(HTML("<script></script>")))
    
    error_state <- reactiveValues(x = FALSE)
    
    error_state_init <- reactiveValues(x = FALSE)
    
    empty_plot <- reactiveValues(x = FALSE)
    
    has_new_plots <- reactiveValues(x = FALSE)
    
    validate_plot <- debounce(reactive({
      hasRun$x
      input$filePlot
      input$sample
      input$klength
      input$maxFreq
      
      has_new_plots$x <- FALSE
      empty_plot$x <- FALSE
      
      updateSelectInput(session, "plotType", selected = "...")
      
      
      if (input$goButton == 0 || any(filepath$x == "")) {
        temporalPlot$x <- list(HTML("<script></script>"))
        return(FALSE)
      }
      
      error_state$x <- !(FastqCleaner:::isNaturalNumber(list(input$sample - 1, 
        input$klength, input$maxFreq)))
      
      if (error_state$x) {
        updateSelectInput(session, "plotType", selected = "...")
        output$plot1 <- renderPrint(HTML("<script>
          $( '#container1' ).addClass('alert');
          $( '#container1' ).append( '<p> Oops... sample size, k-mer length and top sequences must be integer numbers! </p>' );
          $( '.cssload-fond' ).addClass('hidden');
          </script>"))
        error_state_init$x <- TRUE  
        return(FALSE)
        
      }
      
      if (error_state_init$x) {
        output$plot1 <- renderPrint(HTML("<script>
          $( '#container1' ).removeClass('alert');
          $( '#container1' ).children().remove();
          $( '.cssload-fond' ).removeClass('hidden');
          </script>"))
        error_state_init$x <- FALSE
      }
      
      
      if (input$filePlot == "my_output") {
        if (!isPaired$x) {
          length_try <- length(ShortRead::yield(
            temp_stream <- try(
              ShortRead::FastqStreamer(paste0(filepath$x[1], 
                "_out"), 10)), silent = TRUE))
          if (length_try == 0) {
            try(close(temp_stream), silent = TRUE)
            empty_plot$x <- TRUE
            return(TRUE)
          }
        } else {
          length_try_L <- length(ShortRead::yield(
            temp_stream <- try(
              ShortRead::FastqStreamer(paste0(filepath$x[1], 
                "_out"), 10)), silent = TRUE))
          if (length_try_L == 0) {
            try(close(temp_stream), silent = TRUE)
            empty_plot$x <- TRUE
            return(TRUE)
          }
          length_try_R <- length(ShortRead::yield(
            temp_stream <- try(
              ShortRead::FastqStreamer(paste0(filepath$x[1], 
                "_out"), 10)), silent = TRUE))
          if (length_try_R == 0) {
            try(close(temp_stream), silent = TRUE)
            empty_plot$x <- TRUE
            return(TRUE)
          }
        }
      }
      
      
      output$plot1 <- renderPrint(HTML("<script>
        $( '#container1' ).children().remove();
        </script>"))
      
      TRUE
    }), 1000)
    
    
    observe({
      validate_plot
      if (!validate_plot()) {
        return()
      }
      
      
      isolate({
        updateSelectInput(session, "plotType", selected = "...")
        
        progress <- Progress$new(session)
        progress$set(message = "Computing plots...")
        
        if (input$filePlot == "my_input") {
          temporalPlot$x <- FastqCleaner:::myPlot(isPaired$x, 
            filepath$x, input$sample, 
            input$klength, "input", input$maxFreq)
          
        } else if (input$filePlot == "my_output") {
          
          temporalPlot$x <- FastqCleaner:::myPlot(isPaired$x, 
            sprintf("%s_out",filepath$x), 
            input$sample, 
            input$klength,
            "output", input$maxFreq)
          
        }
        progress$close()
        has_new_plots$x <- TRUE
      })
      
    })
    
    
    output$plotPanel <- renderUI({
      
      
      cuantos <- input$fileTypeIn
      if (cuantos == "SR") {
        nplots <- 1
      } else {
        nplots <- 2
      }
      
      
      if (!hasRun$x || input$goButton == 0 || 
          any(filepath$x == "") || error_state$x || 
          empty_plot$x) {
        outplots <- tags$div(id = "container1", 
          style = "height: 400px; margin: auto; width: 800px")
        output$tabBut <- renderUI(HTML("<p></p>"))
      } else {
        
        outplots <- lapply(1:nplots, function(i) {
          plotname <- paste0("plot", i)
          tags$div(id = paste0("container", i), 
            style = "height: 400px; margin: auto; width: 800px")
        })
        outplots <- do.call(tagList, outplots)
      }
      
      div(id = "plot-container", 
        style = "margin-left:150px", 
        div(class = "cssload-fond", 
          div(class = "cssload-container-general", 
            div(class = "cssload-internal", 
              div(class = "cssload-ballcolor cssload-ball_1")), 
            div(class = "cssload-internal", 
              div(class = "cssload-ballcolor cssload-ball_2")), 
            div(class = "cssload-internal", 
              div(class = "cssload-ballcolor cssload-ball_3")),
            div(class = "cssload-internal", 
              div(class = "cssload-ballcolor cssload-ball_4")))), 
        outplots)
      
    })
    
    
    observeEvent(input$plotType, {
      
      if (!has_new_plots$x) {
        return()
      }
      
      if (input$goButton == 0 || any(filepath$x == "") || error_state$x) {
        
        if (!error_state$x) {
          output$plot1 <- renderPrint(HTML("<script></script>"))
        }
        output$plot2 <- renderPrint(HTML("<script></script>"))
        output$tabBut <- renderUI(HTML("<script></script>"))
        output$freqTable1 <- DT::renderDataTable(data.frame())
        output$freqTable2 <- DT::renderDataTable(data.frame())
        return()
      }
      
      isolate({
        
        if (empty_plot$x) {
          
          output$plot1 <- renderPrint(HTML("<script>
            $( '#container1' ).addClass('alert');
            $( '.highcharts-container ' ).empty();
            $( '#container1' ).append( '<p>Oops... the output file is empty! </p>' );
            $( '.cssload-fond' ).addClass('hidden');
            </script>"))
          updateSelectInput(session, "plotType", selected = "...")
          
          return()
        }
        
        if (is.null(input$plotType)) {
          selected <- "a"
        } else {
          selected <- input$plotType
        }
        
        
        output$plot1 <- renderPrint(tagList(HTML("<script>
          $('#container1').removeClass('alert');
          $('#container1').children().remove();
          $('.cssload-fond').removeClass('hidden');
          </script>"), 
          HTML(temporalPlot[["x"]][[1]][[selected]])))
        
        if (input$fileTypeIn == "SR") {
          
          
          if (selected == "j") {
            output$freqTable1 <- DT::renderDataTable(
              temporalPlot$x[[1]][["oligofreq"]], 
              width = "20%", 
              options = list(lengthChange = FALSE, pageLength = 8))
            output$tabBut <- renderUI(
              actionButton("tabBut", "View kmer table"))
          } else if (selected == "i") {
            output$freqTable1 <- DT::renderDataTable(
              temporalPlot$x[[1]][["table"]], 
              width = "20%", options = list(lengthChange = FALSE, 
                pageLength = 8))
            output$tabBut <- renderUI(actionButton("tabBut", "View Table"))
          } else {
            
            output$tabBut <- renderUI(HTML("<p></p>"))
          }
          
        } else {
          
          output$plot1 <- renderPrint(HTML(temporalPlot[["x"]][[1]][[selected]]))
          output$plot2 <- renderPrint(HTML(temporalPlot[["x"]][[2]][[selected]]))
          if (selected == "j") {
            output$freqTable1 <- DT::renderDataTable(
              temporalPlot[["x"]][[1]][["oligofreq"]], 
              width = "20%", options = list(lengthChange = FALSE, 
                pageLength = 8))
            output$freqTable2 <- DT::renderDataTable(
              temporalPlot[["x"]][[2]][["oligofreq"]], 
              width = "20%", options = list(lengthChange = FALSE, 
                pageLength = 8))
            output$tabBut <- renderUI(actionButton("tabBut", 
              "View kmer table"))
          } else if (selected == "i") {
            output$freqTable1 <- DT::renderDataTable(
              temporalPlot[["x"]][[1]][["table"]], 
              width = "20%", options = list(lengthChange = FALSE, 
                pageLength = 8))
            output$freqTable2 <- DT::renderDataTable(
              temporalPlot[["x"]][[2]][["table"]], 
              width = "20%", options = list(lengthChange = FALSE, 
                pageLength = 8))
            output$tabBut <- renderUI(actionButton("tabBut", "View Table"))
          } else {
            output$tabBut <- renderUI(HTML("<p></p>"))
          }
          
        }
      })
    })
    
    callModule(panel2, "panel2")
    
    callModule(panel3, "panel3", arg = reactive(input$navbar))
    
    })

#' plotObjects
#' Create the information required to construct the plots. 
#' This is the input of myplot, which uses the values created for 
#' this function to construct the plots
#' @return List with information to construct the diagnostic plots
#' @keywords internal

plotObjects <- function(fq, klength, basename, maxFreq, sampleSize) { 

out <- list()
fq.width <- width(fq)
fq.length <- length(fq)


if(fq.length == 0) {
    return(out)
}

input_seqs <- sread(fq)
seqs <- input_seqs
qual <- PhredQuality(quality(fq))


if(length(unique(fq.width)) == 1) {
    out$qual <- as.matrix(qual) 
    out$seq <- as.matrix(seqs)
} else {


mymax <- max(fq.width)
cuales <- which(width(seqs) < mymax)
agregar <- mymax - width(seqs[cuales])


to_paste_s <- cpp_create_stringvec(agregar, "-")
seqs[cuales] <- paste0(seqs[cuales], to_paste_s)

to_paste_q <- cpp_create_stringvec(agregar, " ")
qual[cuales] <- paste0(qual[cuales], to_paste_q)

out$seq <-  as.matrix(seqs)
out$qual <- as.matrix(qual)

out$seq[out$seq=="-"] <- NA
mode(out$qual) <- "numeric"
out$qual[out$qual == -1] <- NA
}
colnames(out$qual) <- colnames(out$seq) <- seq_len(ncol(out$seq))




bprange <- apply(out$qual, 2, function(x) { 
round(quantile(x, c(0.1, 0.25, 0.5, 0.75, 0.9),  na.rm = TRUE))
})
out$bp_cycle <- data.frame(colnames(out$qual), t(bprange))
colnames(out$bp_cycle) <- c("Cycle", "min", "low", "mid", "top", "max")


A_ <- T_ <- G_ <- C_ <- out$qual

A_[out$seq %in% c("T", "G", "C")] <- NA
A_ <- colMeans(A_, na.rm=TRUE)

T_[out$seq %in% c("A", "G", "C")] <- NA
T_ <- colMeans(T_, na.rm=TRUE)

G_[out$seq %in% c("T", "A", "C")] <- NA
G_ <- colMeans(G_, na.rm=TRUE)

C_[out$seq %in% c("T", "G", "A")] <- NA
C_ <- colMeans(C_, na.rm=TRUE)

out$meanBaseQuality <- data.frame(Quality=c(A_, C_, G_, T_), 
Base=rep(c("A", "C", "G", "T"), 
        each=length(A_)), 
        Cycle=rep(colnames(out$qual, 4)))

out$meanBaseQuality[, 1] <- round(out$meanBaseQuality[, 1], 2)



qv <- table(round(rowMeans(out$qual)))[as.character(seq(0, max(out$qual,
na.rm=TRUE), 1))]
qv[is.na(qv)] <- 0
names(qv) <- seq(0, max(out$qual, na.rm=TRUE), 1)
meanQReads <- data.frame(qv)
colnames(meanQReads) <- c("Quality", "Percent")
meanQReads[,2] <- 100 * meanQReads[,2] / sum(meanQReads[, 2])
meanQReads[,1] <- factor(meanQReads[,1], 
                    levels=unique(meanQReads[,1]), 
                    ordered=TRUE)
                    out$meanQReads <- meanQReads


                    
qualth <- seq_len(40)
qual_above <- cpp_check_quality_threshold(out$qual, qualth)
qual_above <- 100 * colSums(qual_above) / nrow(out$qual)
out$qual_above <- qual_above

base_content <- 100 * cpp_base_content(out$seq)
out$base_content <- round( base_content , 2)

GC_Content <- cpp_GC_content(out$seq)
GC_Content <- 100 * rowSums(GC_Content[,seq_len(2)],
                        na.rm = TRUE) / rowSums(GC_Content, na.rm =  TRUE)
out$GC <- hist(GC_Content, plot = FALSE, breaks = "Sturges")



freqreads <- table(fq.width)
out$length_distribution$ydata <- unname(freqreads / sum(freqreads) * 100)
out$length_distribution$xdata <- factor(names(freqreads), 
levels=unique(names(freqreads)), ordered=TRUE)



qa1 <- ShortRead::qa(fq, basename(basename), n = sampleSize)
seqtable <- qa1[["frequentSequences"]]
counts <- as.numeric(as.vector(seqtable$count)) 
myDT <-data.frame(round(100* counts/sampleSize, 1))
myDT <-data.frame(Sequences = seqtable[, 1], 
Percent = round(100*seqtable[, 2]/sampleSize, 1))
colnames(myDT)[2] <- paste0("% of sample (n=", sampleSize, ")")
out$dt <- myDT

myintervals <- data.frame(labels = c('1', '2-10', '11-100', 
'101-1k', '1k-10k', '>10k'), 
lower = c(1,2,11,101,1001,10001), 
upper = c(2,11,101,1001,10001, Inf))

count_ocurrence <- vapply(seq_along(myintervals[,1]), 
                function(x) {
                sum(counts >= myintervals[x,2] & counts < myintervals[x,3])
                }, integer(1))
ocurrenceData <- data.frame(labels = myintervals[,1], 
Percent = count_ocurrence)
ocurrenceData[, 1] <- factor(ocurrenceData[,1], 
                        levels = unique(ocurrenceData[,1]), 
                        ordered = TRUE)
                        out$ocurrenceData <- ocurrenceData


                        
                        
oligofreq <- oligonucleotideFrequency(input_seqs, klength)
oligofreq <- colSums(oligofreq)
oligosum <- sum(oligofreq)
oligofreq <- oligofreq[oligofreq  != 0]
oligofreq <- oligofreq[order(oligofreq , decreasing = TRUE)]
oligofreq <- round(100 * oligofreq/oligosum, 1)
oligofreq <- data.frame(sequence = names(oligofreq), 
percent = unname(oligofreq))
colnames(oligofreq)[2]<-"% in sample"
out$oligofreq <- oligofreq
out$oligosum <- oligosum
out$klength <- klength


loopv <- seq_len(min(width(seqs) - (klength-1)))

kcount <- lapply(loopv, function(y) { 
DNAStringSet(start=y, end=y+klength-1, seqs)
})

if(length(unique(fq.width)) == 1) {
    kcount <- vapply(kcount, function(y) length(unique(y)), integer(1))
} else {


kcount <- vapply(kcount, function(y) {
cuales <- grep("-", y)
if(length(cuales) == 0) 
    return(length(unique(y)))
else
    return(length(unique(y[-grep("-", y)])))
}, integer(1))
}
reldiv <- kcount/(5^klength) 
reldiv <- c(reldiv, rep(NA, klength-1)) 
names(reldiv) <- seq_along(reldiv)
kmerData <- data.frame(RelDiv = reldiv, 
Method = rep(c(1), 
each = length(reldiv)), 
Cycle = names(reldiv))
out$kmerData  <- kmerData


out

}


#' myPlot
#' @description Construction of diagnostic plots. The function depends
#' of the values created by plotObject
#' @return List with Highcharts plots
#' @keywords internal


myPlot <- function(isPaired, location, sampleSize,
kmerLength, theFile, maxFreq) {

plotList <- function(out, nplots, theFile, sampleSize) {

if(length(out) == 0) {
    out <- alist(none=, a=, b=, c=, d=, e=, f=,
            g=, h=, i=, j=,table=, oligofreq=)
for(i in seq_len(11)) {
    out[[i]] = shiny::HTML("<script></script>")
}
out[[12]] <- out[[13]] <-  data.frame()
return(out)
}

list(none = "<script></script>",
a = plotA(out, nplots, theFile, sampleSize),
b = plotB(out, nplots, theFile, sampleSize), 
c = plotC(out, nplots, theFile, sampleSize), 
d = plotD(out, nplots, theFile, sampleSize), 
e = plotE(out, nplots, theFile, sampleSize),
f = plotF(out, nplots, theFile, sampleSize), 
g = plotG(out, nplots, theFile, sampleSize),
h = plotH(out, nplots, theFile, sampleSize),
i = plotI(out, nplots, theFile, sampleSize),
j = plotJ(out, nplots, theFile, sampleSize),
table = out$dt, 
oligofreq = out$oligofreq)
}  


if(!isPaired) {
stream <- FastqSampler(location, sampleSize)
data <- yield(stream)
x <- list(plotList(plotObjects(data,  kmerLength, location,
                maxFreq, sampleSize), 
                nplots = 1, theFile, sampleSize))
close(stream)
} else {
streamL <- FastqSampler(location[1], sampleSize)
dataL <- yield(streamL)
streamR <- FastqSampler(location[2], sampleSize)
dataR <- yield(streamR)
x <- list(plotList(plotObjects(dataL, kmerLength, location[1], 
                    maxFreq, sampleSize),  nplots = 1, theFile, sampleSize),
                    plotList(plotObjects(dataR, kmerLength, location[2], 
maxFreq, sampleSize),  nplots = 2, theFile, sampleSize))
close(streamL)
close(streamR)
}
x
}


#' plotA
#' @return Per cycle quality  plot
#' @keywords internal


plotA <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
container <- "container1"
} else  {
container <- "container2"
}

theFile <- match.arg(theFile)

jsondata <- lapply(unname(split(x$bp_cycle[,-1],seq_len(nrow(x$bp_cycle)))),
                    function(y)paste(y, collapse = ", "))
jsondata <- paste("[", jsondata, "]", sep = "", collapse = ",\n")
jsondata <- paste0("[", jsondata, "]")

xAxis <- paste("[",  paste("'", x$bp_cycle[, 1], "'", sep = "", 
collapse = ", "), "]", sep ="")

ymax <- max(x$bp_cycle[,-1], na.rm = TRUE)

if(ymax < 40) { 
    ymax <- 40
}

ymin <- min(x$bp_cycle[,-1], na.rm = TRUE)

if(ymin > 0) { 
    ymin <- 0
}


paste0("<script type= 'text/javascript'>
$(function () {
$('#", container, "').highcharts({

chart: {
type: 'boxplot'
},

title: {
text: 'Per cycle quality - ", theFile, "'
},

legend: {
enabled: false
},

xAxis: { 
categories:", xAxis,",
title: {
text: 'Cycle'
}
},

yAxis: {
max:", ymax, ",
min:", ymin, ",
title: {
text: 'Observations'
}
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
plotOptions: {
boxplot: {
fillColor: '#95CEFF',
lineWidth: 0,
medianColor: '#2A2A93',
medianWidth: 3,
stemColor: '#A63400',
stemDashStyle: 'dot',
stemWidth: 1,
whiskerColor: '#3D9200',
whiskerLength: '50%',
whiskerWidth: 3
}
},

series: [{
name: 'Observations',
data:", jsondata,"}],
exporting: {
enabled: true
},

credits: {
enabled: false
}

});
});
</script>")
}


#' plotB
#' @return Per cycle mean base quality plot
#' @keywords internal


plotB <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {

if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)


xAxis <- paste("[",  paste("'", x$meanBaseQuality[, 3], "'", sep = "", 
collapse = ", "), "]", sep ="")
ymax <- round(max(x$meanBaseQuality[,1], na.rm = TRUE))
if(ymax < 40) { 
    ymax <- ymax  + 1
}

meanBaseQuality <- split(x$meanBaseQuality[, 1], x$meanBaseQuality[,2])
nombres <- names(meanBaseQuality)
jsondata <- lapply(unname(meanBaseQuality),  
                    function(x)paste(x, collapse = ", "))
jsondata <- paste("[", jsondata, "]", sep = "")
names(jsondata) <- nombres


paste0("<script type= 'text/javascript'>

$(function () {
$('#", container, "').highcharts({
title: {
x: -20, //center
text: 'Per cycle mean base quality - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
"},
yAxis: {
max:", ymax, ",
title: {
text: 'Quality'
},
plotLines: [{
value: 0,
width: 1,
color: '#808080'
}]
},
tooltip: {
pointFormat: '<span style=\"color:{series.color}\">{series.name}</span>:'+ 
'<b>{point.y}%</b><br/>'
},
legend: {
layout: 'vertical',
align: 'right',
verticalAlign: 'middle',
borderWidth: 0
},
credits: {
enabled: false
},
series: [{
name:",  paste0("\"",names(jsondata)[1],"\""), ",\n",
"data:", jsondata[[1]],"\n",
"}, {
name:",   paste0("\"",names(jsondata)[2],"\""), ",\n",
"data:", jsondata[[2]],"\n",
"}, {
name:",   paste0("\"",names(jsondata)[3],"\""), ",\n",
"data:", jsondata[[3]], "\n",
"}, {
name:",   paste0("\"",names(jsondata)[4],"\""), ",\n",
"data:", jsondata[[4]], "\n",
"}]
});
});
</script>")

}


#' plotC
#' @return Mean quality of reads distribution plot
#' @keywords internal

plotC <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)

x$meanQReads[, 2] <- round(x$meanQReads[, 2], 2) 
yrange <- range(x$meanQReads[, 2], na.rm = TRUE)
ymax <- min(yrange[2] + yrange/10, 100)

xAxis <- paste("[", paste("'", x$meanQReads[, 1], "'", 
collapse = ", ", sep = ""), "]", sep ="")

plotReads <- unname(x$meanQReads[, 2])
jsondata <- paste(plotReads, collapse = ", ")
jsondata <- paste("[", jsondata, "]", sep = "")


paste0("<script type= 'text/javascript'>
$(function () {
$('#", container, "').highcharts({
chart: {
type: 'column'
},
title: {
text: 'Distribution of mean quality of reads - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
",
crosshair: true,
title: {
text: 'Mean Quality'
}
},
yAxis: {
min: 0,
max:", ymax, ",
title: {
text: '% Reads'
}
},
tooltip: {
pointFormat: '<b>{point.y}%</b><br/>'
},
plotOptions: {
column: {
pointPadding: 0.2,
borderWidth: 0
}
},
legend: {
enabled: false,
},
credits: {
enabled: false
},
series: [{
data:", jsondata,"\n",
"}]
});
});
</script>")

}


#' plotD
#' @return  percent of reads with quality > threshold plot
#' @keywords internal

plotD <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)


jsondata <- paste("[", paste(x$qual_above, collapse = ", "), "]", sep = "")
xAxis <- paste("[",  paste("'", seq_len(40), "'", sep = "",
collapse = ", "), "]", sep ="")

ymin <- 0
ymax <- 100


paste0("<script type= 'text/javascript'>

$(function () {
$('#", container, "').highcharts({
title: {
x: -20, //center
text: 'Quality - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
"},
yAxis: {
max:", ymax, ",
min:", ymin, ",
title: {
text: '% reads'
},
plotLines: [{
value: 0,
width: 1,
color: '#808080'
}]
},
tooltip: {
pointFormat: '<b>{point.y}%</b><br/>'
},
legend: {
enabled: false,
},
credits: {
enabled: false
},
series: [{
name:",  paste0("\"",names(jsondata),"\""), ",\n",
"data:", jsondata,"\n",
"}]
});
});
</script>")

}

#'plotE
#' @return Per cycle base proportion plot
#' @keywords internal


plotE <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)


ymax <- 100
ymin <- 0

jsondata <- list(
'C' = paste("[", paste(x$base_content[1, ], 
collapse = ", "), "]", sep = ""),
'G' = paste("[", paste(x$base_content[2, ], 
collapse = ", "), "]", sep = ""),
'A' = paste("[", paste(x$base_content[3, ], 
collapse = ", "), "]", sep = ""),
'T' = paste("[", paste(x$base_content[4, ], 
collapse = ", "), "]", sep = ""),
'N' = paste("[", paste(x$base_content[5, ], 
collapse = ", "), "]", sep = "")
)


xAxis <- paste("[",  paste("'", names(x$base_content), "'", 
sep = "", collapse = ", "), "]", sep ="")


paste0("<script type= 'text/javascript'>
$(function () {
$('#", container, "').highcharts({
chart: {
type: 'column'
},
title: {
text: 'Per cycle base proportion (%) - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:",  xAxis, 
"},
yAxis: {
min: 0,
max: 100,
title: {
text: 'Base proportion (%)'
}
},
tooltip: {
pointFormat: '<span style=\"color:{series.color}\">{series.name}</span>:'+ 
'<b>{point.y}%</b><br/>',
shared: true
},
plotOptions: {
column: {
stacking: 'percent'
}
},
credits: {
enabled: false
},
series: [{
name:",  paste0("\"",names(jsondata)[1],"\""), ",\n",
"data:", jsondata[[1]],"\n",
"}, {
name:",   paste0("\"",names(jsondata)[2],"\""), ",\n",
"data:", jsondata[[2]],"\n",
"}, {
name:",   paste0("\"",names(jsondata)[3],"\""), ",\n",
"data:", jsondata[[3]], "\n",
"}, {
name:",   paste0("\"",names(jsondata)[4],"\""), ",\n",
"data:", jsondata[[4]], "\n",
"}, {
name:",   paste0("\"",names(jsondata)[5],"\""), ",\n",
"data:", jsondata[[5]], "\n",
"}]
});
});
</script>"
)

}


#'plotF
#'@return Per cycle base proportion plot (lineplot)
#' @keywords internal


plotF <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)

yrange <- range(x$base_content, na.rm = TRUE)
ymax <- min(yrange[2] + yrange[2]/10, 100)
ymin <- max(yrange[1] - yrange[1]/10, 0)

jsondata <- list(
'C' = paste("[", paste(x$base_content[1, ],
collapse = ", "), "]", sep = ""),
'G' = paste("[", paste(x$base_content[2, ], 
collapse = ", "), "]", sep = ""),
'A' = paste("[", paste(x$base_content[3, ], 
collapse = ", "), "]", sep = ""),
'T' = paste("[", paste(x$base_content[4, ], 
collapse = ", "), "]", sep = ""),
'N' = paste("[", paste(x$base_content[5, ], 
collapse = ", "), "]", sep = "")
)


xAxis <- paste("[",  paste("'", names(x$base_content), "'", sep = "", 
collapse = ", "), "]", sep ="")


paste0("<script type= 'text/javascript'>

$(function () {
$('#", container, "').highcharts({
title: {
x: -20, //center
text: 'Per cycle base proportion (%) - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
"},
yAxis: {
max:", ymax, ",
min:", ymin, ",
title: {
text: 'Base proportion (%)'
},
plotLines: [{
value: 0,
width: 1,
color: '#808080'
}]
},
tooltip: {
pointFormat: '<span style=\"color:{series.color}\">{series.name}</span>:'+ 
'<b>{point.y}%</b><br/>'
},
legend: {
layout: 'vertical',
align: 'right',
verticalAlign: 'middle',
borderWidth: 0
},
credits: {
enabled: false
},
series: [
{
name:",  paste0("\"",names(jsondata)[1],"\""), ",\n",
"data:", jsondata[[1]],"\n",
"}, 
{
name:",   paste0("\"",names(jsondata)[2],"\""), ",\n",
"data:", jsondata[[2]],"\n",
"},
{
name:",   paste0("\"",names(jsondata)[3],"\""), ",\n",
"data:", jsondata[[3]],"\n",
"},

{
name:",   paste0("\"",names(jsondata)[4],"\""), ",\n",
"data:", jsondata[[4]],"\n",
"},
{
name:",   paste0("\"",names(jsondata)[5],"\""), ",\n",
"data:", jsondata[[5]],"\n",

"}]
});
});
</script>")


}


#' plotG
#' @return CG content distribution plot
#' @keywords internal


plotG <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)


ydata <- 100 * round(x$GC$counts/sum(x$GC$counts), 2)

yrange <- range(ydata)
ymax <- min(yrange[2] + yrange/10, 100)

jsondata <- paste("[", paste(ydata, collapse = ", "), "]", sep = "")
xAxis <- paste("[",paste( "'", x$GC$breaks,"'", 
sep = "", collapse = ", "), "]", sep ="")


paste0("<script type= 'text/javascript'>
$(function () {
$('#", container, "').highcharts({
chart: {
type: 'column'
},
title: {
text: '% reads with a given % GC - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
",
crosshair: true,
title: {
text: '% GC'
}
},
yAxis: {
min: 0,
max:", ymax, ",
title: {
text: '% Reads'
}
},
tooltip: {
pointFormat: '<b>{point.y}%</b><br/>'
},
plotOptions: {
column: {
pointPadding: 0.2,
borderWidth: 0
}
},
legend: {
enabled: false,
},
credits: {
enabled: false
},
series: [{
data:", jsondata,"\n",
"}]
});
});
</script>")

}


#' plotH
#' @return Read length distribution
#' @keywords internal

plotH <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)


yrange <- range(x$length_distribution$ydata, na.rm = TRUE)
ymax <- min(yrange[2] + yrange/10, 100)

xAxis <- paste("[", paste("'", x$length_distribution$xdata, "'", 
collapse = ", ", sep = ""), "]", sep ="")

jsondata <- paste(x$length_distribution$ydata, collapse = ", ")
jsondata <- paste("[", jsondata, "]", sep = "")


paste0("<script type= 'text/javascript'>
$(function () {
$('#", container, "').highcharts({
chart: {
type: 'column'
},
title: {
text: 'Read length distribution - ", theFile, "' 
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
",
crosshair: true,
title: {
text: 'Length (bp)'
}
},
yAxis: {
min: 0,
max:", ymax, ",
title: {
text: '% Reads'
}
},
tooltip: {
headerFormat: '<span style=\"font-size:10px\">{point.key}</span><table>',
pointFormat: '<tr><td style=\"color:{series.color};padding:0\">{series.name}:'+
'</td> <td style=\"padding:0\"><b>{point.y:.1f} % </b></td></tr>',
footerFormat: '</table>',
shared: true,
useHTML: true
},
plotOptions: {
column: {
pointPadding: 0.2,
borderWidth: 0
}
},
legend: {
enabled: false,
},
credits: {
enabled: false
},
series: [{
data:", jsondata,"\n",
"}]
});
});
</script>")

}


#' plotI
#' @return Read ocurrence distribution plot
#' @keywords internal


plotI <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}
theFile <- match.arg(theFile)


yrange <- range(x$ocurrenceData[, 2], na.rm = TRUE)
ymax <- min(yrange[2] + yrange/10, 100)

xAxis <- paste("[", paste("'", x$ocurrenceData[, 1], "'", 
collapse = ", ", sep = ""), "]", sep ="")

ocurrenceData <- unname(x$ocurrenceData[,2])
jsondata <- paste(ocurrenceData, collapse = ", ")
jsondata <- paste("[", jsondata, "]", sep = "")



paste0("<script type= 'text/javascript'>
$(function () {
$('#", container, "').highcharts({
chart: {
type: 'column'
},
title: {
text: 'Read occurrence distribution - ", theFile, "' 
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
categories:", xAxis,
",
crosshair: true,
title: {
text: 'Read Ocurrence'
}
},
yAxis: {
min: 0,
max:", ymax, ",
title: {
text: '% Reads'
}
},
tooltip: {
headerFormat: '<span style=\"font-size:10px\">{point.key}</span><table>',
pointFormat: '<tr><td style=\"color:{series.color};padding:0\">{series.name}:'+
'</td> <td style=\"padding:0\"><b>{point.y:.1f} % </b></td></tr>',
footerFormat: '</table>',
shared: true,
useHTML: true
},
plotOptions: {
column: {
pointPadding: 0.2,
borderWidth: 0
}
},
legend: {
enabled: false,
},
credits: {
enabled: false
},
series: [{
data:", jsondata,"\n",
"}]
});
});
</script>")

}



#' plotJ
#' @return Relative kmer diversity plot
#' @keywords internal

plotJ <- function(x, nplots = 1, theFile = c("input", "output"), sampleSize) {
if(nplots == 1) {
    container <- "container1"
} else  {
    container <- "container2"
}

theFile <- match.arg(theFile)


xAxis <- paste("[",  paste("'", x$kmerData[, 3], "'", sep = "", 
collapse = ", "), "]", sep ="")
kmerData <- split(x$kmerData[, 1], x$kmerData[, 2])
kmerData[[1]][is.na(kmerData[[1]])] <- NaN
nombres <- names(kmerData)
jsondata <- lapply(unname(kmerData), function(y)paste(y, collapse = ", "))
jsondata <- paste("[", jsondata, "]", sep = "")

names(jsondata) <- ""
ylab <- paste("k", x$klength, "-mer diversity", sep="")

paste0("<script type= 'text/javascript'>

$(function () {
$('#", container, "').highcharts({
title: {
x: -20, //center
text: 'Relative k-mer diversity (observed/expected k-mers) - ", theFile, "'
},
subtitle: {
text: 'sample size: ", sampleSize, "',
x: -20
},
xAxis: {
title: {
text:'Cycle' 
},
categories:", xAxis,
"},
yAxis: {
title: {
text:'", ylab,"' 
},
plotLines: [{
value: 0,
width: 1,
color: '#808080'
}]
},
//tooltip: {
//  valueSuffix: 'kmer-diversity'
//},
legend: {
enabled: false,
},
credits: {
enabled: false
},
series: [{
name:",  paste0("\'",names(jsondata),"\'"), ",\n",
"data:", jsondata,"\n",
"}]
});
});
</script>")

}

---
title: "An Introduction to FastqCleaner"
author:
- affiliation: IIB-INTECH - UNSAM
  email: learoser@gmail.com
  name: Leandro Roser
- affiliation: IIB-INTECH - UNSAM
  name: Fernán Agüero
- affiliation: IIB-INTECH - UNSAM
  name: Daniel Sánchez
date: "Feb 15, 2020"
revised: "`r Sys.Date()`"
package: "`r library(BiocStyle); pkg_ver('FastqCleaner')`"
output:
  BiocStyle::html_document:
    toc_float: true
    code_folding: show
  BiocStyle::pdf_document: default
df_print: paged
abstract: |
    `r Biocpkg("FastqCleaner")` is an interactive application 
    for both quality control and pre-processing of FASTQ files. 
    The interface, based on a Bioconductor back-end, 
    provides diagnostic information for the input and output 
    data and a series of filtering and trimming operations in 
    an interactive framework.
vignette: |
    %\VignetteIndexEntry{An Introduction to FastqCleaner}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::rmarkdown}
chunk_output_type: console
---

Launching the application
===========================

The interactive application can be launched in R with the following command:

```{r eval=FALSE, include=TRUE}
library('FastqCleaner')
launch_fqc()
```

As an alternative method, an RStudio addin 
(RStudio version 0.99.878 or higher required) installed with the package 
can be found in the Addins menu (Figure 1). 
This button allows the direct launch of the application with a single click.

![](images/Fig1.png)
**Figure 1**: addin of the app in RStudio (RStudio version >= 0.99.878 required)

&nbsp;


Description of the application
===============================

The application contains three main panels, as described below. 

First panel
---------------

The first panel includes two elements: a dashboard for selection 
of trimming and  filtering operations, and a menu for selection 
of the input file/s (Fig. 2). 


![](images/Fig2.png)

**Figure 2**: Panel 1, with each compontent indicated with a number.
See explanation for each element below

&nbsp;

### Selecting operations

The "operations menu" (Fig. 2, elements 1 to 8) shows 
the available operations for file processing:

1.  **Remove by N(s)**: removes sequences with a number of Ns 
(non identified bases) above a selected threshold value

2.  **Remove low complexity sequences**: remove sequences with a value of 
complexity above a threshold value

3.  **Remove adapters**: removes adapters and partial adapters. 
Adapter sequences from both ends of single or paired read reads 
can be selected. Sequences can be reverse-complemented before processing. 
The program also allows to consider indels 
and/or anchored adapters.

4.  **Filter by average quality**: computes the average quality of sequences 
and removes those with a value below a given threshold

5.  **Trim low quality 3’ tails**: removes the 3’ tails of sequences that
are below a given threshold

6.  **Trim 3’ or 5’ by a fixed number**: removes a fixed number of bases 
from the 3’ and/or 5’ ends in the complete set of sequences

7.  **Filter sequences by length**: removes all the sequences with a number 
of bases below a threshold value

8.  **Remove duplicated sequences**: removes duplicated reads, conserving
only one copy of each sequence present in the file


### Loading files

The "file selection menu" (Fig. 2, elements 9 to 17) 
contains options to handle the input file (type of file, file selection), 
buttons to run, clear and reset the aplication, and the “advanced” submenu:


9.  **Single-end reads / paired-end reads**: type of input files

10.  **“FILE” button**: to select an input file

11.  **“RUN!” button**: to run the program

12.  **Output format**: to select whether the output file should be compressed 
(.gz) or not

13.  **“CLEAR” button**: to clear the configuration of the operations menu 
that have been selected in the first panel, but keeping the input file(s)

14.  **“RESET” button**: to restart the application, removing the 
input file(s) and the selected configurations

15.  **Selection notificator**:  information of the path of the selected file/s

16.  **Encoding notificator**: information of the input file/s encoding

17.  **Advanced options button**: to select a custom encoding and set 
the number of reads included in each chunk for processing, as described below


### Advanced options

The "advanced options submenu" (Fig. 3) allows to customize some fine aspects 
of the trimming and filtering process:


![](images/Fig3.png)

**Figure 3**: Advanced options submenu

&nbsp;


18.  **Encoding menu**: in addition to the default approach used by the 
program (auto-detection of file encoding), users can select a standard encoding 
from a list

19.  **Chunk size**: the program takes this number of reads at random from
the file (default: 1000000), for encoding detection


&nbsp;


Second panel
----------------


The second panel (“file operations” panel, Fig. 4) shows the operations 
that were sucessfuly performed on the input file after running the program.


![](images/Fig4.png)

**Figure 4**: File operations panel, with its elements

&nbsp;  


The panel contains the following elements: 

20.  **Files location**: location of input and output files

21.  **Operations performed**: operations perfomed on the input file.
Each individual display indicates the number of reads that
passed the corresponding filter


&nbsp;


Third panel
---------------


The third panel ( “live results” panel, Fig. 5) shows 
interactive diagnostics plots for both input and output files. 
The program takes a random sample of reads for 
construction of the plots (default: 10000 reads).


![](images/Fig5.png)

**Figure 5**: Live results panel


&nbsp;


The panel includes the following options in the menu located on the left:

22. **Sample size**: the sample size used for construction of the plots. 
Default: 10000 reads

23. **Input / output**: show diagnostics plots for input or output files?

24. **Diagnostics plots**: the plot to be shown, that can be one of the 
following:


    *   _Per cycle quality_: quality plots across reads for each cycle 
    (i.e., sequence position)

    *   _Per cycle mean quality_: average quality across reads per base, 
    for each cycle (i.e., sequence position)

    *   _Mean quality distribution_: Quality distribution, using for 
    the construction of the histogram the mean quality of each read

    *   _% reads with Phred scores > threshold_: % of reads with all 
    the quality values > threshold

    *   _Per cycle base proportion_: Proportion of each base 
    (average across reads) in each cycle. It also shows the proporion of N’s

    *   _CG content_: % CG and % AT (average across reads) for each cycle

    *   _CG content distribution over all reads_: histogram for % reads with 
    a given % CG

    *   _Read length distribution_: % reads vs read length (bp)

    *   _Read ocurrence distribution_: % reads that ocurr at different 
    frequencies values in the file. The plot also includes a table

    *   _Relative k-mer diversity_:  unique k-mers / all posible 
    kmers for each cycle


25. **Select k-mer size**: k-mer size for the k-mers frequency plot

26. **Top sequences in duplication level analysis**: a list of duplicated 
sequences, ordered from high to low duplication level,
can be desplegated from the "read ocurrence distribution" plot. 
The number selected here indicates how many sequences should be shown.
Note that the frequency of reads are relative to the sample size selected 
(i.e., fold-times in relation to those reads present only once in the sample)

27. **Plot panel**

&nbsp;&nbsp;


A worked example: FASTQ processing in a nutshell
==================================================

A sample FASTQ (gz-compressed) file 'example.fastq.gz' can 
be downloaded with the following command in R:

```{r eval=FALSE, include=TRUE}
download.file("https://goo.gl/hb4Kr9", "example_fastq.gz")
```

A direct download is provided in <a href="https://goo.gl/hb4Kr9"> 
this link </a>.
&nbsp;

A tipical `r Biocpkg("FastqCleaner")` workflow starts with the input file/s 
upload (Fig. 6).


![](images/Fig6.png)

**Figure 6**: File input menu. The example shows a single-end reads case 
(sample file 'example.fastq.gz'). For paired-end reads, the selection of the
corresponding library type generates an additional button to upload the 
second file.

&nbsp;  

The file encoding is automatically detected by the program, but it can also be 
manually specified in the advanced submenu (Fig. 7). This menu also offers an
option to customize the chunk size used for processing.


![](images/Fig7.png)

**Figure 7**: Advanced submanu

&nbsp;

Next, the operations to be performed on the input file are selected from the
operations menu (Fig. 8).

![](images/Fig8.png)

**Figure 8**: Selection of operations. A dialog box shows the input expected 
for the program. To use a filter, the “Use filter?” checkbox must
be checked. A filter in use is indicated with a checkmark in the filter box 

&nbsp;  

The program then starts to run after pressing the “RUN!” button (Fig. 9).


![](images/Fig9.png)

**Figure 9**: "RUN!" button action

&nbsp;    


Post-processing results are shown in the second panel (Fig. 10).  


![](images/Fig10.png)

**Figure 10**: Second panel of the app, showing the operations performed and 
the paths of the input and output files

&nbsp;  


The type of plot to be displayed and the options for the construction of the
plot are available in the third panel (Fig. 11). This panel also show the
selected plot/s.


![](images/Fig11.png)

**Figure 11**: Third panel, showing as example a “CG” content plot. 
for the output file

&nbsp;    

To clean the operations, for example to run a different
configuration, the "CLEAN" i(Fig. 11) must be pressed. The "RESET" button 
(Fig. 11) restarts the interface.


&nbsp;    

Additional help can be found in the "help" button located at the top-right
of the app (Fig. 12).


![](images/Fig12.png)

**Figure 12**: help button. A webpage with information will be open

&nbsp;


Advanced use of the package
=============================

`r Biocpkg("FastqCleaner")` separates the interface from the implementation. 
In consequence, the processing functions of the package can be used 
as standard functions from the command line. 
Most of the functions make intensive 
use of `r Biocpkg("Biostrings")` and `r Biocpkg("ShortRead")`. Trimming
and filtering is performed on ShortReadQ objects. A complete documentation 
for the functions is available in [this link](docs/reference/index.html)

The functions included in the package are described in the following section.

&nbsp;

Main functions
------------------

*   **adapter_filter**

Based on the `r Biocpkg("Biostrings")` trimLRPatterns functions. 
It can remove adapters and partial adapters
from the 3’ and 5’ sequence ends. Adapters can be anchored or not. 
When indels are allowed, the method is based on the “edit distance” of
the sequences.


```{r eval=TRUE, echo=TRUE}
### Examples
```
```{r include=FALSE}
require("Biostrings")
require("ShortRead")
require("FastqCleaner")
set.seed(10)
```

```{r eval = FALSE}
require("Biostrings")
require("ShortRead")
require("FastqCleaner")
```

```{r eval=TRUE, echo=TRUE}

# create sequences
set.seed(15) 
# nota that the use of set.seed before the call to the 
# random generators allows reproducibility of the
# examples

input <- random_seq(6, 43)
input

# create qualities of width 50
set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 50, 
encod = "Sanger")

# create names
input_names <- seq_names(length(input))


### FULL ADAPTER IN 3'
adapter <- "ATCGACT"

# Create sequences with adapter
my_seqs <- paste0(input, adapter)
my_seqs <- DNAStringSet(my_seqs)
my_seqs

# create ShortReadQ object
my_read <- ShortReadQ(sread = my_seqs, quality = input_q, id = input_names)

# trim adapter
filtered <- adapter_filter(my_read, Lpattern = adapter)
sread(filtered)

### PARTIAL ADAPTER IN 5'
adapter <- "ATCGACT"
subadapter <- subseq(adapter, 1, 4)

# Create sequences with adapter
my_seqs <- paste0(input, subadapter)
my_seqs <- DNAStringSet(my_seqs)
my_seqs

# create ShortReadQ object
my_read <- ShortReadQ(sread = my_seqs, quality = subseq(input_q, 1, 47), 
id = input_names)

# trim adapter
filtered <- adapter_filter(my_read, Rpattern = adapter)
sread(filtered)

```



[Documentation of the function](docs/reference/adapter_filter.html)

&nbsp;

*   **complex_filter**

Removes low complexity sequences, computing the entropy with the 
dinucleotide frequency: 
$$H_i = -\sum d_i * log_2(d_i)$$

where: $d_i = D_i/ \sum_i^n D_i$ represents the frequency of 
dinucleotides of the sequence $i$ relative to the frequency 
in the whole pool of sequences. 

The relation $H_i/H_r$ between $H_i$ and a reference entropy value $H_r$ 
is computed, and the obtained relations are compared with a given complexity 
threshold.  By default the program uses a reference entropy 
of 3.908, that corresponds to the entropy of the human genome in bits, 
and a complexity threshold of 0.5.


```{r}

# create  sequences of different width
set.seed(10)
input <- lapply(c(0, 6, 10, 16, 20, 26, 30, 36, 40), 
            function(x) random_seq(1, x))


# create repetitive "CG" sequences with length adequante 
# for a total length input +  CG = 40

CG <- lapply(c(20, 17, 15, 12, 10, 7, 5, 2, 0), 
            function(x) paste(rep("CG", x), collapse = ""))

# concatenate input and CG
input  <- mapply("paste", input, CG, sep = "")
input <- DNAStringSet(input)
input

# plot relative entropy (E, Shannon 1948)
H_plot <- function(x, H_max = 3.908135) {
    freq <- dinucleotideFrequency(x)
    freq  <- freq /rowSums(freq)
    H <- -rowSums(freq  * log2(freq), na.rm = TRUE)
    plot(H/H_max, type="l", xlab = "Sequence", ylab= "E")
    points(H/H_max, col = "#1a81c2", pch = 16, cex = 2)
}

H_plot(input)
```

**Figure 13**: Relative entropy plot for the sequences before the operation

```{r}
# create qualities of widths 40
set.seed(10)
input_q <- random_qual(c(30,40), slength = 9, swidth = 40, 
    encod = "Sanger")

# create names
input_names <- seq_names(9)


# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# apply the filter, 
filtered <- complex_filter(my_read)
sread(filtered)

H_plot(sread(filtered))
```
**Figure 14**: Relative entropy plot for the sequences after the operation


[Documentation of the function](docs/reference/complex_filter.html)

&nbsp;

*   **fixed_filter**

Removes the specified number of bases from 3’ or 5’.

```{r}

# create sequences, qualities and names of width 20
set.seed(10)
input <- random_seq(6, 20)
input

set.seed(10)
input_q <- random_qual(c(30,40), slength = 6, swidth = 20, 
    encod = "Sanger")

input_names <- seq_names(6)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# apply the filter 
filtered3 <- fixed_filter(my_read, trim5 = 5)
sread(filtered3)

filtered5 <- fixed_filter(my_read, trim3 = 5)
sread(filtered5)

filtered3and5 <- fixed_filter(my_read, trim3 = 10, trim5 = 5)
sread(filtered3and5)

```


[Documentation of the function](docs/reference/fixed_filter.html)

&nbsp;

*   **length_filter**

Removes sequences with a length lower than
minimum threshold value or/and higher than a maximum threshold value.

```{r}

# create  ShortReadQ object width widths between 1 and 60
set.seed(10)
input <- random_length(10, widths = 1:60)
sread(input)

# apply the filter, removing sequences with length < 5 or length> 30
filtered <- length_filter(input, rm.min = 5, rm.max = 30)
sread(filtered)
```


[Documentation of the function](docs/reference/length_filter.html)

&nbsp;

*   **n_filter**

Wrapper of the `r Biocpkg("ShortRead")` nFilter function. Removes 
all those sequences with a number of N’s > a given threshold. 

```{r}
# create 10 sequences of width 20
set.seed(10)
input <- random_seq(10, 20)
input

# inject N's
set.seed(10)
input <- inject_letter_random(input, how_many_seqs = 1:5,
    how_many = 1:10)
input

#'  
hist(letterFrequency(input, "N"), breaks = 0:10, 
    main  = "Ns Frequency", xlab = "# Ns",
    col = "#1a81c2")
```
**Figure 15**: N's histogram for the sequences before the filtering operation

```{r}

# Create qualities, names and ShortReadQ object
set.seed(10)
input_q <- random_qual(10, 20)
input_names <- seq_names(10)
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# Apply the filter 
filtered <- n_filter(my_read, rm.N = 3)
sread(filtered)
hist(letterFrequency(sread(filtered), "N"), 
    main = "Ns distribution", xlab = "",
    col = "#1a81c2")
```

**Figure 16**: N's histogram for the sequences after the filtering operation

[Documentation of the function](docs/reference/n_filter.html)

&nbsp;

*   **qmean_filter**

Removes those sequences with quality < a give threshold.

```{r}

# create 30 sequences of width 20, 15 with low quality and 15 with high quality
set.seed(10)
input <- random_seq(30, 20)

set.seed(10)
my_qual_H <- random_qual(c(30,40), slength = 15, swidth = 20,
    encod = "Sanger")

set.seed(10)
my_qual_L <- random_qual(c(5,30), slength = 15, swidth = 20, 
    encod = "Sanger")
input_q<- c(my_qual_H, my_qual_L)

input_names <- seq_names(30)
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# Plot of average qualities
qual_plot <- function(x, cutoff) {
q <- alphabetScore(x) / width(x)
plot(q, type="l", xlab = "Sequence", ylab= "Average quality", ylim = c(0, 40))
points(q, col = "#1a81c2", pch = 16, cex = 2)
lines(seq_along(q), rep(cutoff, length(q)), type="l", col = "red", lty=2)
text(length(q), cutoff+2, cutoff)
}

#' Average qualities before
qual_plot(my_read, cutoff = 30)
```

**Figure 17**:  Average qualities before the filtering operation

```{r}
# Apply the filter
filtered <- qmean_filter(my_read, minq = 30)

# Average qualities after
qual_plot(filtered, cutoff = 30)

```
**Figure 18**:  Average qualities after the filtering operation

[Documentation of the function](docs/reference/qmean_filter.html)

&nbsp;

*   **seq_filter**

Removes sequences that match those passed as argument.


```{r}

# Generate random sequences
set.seed(10)
input <- random_length(30, 3:7)

# Remove sequences that contain the following patterns:
rm.seq  = c("TGGTC", "CGGT", "GTTCT", "ATA")
match_before <- unlist(lapply(rm.seq, function(x) grep(x, 
as.character(sread(input)))))
match_before

filtered <- seq_filter(input,rm.seq =  rm.seq)

# Verify that matching sequences were removed
match_after <- unlist(lapply(rm.seq, function(x) {
                grep(x, as.character(sread(filtered)))}))
match_after
```

[Documentation of the function](docs/reference/seq_filter.html)

&nbsp;

*   **trim3q_filter**

Removes from the 3’ ends in-tandem nucleotides with a quality 
< a threshold value.


```{r}

# Create 6 sequences of width 20
set.seed(10)
input <- random_seq(6, 20)
input

# Create Phred+33 qualities of width 15 and paste to qualities of length 
# 5 used for the tails.
# for three of the sequences, put low qualities in tails
set.seed(10)
my_qual <- random_qual(c(30,40), slength = 6, swidth = 15, 
    encod = "Sanger")
set.seed(10)
tails <-   random_qual(c(30,40), slength = 6, swidth = 5, 
    encod = "Sanger")

# Low quality tails in sequences 2, 3 & 4
set.seed(10)
tails[2:4] <- random_qual(c(3, 20), slength = 3, swidth = 5,
    encod = "Sanger")
my_qual <- paste0(my_qual, tails)
input_q <- BStringSet(my_qual)
input_q

# Watch qualities before filtering
as.matrix(PhredQuality(input_q))

# Create names and ShortReadQ object
input_names <- seq_names(6)
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# Apply the filter 
filtered <- trim3q_filter(my_read, rm.3qual = 28)
sread(filtered)

```


[Documentation of the function](docs/reference/trim3q_filter.html)

&nbsp;

*   **unique_filter**

Wrapper of the `r Biocpkg("ShortRead")` occurrenceFilter function. 
that removes duplicated sequences.

```{r}

# Create duplicated sequences
s <- random_seq(10, 10)
s <- sample(s, 30, replace = TRUE)

# Create a ShortReadQ object
q <- random_qual(30, 10)
n <- seq_names(30)
my_read <- ShortReadQ(sread = s, quality = q, id = n)

# Check presence of duplicates
isUnique(as.character(sread(my_read)))

# Apply the filter
filtered <- unique_filter(my_read)
isUnique(as.character(sread(filtered)))
```


[Documentation of the function](docs/reference/unique_filter.html)

&nbsp;



Auxiliary functions
--------------------

*   **random_seq**

Create a vector of random sequences, for a set of specificied parameters.

[Documentation of the function](docs/reference/random_seq.html)

&nbsp;

*   **random_qual**

Create a vector of random qualities for a given encoding and a set
of specified parameters.

[Documentation of the function](docs/reference/random_qual.html)


&nbsp;


*   **seq_names**

Create a vector of names for a set of sequences.

[Documentation of the function](docs/reference/seq_names.html)

&nbsp;

*   **random_length**

Create a set of sequences with random lengths.

[Documentation of the function](docs/reference/seq_names.html)

&nbsp;

*   **inject_letter_random**

Inject a character (e.g., 'N') at random positions, given a set of parameters.


[Documentation of the function](docs/reference/inject_letter_random.html)

&nbsp;

*   **check_encoding**

The function allows to check quality encoding. It detects encodings with
the following formats:

Format       | Expected range
-------------|------------------
Sanger       |     [0, 40]  
Illumina 1.8 |     [0, 41]  
Illumina 1.5 |     [0, 40]  
Illumina 1.3 |     [3, 40]  
Solexa       |     [-5, 40]

[Documentation of the function](docs/reference/check_encoding.html)

&nbsp;


Contact information
=====================

Mantainer: Leandro Roser - [learoser@gmail.com](mailto:learoser@gmail.com)  

&nbsp;

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotG}
\alias{plotG}
\title{plotG}
\usage{
plotG(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
CG content distribution plot
}
\description{
plotG
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server_functions.R
\name{outputClean_}
\alias{outputClean_}
\title{outputClean_}
\usage{
outputClean_(.myFile, .lengthWidthVec, my_envir)
}
\value{
Vector with chunks length and width  information
}
\description{
outputClean_
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{random_seq}
\alias{random_seq}
\title{Create random sequences}
\usage{
random_seq(
  slength,
  swidth,
  nuc = c("DNA", "RNA"),
  prob = c(0.25, 0.25, 0.25, 0.25)
)
}
\arguments{
\item{slength}{Number of sequences}

\item{swidth}{Width of the sequences}

\item{nuc}{Create sequences of DNA (nucleotides = c('A', 'C', 'G', 'T')) or
RNA (nucleotides = c('A, 'C', 'G', 'U'))?. Default: 'DNA'}

\item{prob}{A vector of four probability values used 
to set the frequency of the nucleotides 'A', 'C', 'G', 'T', for DNA,
or 'A', 'C', 'G', 'U', for RNA. For example = c(0.25, 0.25, 0.5, 0).
Default is = c(0.25, 0.25, 0.25, 0.25) (equiprobability for the 4 bases).
If the sum of the probabilities is > 1, the values will be nomalized 
to the range [0, 1].}
}
\value{
\code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
}
\description{
Create a
\code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
with random sequences
}
\examples{

# For reproducible examples, make a call to set.seed before 
# running each random function

set.seed(10)
s1 <- random_seq(slength = 10, swidth = 20)
s1

set.seed(10)
s2 <- random_seq(slength = 10, swidth = 20, 
prob = c(0.6, 0.1, 0.3, 0))
s2

}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n_filter.R
\name{n_filter}
\alias{n_filter}
\title{Remove sequences with non-identified bases (Ns) from a ShortReadQ object}
\usage{
n_filter(input, rm.N)
}
\arguments{
\item{input}{\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object}

\item{rm.N}{Threshold value of N's to remove a sequence from the output 
(sequences with number of Ns > threshold are removed)
For example, if rm.N is 3, all the sequences 
with a number of Ns > 3 (Ns >= 4) will be removed}
}
\value{
Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
object
}
\description{
This program is a wrapper to
\code{\link[ShortRead:srFilter]{nFilter}}. 
It removes the sequences with a number of N's above 
a threshold value 'rm.N'. 
All the sequences with a number of N > rm.N (N >= rm.N) will be removed
}
\examples{

require('Biostrings')
require('ShortRead')

# create 6 sequences of width 20
set.seed(10)
input <- random_seq(50, 20)

# inject N's
set.seed(10)
input <- inject_letter_random(input, how_many_seqs = 1:30, 
how_many = 1:10)

input <- DNAStringSet(input)


# watch the N's frequency
hist(letterFrequency(input, 'N'), breaks = 0:10, 
main  = 'Ns Frequency', xlab = '# Ns')

# create qualities of width 20
set.seed(10) 
input_q <- random_qual(50, 20)

# create names
input_names <- seq_names(50)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# apply the filter 
filtered <- n_filter(my_read, rm.N = 3)

# watch the filtered sequences
sread(filtered)

# watch the N's frequency
hist(letterFrequency(sread(filtered), 'N'), 
main = 'Ns distribution', xlab = '')

}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{seq_names}
\alias{seq_names}
\title{Create sequences names}
\usage{
seq_names(n, base_name = "s", sep = "_")
}
\arguments{
\item{n}{Number of reads}

\item{base_name}{Base name for strings}

\item{sep}{Character separing base names and the read number. Default: '_}
}
\value{
\code{\link[Biostrings:XStringSet-class]{BStringSet}} object
}
\description{
Create \code{\link[Biostrings:XStringSet-class]{BStringSet}} 
object with names
}
\examples{
snames <- seq_names(10)
snames
snames2 <- seq_names(10, base_name = 's', sep = '.')
snames2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server_functions.R
\name{messageFun_}
\alias{messageFun_}
\title{messageFun_}
\usage{
messageFun_(.who, .chunck, .which_read, my_envir)
}
\value{
Changes the state of reactive vector, without return
}
\description{
messageFun_
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotE}
\alias{plotE}
\title{plotE}
\usage{
plotE(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Per cycle base proportion plot
}
\description{
plotE
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trim3q_filter.R
\name{trim3q_filter}
\alias{trim3q_filter}
\title{Filter sequences with low quality in 3' tails}
\usage{
trim3q_filter(
  input,
  rm.3qual,
  q_format = NULL,
  check.encod = TRUE,
  remove_zero = TRUE
)
}
\arguments{
\item{input}{\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object}

\item{rm.3qual}{Quality threshold for 3' tails}

\item{q_format}{Quality format used for the file, 
as returned by check_encoding}

\item{check.encod}{Check the encoding of the sequence? This argument 
is incompatible with q_format. Default TRUE}

\item{remove_zero}{Remove zero-length sequences?}
}
\value{
Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
object
}
\description{
The program removes from the 3' tails of the sequences 
a set of nucleotides showing a quality < a threshold value in a 
ShortReadQ object
}
\examples{

require('Biostrings')
require('ShortRead')

# create 6 sequences of width 20
set.seed(10)
input <- random_seq(6, 20)

# create qualities of width 15 and paste to qualities
# of length 5 used for the tails.
# for two of the sequences, put low qualities in tails

set.seed(10)
my_qual <- random_qual(c(30,40), slength = 6, swidth = 15, 
encod = 'Sanger')

set.seed(10)
tails <-   random_qual(c(30,40), slength = 6, swidth = 5, 
 encod = 'Sanger')
 
set.seed(10)
tails[2:3] <- random_qual(c(3, 20), slength = 2, 
swidth = 5,  encod = 'Sanger')
my_qual <- paste0(my_qual, tails)
input_q <- BStringSet(my_qual)
# create names
input_names <- seq_names(6)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input,
quality = input_q, id = input_names)

# apply the filter 
filtered <- trim3q_filter(my_read, rm.3qual = 28)

# look at the trimmed sequences
sread(filtered)
}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server_functions.R
\name{check_onclick_}
\alias{check_onclick_}
\title{check onclick}
\usage{
check_onclick_(.menu_react, .butt_number, my_envir)
}
\value{
Change value of reactive output, without return
}
\description{
Function to put a tickmark on click
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{random_length}
\alias{random_length}
\title{Create a named object with random sequences and qualities}
\usage{
random_length(
  n,
  widths,
  random_widths = TRUE,
  replace = TRUE,
  len_prob = NULL,
  seq_prob = c(0.25, 0.25, 0.25, 0.25),
  q_prob = NULL,
  nuc = c("DNA", "RNA"),
  qual = NULL,
  encod = c("Sanger", "Illumina1.8", "Illumina1.5", "Illumina1.3", "Solexa"),
  base_name = "s",
  sep = "_"
)
}
\arguments{
\item{n}{number of sequences}

\item{widths}{width of the sequences}

\item{random_widths}{width must be picked at random from the 
passed parameter 'widths', considering the value
as an interval where any integer can be picked. Default TRUE. 
Otherwise, widths are picked only from the vector passed.}

\item{replace}{sample widths with replacement? Default TRUE.}

\item{len_prob}{vector with probabilities for each width value. 
Default NULL (equiprobability)}

\item{seq_prob}{a vector of four probabilities values to set 
the frequency of the nucleotides 'A', 'C', 'G', 'T', for DNA,
or 'A', 'C', 'G', 'U', for RNA. For example = c(0.25, 0.25, 0.5, 0).
Default is = c(0.25, 0.25, 0.25, 0.25) (equiprobability for the 4 bases).
If the sum of the probabilities is > 1, the values will be nomalized 
to the range [0, 1].}

\item{q_prob}{a vector of range = range(qual), with probabilities to set 
the frequency of each quality value. Default is equiprobability.
If the sum of the probabilities is > 1, the values will be nomalized 
to the range [0, 1].}

\item{nuc}{create sequences of DNA (nucleotides = c('A', 'C', 'G', 'T')) or
RNA (nucleotides = c('A, 'C', 'G', 'U'))?. Default: 'DNA'}

\item{qual}{quality range for the sequences. It must be a range included 
in the selected encoding:

'Sanger' = [0, 40]

'Illumina1.8' = [0, 41]

'Illumina1.5' = [0, 40]

'Illumina1.3' = [3, 40]

'Solexa' = [-5, 40]

example: for a range from 20 to 30 in Sanger encoding, 
pass the argument = c(20, 30)}

\item{encod}{sequence encoding}

\item{base_name}{Base name for strings}

\item{sep}{Character separing base names and the read number. Default: '_'}
}
\value{
\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
}
\description{
Create a \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}}
object with random sequences and qualities
}
\examples{

# For reproducible examples, make a call to set.seed before 
# running each random function

set.seed(10)
s1 <- random_seq(slength = 10, swidth = 20)
s1

set.seed(10)
s2 <- random_seq(slength = 10, swidth = 20, 
prob = c(0.6, 0.1, 0.3, 0))
s2

}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotC}
\alias{plotC}
\title{plotC}
\usage{
plotC(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Mean quality of reads distribution plot
}
\description{
plotC
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unique_filter.R
\name{unique_filter}
\alias{unique_filter}
\title{Remove duplicated sequences in a FASTQ file}
\usage{
unique_filter(input)
}
\arguments{
\item{input}{\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object}
}
\value{
Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object
}
\description{
This program is a wrapper to  
\code{\link[ShortRead:srFilter]{occurrenceFilter}}. 
It removes the duplicated sequences of a FASTQ file.
}
\examples{

require('Biostrings')
require('ShortRead')

set.seed(10)
s <- random_seq(10, 10)
s <- sample(s, 30, replace = TRUE)
q <- random_qual(30, 10)
n <- seq_names(30)

my_read <- ShortReadQ(sread = s, quality = q, id = n)

# check presence of duplicates
isUnique(as.character(sread(my_read)))

# apply the filter
filtered <- unique_filter(my_read)

isUnique(as.character(sread(filtered)))
}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotH}
\alias{plotH}
\title{plotH}
\usage{
plotH(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Read length distribution
}
\description{
plotH
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotA}
\alias{plotA}
\title{plotA}
\usage{
plotA(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Per cycle quality  plot
}
\description{
plotA
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{myPlot}
\alias{myPlot}
\title{myPlot}
\usage{
myPlot(isPaired, location, sampleSize, kmerLength, theFile, maxFreq)
}
\value{
List with Highcharts plots
}
\description{
Construction of diagnostic plots. The function depends
of the values created by plotObject
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxiliar.R
\name{check_encoding}
\alias{check_encoding}
\title{Check quality encoding}
\usage{
check_encoding(x = NULL, custom = NULL)
}
\arguments{
\item{x}{Quality values}

\item{custom}{custom encoding from the following:

'Sanger' --------> expected range: [0, 40]

'Illumina1.8' --------> expected range: [0, 41]

'Illumina1.5' --------> expected range: [0, 40]

'Illumina1.3' --------> expected range: [3, 40]

'Solexa' --------> expected range: [-5, 40]}
}
\value{
List with encoding information
}
\description{
Check quality encoding
}
\examples{

require(Biostrings)

x <- list(PhredQuality(0:40), SolexaQuality(-5:40), IlluminaQuality(3:40))
x <- lapply(x, function(i)utf8ToInt(as.character(i)[1]))
lapply(x, check_encoding)
 
SolexaQuality(0:40)
IlluminaQuality(0:40)
}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/length_filter.R
\name{length_filter}
\alias{length_filter}
\title{Filter sequences of a FASTQ file by length}
\usage{
length_filter(input, rm.min = NA, rm.max = NA)
}
\arguments{
\item{input}{\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object}

\item{rm.min}{Threshold value for the minimun number of bases}

\item{rm.max}{Threshold value for the  maximum number of bases}
}
\value{
Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
object
}
\description{
The program removes from a ShortReadQ object those sequences
with a length lower than rm.min or/and higher than rm.max
}
\examples{
require('Biostrings')
require('ShortRead')

# create  ShortReadQ object width widths between 1 and 100
 
 set.seed(10)
input <- random_length(100, widths = 1:100)

# apply the filter, removing sequences length < 10 or length > 80
filtered <- length_filter(input, rm.min = 10, rm.max = 80)

# look at the filtered sequences
sread(filtered)
}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server_functions.R
\name{processingFunction_}
\alias{processingFunction_}
\title{processingFunction_}
\usage{
processingFunction_(my_envir)
}
\value{
Processes the input FASTQ file, without return
}
\description{
This function is the core of the application. It is used 
for the program to process the FASTQ file/s in the environment 
of the Shiny app. Note that this program makes a call to
create_cleanfunction
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxiliar.R
\name{asc2int}
\alias{asc2int}
\title{ASCII to integer}
\usage{
asc2int(x)
}
\value{
Integer
}
\description{
ASCII to integer
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching.R
\name{cutRseq}
\alias{cutRseq}
\alias{cutLseq}
\title{Remove left and right full and partial patterns}
\usage{
cutRseq(
  subject,
  Rpattern,
  with.indels = FALSE,
  fixed = "subject",
  error_rate = 0.2,
  anchored = TRUE,
  ranges = FALSE,
  checks = TRUE,
  min_match_flank = 2L,
  ...
)

cutLseq(
  subject,
  Lpattern,
  with.indels = FALSE,
  fixed = "subject",
  error_rate = 0.2,
  anchored = TRUE,
  ranges = FALSE,
  min_match_flank = 3L,
  checks = TRUE,
  ...
)
}
\arguments{
\item{subject}{\code{\link[Biostrings:DNAString-class]{DNAString}} or 
\code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object}

\item{Rpattern}{3' pattern,
\code{\link[Biostrings:DNAString-class]{DNAString}} object}

\item{with.indels}{Allow indels?}

\item{fixed}{Parameter passed to 
\code{\link[Biostrings]{trimLRPatterns}} 
Default 'subject',  ambiguities in the pattern only are interpreted 
as wildcard. See the argument fixed in 
\code{\link[Biostrings]{trimLRPatterns}}}

\item{error_rate}{Error rate (value in [0, 1]). 
The error rate is the proportion of mismatches allowed between
the adapter and the aligned portion of the subject.
For a given adapter A, the number of allowed mismatches between each 
subsequence s of A and the subject is computed as: error_rate * L_s,
where L_s is the length of the subsequence s.}

\item{anchored}{Can the adapter or partial adapter be within 
the sequence? (anchored = FALSE)
or only in the terminal regions of the sequence? (anchored = TRUE).
Default TRUE (trim only flanking regions)}

\item{ranges}{Return ranges? Default FALSE}

\item{checks}{Perform internal checks? Default TRUE}

\item{min_match_flank}{Do not trim in flanks of the subject,
if a match has min_match_flank of less length. Default 1L 
(only trim with >=2 coincidences in a flank match)}

\item{...}{additional parameters passed to
\code{\link[Biostrings]{trimLRPatterns}}}

\item{Lpattern}{5' pattern,
\code{\link[Biostrings:DNAString-class]{DNAString}} object}
}
\value{
Edited \code{\link[Biostrings:DNAString-class]{DNAString}} or 
\code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object
}
\description{
This set of programs are internal, 
and the function adapter_filter is recommended for trimming. 
The programs can remove adapters and partial 
adapters from 3' and 5'. The adapters can be anchored or not. 
When indels are allowed, the error rate consists in the edit distance. 
IUPAC simbols are allowed. The methods use the 
\code{\link[Biostrings]{trimLRPatterns}} function
 of the \pkg{Biostrings} package, with some additions 
 to take into account e.g., partial adaptors.
 IUPAC symbols are allowed in all the cases. The present function 
also removes partial adapters, without the need of additional steps
 (for example, creating a padded adapter with 'Ns', etc). 
A similar result to the output of \code{\link[Biostrings]{trimLRPatterns}} 
can be obtained with the option anchored = TRUE.
When several matches are found, the function removes the subsequence 
that starts in the first match when cutRseq is used, or ends 
in the last match when cutLseq is used.
}
\examples{
library(Biostrings)

subject <- DNAStringSet(c('ATCATGCCATCATGAT',
'CATGATATTA', 'TCATG', 'AAAAAA', 'AGGTCATG'))

Lpattern <- Rpattern <- 'TCATG'

FastqCleaner:::cutLseq(subject, Lpattern)
FastqCleaner:::cutLseq(subject, Lpattern, ranges = TRUE)
FastqCleaner:::cutRseq(subject, Rpattern)


FastqCleaner:::cutLseq(subject, Lpattern, anchored = FALSE)
FastqCleaner:::cutLseq(subject, Lpattern,  error_rate = 0.2)
FastqCleaner:::cutLseq(subject, Lpattern,  error_rate = 0.2, 
with.indels = TRUE)

}
\author{
Leandro Roser \email{learoser@gmail.com}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{create_uniform_width}
\alias{create_uniform_width}
\title{Create fastq/sequences/qualities with uniform width}
\usage{
create_uniform_width(input, type = c("fastq", "sequence", "quality"))
}
\arguments{
\item{input}{input to edit}

\item{type}{type of the input: 'fastq' (ShortReadQ),
'sequence' (DNAStringSet), 'quality' (BStringset)}
}
\value{
ShortReadQ object or character vector with sequences 
or qualities, with uniform widht (padded with Ns or \})
}
\description{
Create fastq/sequences/qualities with uniform width
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotD}
\alias{plotD}
\title{plotD}
\usage{
plotD(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
percent of reads with quality > threshold plot
}
\description{
plotD
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seq_filter.R
\name{seq_filter}
\alias{seq_filter}
\title{Remove a set of sequences}
\usage{
seq_filter(input, rm.seq)
}
\arguments{
\item{input}{\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object}

\item{rm.seq}{Ccharacter vector with sequences to remove}
}
\value{
Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
object
}
\description{
Removes a set of sequences
}
\examples{

require(ShortRead)

set.seed(10)
input <- random_length(30, 3:7)
rm.seq  = c('TGGTC', 'CGGT', 'GTTCT', 'ATA')

# verify that some sequences match
match_before <- unlist(lapply(rm.seq,
 function(x) grep(x, as.character(sread(input)))))

filtered <- seq_filter(input,rm.seq =  rm.seq)

# verify that matching sequences were removed
match_after <- unlist(lapply(rm.seq, 
function(x) grep(x, as.character(sread(filtered)))))

}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotB}
\alias{plotB}
\title{plotB}
\usage{
plotB(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Per cycle mean base quality plot
}
\description{
plotB
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/launch_fqc.R
\name{launch_fqc}
\alias{launch_fqc}
\title{Launch FastqCleaner application}
\usage{
launch_fqc(launch.browser = TRUE, ...)
}
\arguments{
\item{launch.browser}{Launch in browser? Default TRUE}

\item{...}{Additional parameters passed to \code{\link[shiny]{runApp}}}
}
\value{
Launch the application, without return value
}
\description{
Launch FastqCleaner application
}
\examples{
# Uncomment and paste in te console to launch the application:
# launch_fqc() 

NULL
}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotJ}
\alias{plotJ}
\title{plotJ}
\usage{
plotJ(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Relative kmer diversity plot
}
\description{
plotJ
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server_functions.R
\name{create_cleanfunction_}
\alias{create_cleanfunction_}
\title{create_cleanfunction_}
\usage{
create_cleanfunction_(my_envir, .which_read = c("FORWARD", "REVERSE"))
}
\value{
Function with selected cleaning operations
}
\description{
Create a function to process FASTQ files in function
of the Shiny parameters selected by the user
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxiliar.R
\name{isNaturalNumber}
\alias{isNaturalNumber}
\title{Is natural number}
\usage{
isNaturalNumber(x)
}
\value{
Logical
}
\description{
Is natural number
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotObjects}
\alias{plotObjects}
\title{plotObjects
Create the information required to construct the plots. 
This is the input of myplot, which uses the values created for 
this function to construct the plots}
\usage{
plotObjects(fq, klength, basename, maxFreq, sampleSize)
}
\value{
List with information to construct the diagnostic plots
}
\description{
plotObjects
Create the information required to construct the plots. 
This is the input of myplot, which uses the values created for 
this function to construct the plots
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxiliar.R
\name{int2asc}
\alias{int2asc}
\title{Integer to ASCII}
\usage{
int2asc(n)
}
\value{
ASCII character
}
\description{
Integer to ASCII
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotI}
\alias{plotI}
\title{plotI}
\usage{
plotI(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Read ocurrence distribution plot
}
\description{
plotI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObjects.R
\name{plotF}
\alias{plotF}
\title{plotF}
\usage{
plotF(x, nplots = 1, theFile = c("input", "output"), sampleSize)
}
\value{
Per cycle base proportion plot (lineplot)
}
\description{
plotF
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qmean_filter.R
\name{qmean_filter}
\alias{qmean_filter}
\title{Filter sequences by their average quality}
\usage{
qmean_filter(input, minq, q_format = NULL, check.encod = TRUE)
}
\arguments{
\item{input}{\code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} object}

\item{minq}{Quality threshold}

\item{q_format}{Quality format used for the file, as returned 
by check.encoding}

\item{check.encod}{Check the encoding of the sequence? This argument 
is incompatible with q_format}
}
\value{
Filtered \code{\link[ShortRead:ShortReadQ-class]{ShortReadQ}} 
object
}
\description{
The program removes the sequences with a quality 
lower the 'minq' threshold
}
\examples{

require(ShortRead)

set.seed(10)
# create 30 sequences of width 20
input <- random_seq(30, 20)

# create qualities of width 20 
## high quality (15 sequences)
set.seed(10)
my_qual <- random_qual(c(30,40), slength = 15, swidth = 20,
                       encod = 'Sanger')
## low quality (15 sequences)
set.seed(10)
my_qual_2 <-   random_qual(c(5,30), slength = 15, swidth = 20,
                           encod = 'Sanger')

# concatenate vectors
input_q<- c(my_qual, my_qual_2)

# create names
input_names <- seq_names(30)

# create ShortReadQ object
my_read <- ShortReadQ(sread = input, quality = input_q, id = input_names)

# watch the average qualities
alphabetScore(my_read) / width(my_read)

# apply the filter
filtered <- qmean_filter(my_read, minq = 30)

# watch the average qualities
alphabetScore(my_read) / width(my_read)


# watch the filtered sequences
sread(filtered)

}
\author{
Leandro Roser \email{learoser@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{random_qual}
\alias{random_qual}
\title{Create random qualities for a given encoding}
\usage{
random_qual(
  slength,
  swidth,
  qual = NULL,
  encod = c("Sanger", "Illumina1.8", "Illumina1.5", "Illumina1.3", "Solexa"),
  prob = NULL
)
}
\arguments{
\item{slength}{number of sequences}

\item{swidth}{width of the sequences}

\item{qual}{quality range for the sequences. It must be a range included 
in the selected encoding:

'Sanger' = [0, 40]

'Illumina1.8' = [0, 41]

'Illumina1.5' = [0, 40]

'Illumina1.3' = [3, 40]

'Solexa' = [-5, 40]

example: for a range from 20 to 30 in Sanger encoding,
 pass the argument = c(20, 30)}

\item{encod}{sequence encoding}

\item{prob}{a vector of range = range(qual), with probabilities to set 
the frequency of each quality value. Default is equiprobability.
If the sum of the probabilities is > 1, the values will be nomalized 
to the range [0, 1].}
}
\value{
\code{\link[Biostrings:XStringSet-class]{BStringSet}} object
}
\description{
Create a \code{\link[Biostrings:XStringSet-class]{BStringSet}} 
object
with random qualities
}
\examples{

q <- random_qual(30, 20)
q

}
\author{
Leandro Roser \email{learoser@gmail.com}
}
